// Copyright (C) 2008 Andrey Gruber (aka lamer)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0500
#endif

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>

#include "notes.h"
#include "vsplitter.h"

#define	_AG_SPLT_CLASS		L"_ag_splt_class_class_split_ag_"
#define	SPLIT_WIDTH			4

enum {OF_MIN = 0, OF_VERT = 4, OF_LEFT = 8, OF_RIGHT = 12, SH_TOP = 16};

typedef struct _SPL_CONTROL *PSPL_CONTROL;
typedef struct _SPL_CONTROL{
	HWND			hwnd;
	PSPL_CONTROL	pNext;
}SPL_CONTROL; 

static BOOL RegisterSplitter(HINSTANCE hInstance);
static LRESULT CALLBACK Splitter_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void Splitter_OnPaint(HWND hwnd);
static void Splitter_OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags);
static void Splitter_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags);
static void Splitter_OnLButtonUp(HWND hwnd, int x, int y, UINT keyFlags);
static BOOL Splitter_OnWindowPosChanging(HWND hwnd, LPWINDOWPOS lpwpos);
static void MoveLeftControl(HWND hwnd, HWND hLeft);
static void MoveRightControl(HWND hwnd, HWND hRight);

static WORD			m_Pattern[] = {0x00AA,0x0055,0x00AA,0x0055,0x00AA,0x0055,0x00AA,0x0055};
static BOOL			m_InDrag = FALSE, m_InWindow = FALSE;

HWND CreateSplitter(HWND hParent, HWND hLeft, HWND hRight, int left, int minLeft, int minRight, int upper, int lower, int topShift){
	HINSTANCE 	hInstance;
	HWND		hSplitter = NULL;

	hInstance = (HINSTANCE)GetWindowLongW(hParent, GWL_HINSTANCE);
	hSplitter = CreateWindowExW(WS_EX_TOPMOST, _AG_SPLT_CLASS, NULL, WS_CHILD | WS_VISIBLE, left, 0, SPLIT_WIDTH, 400, hParent, NULL, hInstance, NULL);
	if(hSplitter == NULL){
		if(GetLastError() == ERROR_CANNOT_FIND_WND_CLASS){
			if(!RegisterSplitter(hInstance)){
				return NULL;
			}
			hSplitter = CreateWindowExW(WS_EX_TOPMOST, _AG_SPLT_CLASS, NULL, WS_CHILD | WS_VISIBLE, left, 0, SPLIT_WIDTH, 400, hParent, NULL, hInstance, NULL);
			if(hSplitter == NULL){
				return NULL;
			}
		}
	}
	
	SetWindowLongW(hSplitter, OF_MIN, MAKELONG(minLeft, minRight));
	SetWindowLongW(hSplitter, OF_VERT, MAKELONG(upper, lower));
	SetWindowLongW(hSplitter, OF_LEFT, (LONG)hLeft);
	SetWindowLongW(hSplitter, OF_RIGHT, (LONG)hRight);
	SetWindowLongW(hSplitter, SH_TOP, (LONG)topShift);
	return hSplitter;
}

static BOOL RegisterSplitter(HINSTANCE hInstance){
	WNDCLASSEXW		wcx;

	ZeroMemory(&wcx, sizeof(wcx));
	wcx.cbSize = sizeof(wcx);
	wcx.cbWndExtra = 20;
	wcx.hCursor = LoadCursorW(hInstance, MAKEINTRESOURCEW(IDR_CUR_V_SPLIT));
	wcx.hInstance = hInstance;
	wcx.lpszClassName = _AG_SPLT_CLASS;
	wcx.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wcx.lpfnWndProc = Splitter_WndProc;
	if(RegisterClassExW(&wcx)){
		return TRUE;
	}
	else{
		if(GetLastError() == ERROR_CLASS_ALREADY_EXISTS){
			return TRUE;
		}
		return FALSE;
	}
}

static void MoveRightControl(HWND hwnd, HWND hRight){
	RECT		rc, rcParent;

	GetClientRect(GetParent(hwnd), &rcParent);
	GetWindowRect(hwnd, &rc);
	MapWindowPoints(HWND_DESKTOP, GetParent(hwnd), (LPPOINT)&rc, 2);
	SetWindowPos(hRight, 0, rc.right, rc.top, rcParent.right - rc.right, rc.bottom - rc.top, SWP_NOZORDER | SWP_SHOWWINDOW);
	RedrawWindow(hRight, NULL, NULL, RDW_INVALIDATE);
}

static void MoveLeftControl(HWND hwnd, HWND hLeft){
	RECT		rc, rcLeft;

	GetWindowRect(hwnd, &rc);
	GetWindowRect(hLeft, &rcLeft);
	rc.top += GetWindowLongW(hwnd, SH_TOP);
	MapWindowPoints(HWND_DESKTOP, GetParent(hwnd), (LPPOINT)&rc, 2);
	MapWindowPoints(HWND_DESKTOP, GetParent(hwnd), (LPPOINT)&rcLeft, 2);
	rcLeft.right = rc.left;
	SetWindowPos(hLeft, 0, 0, rc.top, rcLeft.right - rcLeft.left, rc.bottom - rc.top, SWP_NOZORDER | SWP_SHOWWINDOW);
	RedrawWindow(hLeft, NULL, NULL, RDW_INVALIDATE);
}

static LRESULT CALLBACK Splitter_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	HANDLE_MSG (hwnd, WM_PAINT, Splitter_OnPaint);
	HANDLE_MSG (hwnd, WM_LBUTTONDOWN, Splitter_OnLButtonDown);
	HANDLE_MSG (hwnd, WM_MOUSEMOVE, Splitter_OnMouseMove);
	HANDLE_MSG (hwnd, WM_LBUTTONUP, Splitter_OnLButtonUp);
	HANDLE_MSG (hwnd, WM_WINDOWPOSCHANGING, Splitter_OnWindowPosChanging);
	case WM_MOUSELEAVE:{
		m_InWindow = FALSE;
		m_InDrag = FALSE;
		RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);
		return FALSE;
	}
	case _SPM_ADD_CONTROL:{
		if(lParam == _SPC_LEFT){
			SetWindowLongW(hwnd, OF_LEFT, wParam);
			MoveLeftControl(hwnd, (HWND)wParam);
		}
		else{
			SetWindowLongW(hwnd, OF_RIGHT, wParam);
			MoveRightControl(hwnd, (HWND)wParam);
		}
		return FALSE;
	}
	case _SPM_PARENT_SIZED:{
		RECT	rc;
		GetWindowRect(hwnd, &rc);
		MapWindowPoints(HWND_DESKTOP, GetParent(hwnd), (LPPOINT)&rc, 2);

		SetWindowPos(hwnd, HWND_TOP, rc.left, wParam, SPLIT_WIDTH, lParam - wParam, SWP_SHOWWINDOW);
		
		MoveLeftControl(hwnd, (HWND)GetWindowLongW(hwnd, OF_LEFT));
		MoveRightControl(hwnd, (HWND)GetWindowLongW(hwnd, OF_RIGHT));
		return FALSE;
	}
	default: return DefWindowProc (hwnd, msg, wParam, lParam);
	}
}

static BOOL Splitter_OnWindowPosChanging(HWND hwnd, LPWINDOWPOS lpwpos)
{
	RECT	rc;
	int		minLeft, minRight, min, vert, upper;

	GetClientRect(GetParent(hwnd), &rc);
	vert = GetWindowLongW(hwnd, OF_VERT);
	upper =  LOWORD(vert);
	min = GetWindowLongW(hwnd, OF_MIN);
	minLeft = LOWORD(min);
	minRight = HIWORD(min);

	if(lpwpos->y != upper){
		lpwpos->y = upper;
	}
	if(lpwpos->x < minLeft)
		lpwpos->x = minLeft;
	if(rc.right - lpwpos->x < minRight)
		lpwpos->x = rc.right - minRight;

	return FALSE;
}

static void Splitter_OnLButtonUp(HWND hwnd, int x, int y, UINT keyFlags)
{
	m_InDrag = FALSE;
	RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);
}

static void Splitter_OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
	m_InDrag = TRUE;
	RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);
}

static void Splitter_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags)
{
	if(!m_InWindow){
		TRACKMOUSEEVENT			tm;

		m_InWindow = TRUE;
		tm.cbSize = sizeof(tm);
		tm.hwndTrack = hwnd;
		tm.dwFlags = TME_LEAVE;
		tm.dwHoverTime = 0;
		TrackMouseEvent(&tm);
	}
	if(keyFlags == MK_LBUTTON && m_InDrag){
		BOOL		fFoolDrag;
		HWND		hLeft, hRight;
		SystemParametersInfoW(SPI_GETDRAGFULLWINDOWS, 0, &fFoolDrag, FALSE);
		SystemParametersInfoW(SPI_SETDRAGFULLWINDOWS, TRUE, 0, FALSE);
		
		ReleaseCapture();
		SendMessageW(hwnd, WM_NCLBUTTONDOWN, HTCAPTION, 0);
		
		SystemParametersInfoW(SPI_SETDRAGFULLWINDOWS, fFoolDrag, 0, FALSE);
		m_InDrag = FALSE;
		RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);
		hLeft = (HWND)GetWindowLongW(hwnd, OF_LEFT);
		hRight = (HWND)GetWindowLongW(hwnd, OF_RIGHT);
		if(hLeft){
			MoveLeftControl(hwnd, hLeft);
		}
		if(hRight){
			MoveRightControl(hwnd, hRight);
		}
		SendMessageW(GetParent(hwnd), _SPM_SPLITTER_MOVED, 0, 0);
	}
}

static void Splitter_OnPaint(HWND hwnd)
{
	PAINTSTRUCT		ps;
	RECT			rc;
	HBITMAP			hBmp;
	HBRUSH			hBrush, hOldBrush;

	BeginPaint(hwnd, &ps);
	GetClientRect(hwnd, &rc);
	if(m_InDrag){
		hBmp = CreateBitmap(8, 8, 1, 1, m_Pattern);
		hBrush = CreatePatternBrush(hBmp);
		hOldBrush = SelectObject(ps.hdc, hBrush);
		PatBlt(ps.hdc, 0, 0, rc.right - rc.left, rc.bottom - rc.top, PATINVERT);
		SelectObject(ps.hdc, hOldBrush);
		DeleteObject(hBrush);
		DeleteObject(hBmp);
	}
	else{
		FillRect(ps.hdc, &rc, GetSysColorBrush(COLOR_BTNFACE));
	}
	EndPaint(hwnd, &ps);
}
