// Copyright (C) 2007 Andrey Gruber (aka lamer)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include <stdlib.h>
#include <time.h>
#include "notes.h"
#include "skin.h"
#include "skinlessdrawing.h"
#include "send.h"

typedef union _TIMEUNION {
	FILETIME 		fileTime;
	ULARGE_INTEGER 	ul;
} TIMEUNION;

typedef struct {
	HWND	hAppearance;
	HWND	hSchedule;
}NDLG_WNDS;

typedef struct {
	HWND	hOnce;
	HWND	hEveryDay;
	HWND	hRepeat;
	HWND	hWeekly;
	HWND	hAfter;
}DNS_WNDS;

/** Enums ********************************************************/
typedef enum {NDF, CC_TEXT, CC_HIGHLIGHT} cmd_colors;
typedef enum {FD_BOLD, FD_ITALIC, FD_UNDERLINE, FD_STRIKETHROUGH} font_decoration;

/** Prototypes ********************************************************/
static void DoAlarm(HWND hwnd);
static int CheckDockPositionForAlarm(HWND hwnd);
static BOOL CALLBACK PropEnumProc(HWND hwnd, LPCWSTR lpszString, HANDLE hData);
static void CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime);
static void FormatNewNoteName(wchar_t * lpID);
static void FormatNewNoteID(wchar_t * lpID);
static void SaveNote(HWND hwnd, P_NOTE_FLAGS pF);
static LRESULT CALLBACK Note_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void Note_OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT * lpDrawItem);
static void Note_OnMeasureItem(HWND hwnd, MEASUREITEMSTRUCT * lpMeasureItem);
static void Note_OnKillFocus(HWND hwnd, HWND hwndNewFocus);
static void Note_OnRButtonUp(HWND hwnd, int x, int y, UINT flags);
static void Note_OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags);
static void Note_OnRButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags);
static void Note_OnSetFocus(HWND hwnd, HWND hwndOldFocus);
static void Note_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
static void Note_OnClose(HWND hwnd);
static BOOL Note_OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct);
static void Note_OnPaint(HWND hwnd);
static void Note_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags);
static void Note_OnSize(HWND hwnd, UINT state, int cx, int cy);
static BOOL Note_OnWindowPosChanging(HWND hwnd, LPWINDOWPOS lpwpos);
static HBRUSH Note_OnCtlColorEdit(HWND hwnd, HDC hdc, HWND hwndChild, int type);
static void Note_OnDropFiles(HWND hwnd, HDROP hdrop);
static void Note_OnLButtonDblClk(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags);
static void Note_OnMouseLeave(HWND hwnd);
static void Note_OnInitMenuPopup(HWND hwnd, HMENU hMenu, UINT item, BOOL fSystemMenu);
static void OnResizeNote(HWND hwnd, LPRECT lpRc);
static LRESULT CALLBACK Edit_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void Edit_OnRButtonUp(HWND hwnd, int x, int y, UINT keyFlags);
static BOOL CALLBACK Adjust_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void Adjust_OnClose(HWND hwnd);
static BOOL Adjust_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam);
static void Adjust_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
static void AddTabs(HWND hwnd);
static BOOL IsAfterMatch(P_SCHEDULE_TYPE pS, LPSYSTEMTIME lpst);
static BOOL IsLeapYear(int year);
static void FillScheduleTypes(HWND hwnd, wchar_t * lpFile);
static void FillDaysOfWeek(wchar_t * lpLang);
static BOOL ScheduleChanged(HWND hwnd);
static void SetSizes(HWND hwnd, int cx, int cy);
static BOOL CALLBACK SaveAs_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static BOOL SaveAs_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam);
static void SaveAs_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
static void SaveAs_OnClose(HWND hwnd);
static void SetTooltip(HWND hOwner, HWND hTooltip, P_NOTE_DATA pD, P_NOTE_RTHANDLES pH, BOOL bUpdate);
static int SetRepeatCount(LPSYSTEMTIME lst);
static void ColorChanged(HWND hNote, int id);
static void SetNoteCaption(HWND hwnd);
static LRESULT CALLBACK Grip_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void Grip_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags);
static void PaintGrip(HWND hwnd, HDC hdcPS, COLORREF crWindow);
static void PaintSkinnableWindow(HWND hwnd, HDC hdcPS, P_NOTE_RTHANDLES pH);
static void PaintSimpleWindow(HWND hwnd, HDC hdcPS, P_NOTE_RTHANDLES pH);
static void MeasureSkinlessNoteRectangles(P_NOTE_RTHANDLES pH, int cx, int cy);
static void RestoreStandardView(HWND hNote);
static void ClearOnAdjustClose(HWND hNote);
static void SetREFontCommon(HWND hEdit, LPLOGFONTW lf);
static void SetREFontColorCommon(HWND hEdit, COLORREF crText);
static void SetREHighlight(HWND hNote, COLORREF crColor);
static void SetREFontFace(HWND hNote, P_FONT_TYPE pft);
static void SetREFontSize(HWND hNote, int size);
static void SetREFontColor(HWND hNote, COLORREF crColor);
static void SetREFontDecoration(HWND hNote, font_decoration fd);
static void SetREAlignment(HWND hNote, int alignment);
static void SetREBullets(HWND hNote);
static DWORD InStreamCallback(DWORD_PTR dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb);
static DWORD OutStreamCallback(DWORD_PTR dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb);
static LRESULT CALLBACK Command_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void Command_OnPaint(HWND hwnd);
static void Command_OnLButtonUp(HWND hwnd, int x, int y, UINT keyFlags);
static void Tbr_OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT * lpDrawItem);
static void Tbr_OnMeasureItem(HWND hwnd, MEASUREITEMSTRUCT * lpMeasureItem);
static void Tbr_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
static void Tbr_OnInitMenuPopup(HWND hwnd, HMENU hMenu, UINT item, BOOL fSystemMenu);
static void SetNoteCommandsBar(HWND hwnd, P_NOTE_RTHANDLES pH);
static void SetCommandsTooltips(HWND hwnd);
static void RecreateCommandTooltip(HWND hwnd);
static LRESULT CALLBACK Tbr_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void DeleteRTFile(wchar_t * id);
static void DeleteAutoSavedFile(wchar_t * id);
static int GetTemporarySaveName(HWND hNote, wchar_t * lpName, int nMax);
static void SetCurrentFontFace(HWND hEdit);
static int GetCurrentFontSize(HWND hEdit);
static void AutosaveNote(HWND hwnd, P_NOTE_FLAGS pF);
static BOOL ApplyNoteChanges(HWND hwnd);
static BOOL CALLBACK NApp_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void NApp_OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT * lpDrawItem);
static void NApp_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
static BOOL NApp_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam);
static BOOL CALLBACK NSched_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void NSched_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
static void NSched_OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT * lpDrawItem);
static void InitAppearancePart(HWND hwnd);
static void InitSchedulePart(HWND hwnd);
static void AddGroupsMenuItems(HWND hwnd, HMENU hMenu);
static void RemoveBell(HWND hwnd);
static void rseed(int seed);
static int rnd(int base);
static COLORREF MakeRandomColor(void);
static BOOL CALLBACK Dns_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void Dns_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
static BOOL Dns_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam);
static void SetDockType(int * dockData, dock_type dockType);
static void DockNote(HWND hwnd, dock_type dockType, P_NOTE_DATA pD);
static void UndockNote(HWND hwnd, P_NOTE_DATA pD);
static void WriteNotePlacement(HWND hwnd, BOOL fSave);
static void SaveVisibility(HWND hwnd, BOOL visible);
static void SaveNewBColor(HWND hwnd, COLORREF crWindow);
static void SaveNewCFont(HWND hwnd, LPLOGFONTW lplf);
static void SaveNewCColor(HWND hwnd, COLORREF crCaption);
static int ArrowRequired(HWND hwnd, dock_type dockType, int dockData);
static void ApplyNewSkin(HWND hNote, P_NOTE_RTHANDLES pH);
static void GetPesterPart(LPSYSTEMTIME lps);
static void SetPesterPart(LPSYSTEMTIME lps);
static void ShowHideNoteToolbar(HWND hNote);
static void ShiftNotes(int pos);
static SIZE GetScreenMetrics(void);
static void AddToFavorites(HWND hwnd);
static void SetWeekdayText(int id, int day, const wchar_t * lpFile);
static short SelectedWeekdays(void);
static short RealDayOfWeek(int dof);

/** Constants ********************************************************/
#define	TIMER_INTERVAL			1000

/** Module variables ********************************************************/
static BOOL				m_fFullDrag;
static NOTE_RTHANDLES	m_TNHandles;
static NOTE_APPEARANCE	m_TNAppearance;
static RECT				m_rWindow, m_rCaption;
static BOOL				m_JustCreated = TRUE;
static cmd_colors		m_CurrColorsChoice = NDF;
static BOOL				m_Reload = FALSE;
static NDLG_WNDS		m_Dlgs;
static DNS_WNDS			m_Dns;
static BOOL				m_DblClickSize = FALSE;
static HMENU			m_hTempMenu = NULL, m_hMenuTColor, m_hMenuTHighlight;
static int 				rnd_seed = 12345678;

/** Popup menus ********************************************************/
static MItem			m_NMenus[] = {{IDM_SAVE, 6, 0, -1, -1, MT_REGULARITEM, FALSE, L"Save", L""}, 
									{IDM_SAVE_AS, 10, 0, -1, -1, MT_REGULARITEM, FALSE, L"Save As", L""}, 
									{IDM_ADJUST, 7, 0, -1, -1, MT_REGULARITEM, FALSE, L"Adjust", L""}, 
									{IDM_HIDE, 16, 0, -1, -1, MT_REGULARITEM, FALSE, L"Hide", L""}, 
									{IDM_DEL, 9, 0, -1, -1, MT_REGULARITEM, FALSE, L"Delete", L""},
									{IDM_DOCK, 48, 0, -1, -1, MT_REGULARITEM, FALSE, L"Dock", L""},
									{IDM_DOCK_NONE, -1, 0, -1, -1, MT_REGULARITEM, FALSE, L"None", L""},
									{IDM_DOCK_LEFT, 44, 0, -1, -1, MT_REGULARITEM, FALSE, L"Left", L""},
									{IDM_DOCK_TOP, 45, 0, -1, -1, MT_REGULARITEM, FALSE, L"Top", L""},
									{IDM_DOCK_RIGHT, 46, 0, -1, -1, MT_REGULARITEM, FALSE, L"Right", L""},
									{IDM_DOCK_BOTTOM, 47, 0, -1, -1, MT_REGULARITEM, FALSE, L"Bottom", L""},
									{IDM_EMAIL, 25, 0, -1, -1, MT_REGULARITEM, FALSE, L"Send", L""},
									{IDM_ATTACHMENT, 70, 0, -1, -1, MT_REGULARITEM, FALSE, L"Send As Attachment", L""},
									{IDM_NOTE_GROUPS, 35, 0, -1, -1, MT_REGULARITEM, FALSE, L"Groups", L""},
									{IDM_ADD_TO_FAVORITES, 67, 0, -1, -1, MT_REGULARITEM, FALSE, L"Add To Favorites", L""},
									{IDM_ON_TOP, -1, 0, 22, 0, MT_REGULARITEM, FALSE, L"On Top"}};
static MItem			m_EMenus[] = {{IDM_UNDO, 40, 0, -1, -1, MT_REGULARITEM, FALSE, L"Undo", L""}, 
									{IDM_REDO, 41, 0, -1, -1, MT_REGULARITEM, FALSE, L"Redo", L""},
									{IDM_CUT, 36, 0, -1, -1, MT_REGULARITEM, FALSE, L"Cut", L""},
									{IDM_COPY, 37, 0, -1, -1, MT_REGULARITEM, FALSE, L"Copy", L""},
									{IDM_PASTE, 38, 0, -1, -1, MT_REGULARITEM, FALSE, L"Paste", L""},
									{IDM_FORMAT_NOTE_TEXT, -1, 0, -1, -1, MT_REGULARITEM, FALSE, L"Format", L""},								
									{IDM_FORMAT_FONT_FAMILY, 55, 0, -1, -1, MT_REGULARITEM, FALSE, L"Font", L""},
									{IDM_FORMAT_FONT_SIZE, 56, 0, -1, -1, MT_REGULARITEM, FALSE, L"Font Size", L""},
									{IDM_FORMAT_FONT_COLOR, 57, 0, -1, -1, MT_REGULARITEM, FALSE, L"Font Color", L""},
									{IDM_FORMAT_FONT_BOLD, 58, 0, -1, -1, MT_REGULARITEM, FALSE, L"Bold", L""},
									{IDM_FORMAT_FONT_ITALIC, 59, 0, -1, -1, MT_REGULARITEM, FALSE, L"Italic", L""},
									{IDM_FORMAT_FONT_UNDER, 60, 0, -1, -1, MT_REGULARITEM, FALSE, L"Underline", L""},
									{IDM_FORMAT_FONT_STRIKE, 61, 0, -1, -1, MT_REGULARITEM, FALSE, L"Strikethrough", L""},
									{IDM_FORMAT_FONT_HILIT, 62, 0, -1, -1, MT_REGULARITEM, FALSE, L"Highlight", L""},
									{IDM_FORMAT_ALIGN_LEFT, 63, 0, -1, -1, MT_REGULARITEM, FALSE, L"Align Left", L""},
									{IDM_FORMAT_ALIGN_CENTER, 64, 0, -1, -1, MT_REGULARITEM, FALSE, L"Center", L""},
									{IDM_FORMAT_ALIGN_RIGHT, 65, 0, -1, -1, MT_REGULARITEM, FALSE, L"Align Right", L""},
									{IDM_FORMAT_BULLETS, 66, 0, -1, -1, MT_REGULARITEM, FALSE, L"Bullets", L""},
									{IDM_SEARCH_NOTE, 43, 0, -1, -1, MT_REGULARITEM, FALSE, L"Find", L""},
									{IDM_SEARCH_NOTE_NEXT, 53, 0, -1, -1, MT_REGULARITEM, FALSE, L"Find Next", L""},
									{IDM_REPLACE_NOTE, 54, 0, -1, -1, MT_REGULARITEM, FALSE, L"Replace", L""},
									{IDM_SELECT_ALL, 39, 0, -1, -1, MT_REGULARITEM, FALSE, L"Select All", L""}};

BOOL RegisterNoteClass(HINSTANCE hInstance){

	WNDCLASSEXW		wcx;

	ZeroMemory(&wcx, sizeof(wcx));
	wcx.cbSize = sizeof(wcx);
	wcx.style = CS_OWNDC | CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wcx.lpfnWndProc = (WNDPROC)Note_WndProc;
	wcx.hInstance = hInstance;
	wcx.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcx.cbWndExtra = 28;
	wcx.lpszClassName = NWC_SKINNABLE_CLASS;
	//register skinnable class
	if(!RegisterClassExW(&wcx))
		return FALSE;
	//register grip class
	wcx.cbWndExtra = 0;
	wcx.lpfnWndProc = (WNDPROC)Grip_WndProc;
	wcx.lpszClassName = NWC_GRIP_CLASS;
	wcx.hCursor = LoadCursor(NULL, IDC_SIZENWSE);
	if(!RegisterClassExW(&wcx))
		return FALSE;
	//register command button class
	wcx.lpfnWndProc = (WNDPROC)Command_WndProc;
	wcx.lpszClassName = NWC_TBR_BUTTON_CLASS;
	wcx.hCursor = LoadCursor(NULL, IDC_HAND);
	if(!RegisterClassExW(&wcx))
		return FALSE;
	//register toolbar class
	wcx.lpfnWndProc = (WNDPROC)Tbr_WndProc;
	wcx.lpszClassName = NWC_TOOLBAR_CLASS;
	wcx.hCursor = LoadCursor(NULL, IDC_ARROW);
	if(!RegisterClassExW(&wcx))
		return FALSE;
	return TRUE;
}

/*-@@+@@------------------------------------------------------------------
 Procedure: CreateNoteWindow
 Created  : Thu May 10 11:40:51 2007
 Modified : Fri May 11 21:55:34 2007

 Synopsys : Creates and shows (optional) note window
 Input    : Application instance, parent window (main), pointer to 
            structure with needed handles, visibility flag, existing note 
            id
 Output   : Created window handle
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/
HWND CreateNoteWindow(HINSTANCE hInstance, HWND hwndParent, 
						P_NOTE_RTHANDLES pHandles, 
						wchar_t * lpID, 
						BOOL loadNew, wchar_t * lpFile){

	HWND				hwnd = NULL, hEdit, hStatic, hGrip, hToolbar, hSysToolbar;
	int					style = WS_POPUP;
	BITMAP				bm;
	P_NOTE_FLAGS 		pF;
	P_NOTE_RTHANDLES	pH;
	P_NOTE_APPEARANCE	pA;
	P_SCHEDULE_TYPE		pS;
	P_NOTE_DATA			pD;
	P_NCMD_BAR			pC, pDH;
	NOTE_APPEARANCE		tmpA;
	wchar_t				szID[28], szPath[MAX_PATH];
	RECT				rc, rcEdit;
	BOOL				res, useSkin = FALSE, fOld = FALSE, fSetPrivate;
	HANDLE				hFile;
	NOTE_REL_POSITION	nrp;
	int					left, top, width, height;

	SetRectEmpty(&rc);
	SetRectEmpty(&rcEdit);
	ZeroMemory(&nrp, sizeof(nrp));

	// if(fVisible)
		// style |= WS_VISIBLE;

	if(pHandles->hbSkin){
		useSkin = TRUE;
		//get skin dimensions
		GetObject(pHandles->hbSkin, sizeof(bm), &bm);
		//create note window with skin dimensions
		hwnd = CreateWindowExW(WS_EX_LAYERED | WS_EX_ACCEPTFILES, NWC_SKINNABLE_CLASS, g_NotePaths.ProgName, style, 0, 0, bm.bmWidth, bm.bmHeight, hwndParent, NULL, hInstance, NULL);
	}
	else{
		//create skinless note
		hwnd = CreateWindowExW(WS_EX_LAYERED | WS_EX_ACCEPTFILES, NWC_SKINNABLE_CLASS, g_NotePaths.ProgName, style, 0, 0, pHandles->szDef.cx, pHandles->szDef.cy, hwndParent, NULL, hInstance, NULL);
	}
	if(hwnd == NULL)
		return hwnd;
	else{
		m_JustCreated = TRUE;
		//create "toolbar buttons"
		hToolbar = CreateWindowExW(0, NWC_TOOLBAR_CLASS, NULL, WS_CHILD, 0, 0, 240, 240, hwnd, NULL, hInstance, NULL);
		SetPropW(hwnd, PH_COMMANDS, hToolbar);
		pC = calloc(CMD_TBR_BUTTONS, sizeof(NCMD_BAR));
		if(pC){
			SetWindowLongW(hwnd, OFFSET_COMMANDS, (LONG)pC);
			for(int i = CMD_FONT; i < CMD_TBR_BUTTONS; i++){
				pC->hCmd = CreateWindowExW(0, NWC_TBR_BUTTON_CLASS, NULL, WS_CHILD | WS_VISIBLE, 0, 0, 24, 24, hToolbar, (HMENU)(5000 + i), hInstance, NULL);
				pC++;
			}
		}
		//create delete/hide toolbar and buttons
		hSysToolbar = CreateWindowExW(0, NWC_TOOLBAR_CLASS, NULL, WS_CHILD | WS_VISIBLE, 0, 0, 240, 240, hwnd, NULL, hInstance, NULL);
		SetPropW(hwnd, PH_DELHIDE, hSysToolbar);
		pDH = calloc(2, sizeof(NCMD_BAR));
		if(pDH){
			SetWindowLongW(hwnd, OFFSET_DH, (LONG)pDH);
			for(int i = 0; i < CMD_SYS_BUTTONS; i++){
				pDH->hCmd = CreateWindowExW(0, NWC_TBR_BUTTON_CLASS, NULL, WS_CHILD | WS_VISIBLE, 0, 0, 24, 24, hSysToolbar, (HMENU)(IDM_HIDE + i), hInstance, NULL);
				pDH++;
			}
		}

		if(!useSkin){
			//create grip window for skinless note
			hGrip = CreateWindowExW(0, NWC_GRIP_CLASS, NULL, WS_CHILD | WS_VISIBLE, 0, 0, 0, 0, hwnd, NULL, hInstance, NULL);
			SetPropW(hwnd, PH_GRIP, hGrip);
			if(g_NoteSettings.showScrollbar)
				hEdit = CreateWindowExW(0, RICHEDIT_CLASSW, NULL, E_STYLE_WITH_SCROLL, 0, 0, 0, 0, hwnd, (HMENU)IDC_NOTE_EDIT, hInstance, NULL);
			else
				hEdit = CreateWindowExW(0, RICHEDIT_CLASSW, NULL, E_STYLE_WITHOUT_SCROLL, 0, 0, 0, 0, hwnd, (HMENU)IDC_NOTE_EDIT, hInstance, NULL);
		}
		else{
			//create static window for rich edit transparency simulation
			hStatic = CreateWindowExW(0, __T("Static"), NULL, WS_CHILD | WS_VISIBLE | SS_BITMAP | SS_REALSIZEIMAGE, 0, 0, 10, 10, hwnd, NULL, hInstance, NULL);
			SetPropW(hwnd, PH_STATIC, hStatic);
			if(g_NoteSettings.showScrollbar)
				hEdit = CreateWindowExW(WS_EX_TRANSPARENT, RICHEDIT_CLASSW, NULL, E_STYLE_WITH_SCROLL, 0, 0, 0, 0, hwnd, (HMENU)IDC_NOTE_EDIT, hInstance, NULL);
			else
				hEdit = CreateWindowExW(WS_EX_TRANSPARENT, RICHEDIT_CLASSW, NULL, E_STYLE_WITHOUT_SCROLL, 0, 0, 0, 0, hwnd, (HMENU)IDC_NOTE_EDIT, hInstance, NULL);
		}
	}
	SetPropW(hwnd, PH_EDIT, hEdit);
	RichEdit_AutoURLDetect(hEdit, TRUE);

	//store needed handles
	pH = calloc(1, sizeof(NOTE_RTHANDLES));
	SetWindowLongW(hwnd, OFFSET_HANDLES, (LONG)pH);
	memcpy(pH, pHandles, sizeof(NOTE_RTHANDLES));
	//set mask color for skinless note in order to prevent becoming selected text (black) transparent
	if(!useSkin)
		pH->crMask = CLR_MASK;
	//make room for possible individual appearance
	pA = calloc(1, sizeof(NOTE_APPEARANCE));
	SetWindowLongW(hwnd, OFFSET_APPEARANCE, (LONG)pA);
	//make room for run-time properties
	pF = calloc(1, sizeof(NOTE_FLAGS));
	SetWindowLongW(hwnd, OFFSET_FLAGS, (LONG)pF);
	pF->saved = TRUE;
	//make room for possible scheduling
	pS = calloc(1, sizeof(SCHEDULE_TYPE));
	SetWindowLongW(hwnd, OFFSET_SCHEDULE, (LONG)pS);
	//make room for note data
	pD = calloc(1, sizeof(NOTE_DATA));
	SetWindowLongW(hwnd, OFFSET_DATA, (LONG)pD);
	//create tooltip
	pH->hTooltip = CreateWindowExW(WS_EX_TOPMOST, __T("Tooltips_class32"), NULL, TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
	//copy common appearance
	memcpy(pA, &g_Appearance, sizeof(NOTE_APPEARANCE));

	if(lpID != NULL && !loadNew){				//in case of loading existing note from program DB
		pF->fromDB = TRUE;
		wcscpy(szPath, g_NotePaths.DataDir);
		wcscat(szPath, lpID);
		wcscat(szPath, NOTE_EXTENTION);
		//load RTF file contents
		hFile = CreateFileW(szPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if(hFile != INVALID_HANDLE_VALUE){
			EDITSTREAM			esm;
			esm.dwCookie = (DWORD)hFile;
			esm.dwError = 0;
			esm.pfnCallback = InStreamCallback;
			RichEdit_StreamIn(hEdit, SF_RTF | SF_UNICODE, &esm);
			CloseHandle(hFile);
		}		
		//get note data
		//for version 4.0
		if(!GetPrivateProfileStructW(lpID, S_DATA, pD, sizeof(NOTE_DATA), g_NotePaths.DataFile)){
			//for version 3.5
			GetPrivateProfileStructW(lpID, S_POSITION, &rc, sizeof(rc), g_NotePaths.DataFile);
			pD->rcp = rc;
			//set note initally visible
			pD->visible = TRUE;
			if(!GetPrivateProfileStructW(lpID, S_DATA, pD, sizeof(NOTE_DATA) - sizeof(RECT) - sizeof(BOOL) - sizeof(BOOL) - sizeof(BOOL) - 2 * sizeof(int), g_NotePaths.DataFile)){
				wcscpy(pD->szName, lpID);
				GetLocalTime(&pD->stChanged);
			}
			//save note new data
			WritePrivateProfileStructW(lpID, S_DATA, pD, sizeof(NOTE_DATA), g_NotePaths.DataFile);
		}
		else{
			rc = pD->rcp;
		}
		GetPrivateProfileStructW(lpID, IK_RELPOSITION, &nrp, sizeof(nrp), g_NotePaths.DataFile);

		//save prevOnTop
		pD->prevOnTop = pD->onTop;
		//get scheduling (if any)
		if(!GetPrivateProfileStructW(lpID, S_SCHEDULE, pS, sizeof(SCHEDULE_TYPE), g_NotePaths.DataFile)){
			//check for version previous to 4.5
			GetPrivateProfileStructW(lpID, S_SCHEDULE, pS, sizeof(SCHEDULE_TYPE) - sizeof(SYSTEMTIME), g_NotePaths.DataFile);
		}
		//reset changed flag
		pF->maskChanged = 0;
		//get individual appearance
		ZeroMemory(&tmpA, sizeof(tmpA));
		res = GetPrivateProfileStructW(lpID, S_APPEARANCE, &tmpA, sizeof(NOTE_APPEARANCE), g_NotePaths.DataFile);
		if(!res){
			//try to get structure from previous version
			res = GetPrivateProfileStructW(lpID, S_APPEARANCE, &tmpA, 
				sizeof(NOTE_APPEARANCE) - sizeof(LOGFONTW) - sizeof(COLORREF) -
				sizeof(COLORREF) - sizeof(int), g_NotePaths.DataFile);
			fOld = TRUE;
		}
		if(res){
			//check skin
			if(*tmpA.szSkin){
				fSetPrivate = FALSE;
				if((tmpA.nPrivate & F_SKIN) == F_SKIN){
					fSetPrivate = TRUE;
				}
				else{
					if(fOld){
						if(wcscmp(tmpA.szSkin, pA->szSkin) != 0){
							fSetPrivate = TRUE;
						}
					}
				}
				if(fSetPrivate){
					//save private skin
					wcscpy(pA->szSkin, tmpA.szSkin);
					//prepare needed handles
					if(useSkin){
						//change window size if it has a skin different from global settings
						GetSkinProperties(hwnd, pH, pA->szSkin, FALSE);
						//get skin dimensions
						GetObject(pH->hbSkin, sizeof(bm), &bm);
					}
					pA->nPrivate |= F_SKIN;
				}
			}

			//check note's font
			if(!tmpA.fFontSet){
				//set richtext font to common
				SetREFontCommon(hEdit, &pA->lfFont);
				SetREFontColorCommon(hEdit, pA->crFont);
			}
			else
				pA->nPrivate |= F_RTFONT;

			//check skinless note caption's font
			if(*tmpA.lfCaption.lfFaceName && (tmpA.nPrivate & F_C_FONT) == F_C_FONT){
					//save private caption font
					memcpy(&pA->lfCaption, &tmpA.lfCaption, sizeof(tmpA.lfCaption));
					pH->hFCaption = CreateFontIndirectW(&pA->lfCaption);
					if(!fOld)
						pA->nPrivate |= F_C_FONT;
			}

			//check skinless note caption's font color
			if(tmpA.crCaption && (tmpA.nPrivate & F_C_COLOR) == F_C_COLOR){
				//save private caption font color
				pA->crCaption = tmpA.crCaption;
				if(!fOld)
					pA->nPrivate |= F_C_COLOR;
			}

			//check skinless note background color
			if(tmpA.crWindow && (tmpA.nPrivate & F_B_COLOR) == F_B_COLOR){
				//save private background color
				pA->crWindow = tmpA.crWindow;
				if(!fOld)
					pA->nPrivate |= F_B_COLOR;
			}
		}
		else{
			//set richtext font to common
			SetREFontCommon(hEdit, &pA->lfFont);
			SetREFontColorCommon(hEdit, pA->crFont);
		}
		//mesure rectangles for skinless note
		if(!useSkin){
			CreateSkinlessToolbarBitmap(hwnd, pA, pH, &pH->hbCommand);
			CreateSkinlessSysBitmap(hwnd, pA, pH, &pH->hbDelHide);
			MeasureSkinlessNoteRectangles(pH, rc.right - rc.left, rc.bottom - rc.top);
		}
			
		//prepare edit box rectangle
		CopyRect(&rcEdit, &pH->rcEdit);
		
		//set note id
		wcscpy(pF->id, lpID);
		//position window
		if(IsBitOn(g_NoteSettings.reserved1, SB_RELPOSITION)){
			SIZE	sz = GetScreenMetrics();
			left = (int)(nrp.left * sz.cx);
			top = (int)(nrp.top * sz.cy);
		}
		else{
			left = rc.left;
			top = rc.top;
		}
		if(useSkin){
			width = bm.bmWidth;
			height = bm.bmHeight;
		}
		else{
			width = rc.right - rc.left;
			height = rc.bottom - rc.top;
		}
		MoveWindow(hwnd, left, top, width, height, TRUE);
		if(g_NoteSettings.rollOnDblClick){
			if(!useSkin && DockType(pD->dockData) == DOCK_NONE){
				if(pD->rolled && pD->idGroup != GROUP_RECYCLE){
					m_DblClickSize = TRUE;
					MoveWindow(hwnd, left, top, width, pH->rcCaption.bottom - pH->rcCaption.top, TRUE);
					pH->rolled = TRUE;
					m_DblClickSize = FALSE;
				}
			}
		}

		//check for OnTop
		if(pD->onTop){
			SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
		}

		if(pS->scType != 0 && pD->idGroup != GROUP_RECYCLE){
			SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
			if(pS->scType == SCH_REP)	//set repeat count according to timer interval
				pH->nRepeat = SetRepeatCount(&pS->scDate);
			SetTimer(hwnd, 1, TIMER_INTERVAL, TimerProc);
			pH->idTimer = 1;
		}
		SetTooltip(hwnd, pH->hTooltip, pD, pH, FALSE);

		if(DockType(pD->dockData) == DOCK_NONE || pD->idGroup == GROUP_RECYCLE){
			//set dock index to -1 for notes from previous version
			if(DockIndex(pD->dockData) != -777)
				SetDockIndex(&pD->dockData, -777);
		}
		else{
			AddDockItem(DHeader(DockType(pD->dockData)), hwnd, DockIndex(pD->dockData));
		}
	}
	else if(lpID != NULL && loadNew){		//load note which is not from program DB
		//load RTF file contents
		wcscpy(szPath, g_NotePaths.DataDir);
		wcscat(szPath, lpID);
		wcscat(szPath, NOTE_EXTENTION);
		CopyFileW(lpFile, szPath, FALSE);
		hFile = CreateFileW(szPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if(hFile != INVALID_HANDLE_VALUE){
			EDITSTREAM			esm;
			esm.dwCookie = (DWORD)hFile;
			esm.dwError = 0;
			esm.pfnCallback = InStreamCallback;
			RichEdit_StreamIn(hEdit, SF_RTF | SF_UNICODE, &esm);
			CloseHandle(hFile);
		}
		//store note's id
		wcscpy(pF->id, lpID);
		//store temporary note's name
		wcscpy(pD->szName, lpID);
		//set dock index to -1
		SetDockIndex(&pD->dockData, -777);
		//prepare edit box rectangle
		CopyRect(&rcEdit, &pH->rcEdit);
		//check for random color setting
		if(IsBitOn(g_NoteSettings.reserved1, SB_RANDCOLOR)){		
			pA->crWindow = MakeRandomColor();
			pA->nPrivate |= F_B_COLOR;
			pF->maskChanged |= F_B_COLOR;
		}
		//measure skinless note rectangles
		if(!useSkin){
			CreateSkinlessToolbarBitmap(hwnd, pA, pH, &pH->hbCommand);
			CreateSkinlessSysBitmap(hwnd, pA, pH, &pH->hbDelHide);
			MeasureSkinlessNoteRectangles(pH, pH->szDef.cx, pH->szDef.cy);
		}
		SetTooltip(hwnd, pH->hTooltip, pD, pH, FALSE);
		SaveNote(hwnd, pF);
	}
	else{			//new note
		//set dock index to -1
		SetDockIndex(&pD->dockData, -777);
		FormatNewNoteID(szID);
		//set note id
		wcscpy(pF->id, szID);
		FormatNewNoteName(szID);
		wcscpy(pD->szName, szID);
		//prepare edit box rectangle
		CopyRect(&rcEdit, &pH->rcEdit);
		//check for random color setting
		if(IsBitOn(g_NoteSettings.reserved1, SB_RANDCOLOR)){		
			pA->crWindow = MakeRandomColor();
			pA->nPrivate |= F_B_COLOR;
			pF->maskChanged |= F_B_COLOR;
		}
		//measure skinless note rectangles
		if(!useSkin){
			CreateSkinlessToolbarBitmap(hwnd, pA, pH, &pH->hbCommand);
			CreateSkinlessSysBitmap(hwnd, pA, pH, &pH->hbDelHide);
			MeasureSkinlessNoteRectangles(pH, pH->szDef.cx, pH->szDef.cy);
		}
		SetTooltip(hwnd, pH->hTooltip, pD, pH, FALSE);
		//set richtext font to common
		SetREFontCommon(hEdit, &pA->lfFont);
		SetREFontColorCommon(hEdit, pA->crFont);
		if(g_NoteSettings.newOnTop){
			//new note is always on-top
			pD->onTop = TRUE;
			SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
		}
	}

	if(!useSkin){
		//set richtext backcolor
		RichEdit_SetBkgndColor(hEdit, 0, pA->crWindow);
		//position and size edit box
		MoveWindow(hEdit, 1, pH->rcCaption.bottom + 1, (pH->rcCaption.right - pH->rcCaption.left - 2), pH->rcGrip.top - (pH->rcCaption.bottom - pH->rcCaption.top) - 1, TRUE);
		//position and size grip box
		MoveWindow(hGrip, pH->rcGrip.left, pH->rcGrip.top, pH->rcGrip.right - pH->rcGrip.left, pH->rcGrip.bottom - pH->rcGrip.top, TRUE);
	}
	else{
		//position and size static window
		MoveWindow(hStatic, rcEdit.left, rcEdit.top, rcEdit.right - rcEdit.left, rcEdit.bottom - rcEdit.top, TRUE);
		SendMessageW(hStatic, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)pH->hbBack);
		//position and size edit box
		MoveWindow(hEdit, rcEdit.left, rcEdit.top, rcEdit.right - rcEdit.left, rcEdit.bottom - rcEdit.top, TRUE);
	}

	if(useSkin){
		SetLayeredWindowAttributes(hwnd, pH->crMask, 255, LWA_COLORKEY);
	}
	else{
		SetLayeredWindowAttributes(hwnd, CLR_MASK, 255, LWA_COLORKEY);
		SetRect(&pH->rcSize, left, top, left + width, top + height);
	}

	SetNoteCommandsBar(hwnd, pH);

	//initialize rich edit properties
	RichEdit_EmptyUndoBuffer(hEdit);
	RichEdit_SetEventMask(hEdit, ENM_CHANGE | ENM_DROPFILES | ENM_LINK | ENM_KEYEVENTS | ENM_MOUSEEVENTS);
	RichEdit_SetModify(hEdit, FALSE);
	SetWindowLongW(hEdit, GWL_USERDATA, (LONG)SetWindowLongW(hEdit, GWL_WNDPROC, (LONG)Edit_WndProc));

	SetCommandsTooltips(hwnd);

	// if(fVisible){
		if(pD->idGroup > GROUP_RECYCLE){
			if(lpID != NULL && !loadNew){
				if(pD->visible){
					ShowWindow(hwnd, SW_SHOW);
					UpdateWindow(hwnd);
				}
			}
			else{
				//new note is always visible
				pD->visible = TRUE;
				ShowWindow(hwnd, SW_SHOW);
				UpdateWindow(hwnd);
			}
		}
		else{
			pD->visible = FALSE;
		}
	// }

	SetNoteCaption(hwnd);
	if(DockType(pD->dockData) == DOCK_NONE){ //not docked and from DB
		if(lpID != NULL)
			WriteNotePlacement(hwnd, TRUE);
		else
			WriteNotePlacement(hwnd, FALSE);
	}
	m_JustCreated = FALSE;
	SetForegroundWindow(hwnd);
	SetFocus(hEdit);
	ShowHideNoteToolbar(hwnd);
	return hwnd;
}

void AddColorsMenu(void){
	MENUITEMINFOW	mi;
	HMENU			hFormat, hSize;
	wchar_t 		szAutomatic[256], szNoFill[256], szFile[MAX_PATH], szKey[12];

	wcscpy(szFile, g_NotePaths.LangDir);
	wcscat(szFile, g_NotePaths.LangFile);
	_itow(ID_FC_CAPTION, szKey, 10);
	GetPrivateProfileStringW(S_COMMANDS, szKey, L"Automatic", szAutomatic, 256, szFile);
	_itow(ID_FH_CAPTION, szKey, 10);
	GetPrivateProfileStringW(S_COMMANDS, szKey, L"No Fill", szNoFill, 256, szFile);
	ZeroMemory(&mi, sizeof(mi));
	mi.cbSize = sizeof(mi);
	mi.fMask = MIIM_SUBMENU | MIIM_ID;
	GetMenuItemInfoW(g_hEditPopUp, GetMenuPosition(g_hEditPopUp, IDM_FORMAT_NOTE_TEXT), TRUE, &mi);
	hFormat = mi.hSubMenu;
	GetMenuItemInfoW(hFormat, GetMenuPosition(hFormat, IDM_FORMAT_FONT_COLOR), TRUE, &mi);
	m_hMenuTColor = mi.hSubMenu;
	CreateColorMenu(m_hMenuTColor, &g_FormatColorID, IDI_FORMAT_COLOR_START, szAutomatic);
	GetMenuItemInfoW(hFormat, GetMenuPosition(hFormat, IDM_FORMAT_FONT_HILIT), TRUE, &mi);
	m_hMenuTHighlight = mi.hSubMenu;
	CreateColorMenu(m_hMenuTHighlight, &g_FormatHighlightID, IDI_FORMAT_HIGHLIGHT_START, szNoFill);
	GetMenuItemInfoW(hFormat, GetMenuPosition(hFormat, IDM_FORMAT_FONT_SIZE), TRUE, &mi);
	hSize = mi.hSubMenu;
	CreateSizeMenu(&hSize, FALSE);

	// if(g_PMEdit){
		// free(g_PMEdit);
	// }
	// g_PMECount = 0;
	// g_PMEdit = (PMItem)calloc(NELEMS(m_EMenus), NELEMS(m_EMenus) * sizeof(MItem));
	// memcpy(g_PMEdit, m_EMenus, NELEMS(m_EMenus) * sizeof(MItem));
	// count = GetMenuItemCount(hFColors);
	// for(int k = 0; k < 2; k++){
		// for(int i = 0; i< count; i++){
			// //add new PMItem
			// g_PMECount++;
			// number = NELEMS(m_EMenus);
			// number += g_PMECount;
			// size = number * sizeof(MItem);
			// g_PMEdit = realloc(g_PMEdit, size);
			// if(g_PMEdit){
				// PMItem	pmi = g_PMEdit;
				// for(int j = 0; j < number - 1; j++)
					// pmi++;
				// if(k == 0)
					// pmi->id = GetMenuItemID(hFColors, i);
				// else
					// pmi->id = GetMenuItemID(hHighlight, i);
				// pmi->xPos = -1;
				// pmi->yPos = -1;
				// pmi->xCheck = 0;
				// pmi->yCheck = 0;
				// if(i > 0){
					// pmi->type = MT_COLORITEM;
					// wcscpy(pmi->szText, L"");
				// }
				// else{
					// pmi->type = MT_REGULARITEM;
					// if(k == 0)
						// wcscpy(pmi->szText, szAutomatic);
					// else
						// wcscpy(pmi->szText, szNoFill);
				// }
			// }
		// }
	// }
	// count = GetMenuItemCount(hSize);
	// for(int i = 0; i< count; i++){
		// //add new PMItem
		// g_PMECount++;
		// number = NELEMS(m_EMenus);
		// number += g_PMECount;
		// size = number * sizeof(MItem);
		// g_PMEdit = realloc(g_PMEdit, size);
		// if(g_PMEdit){
			// PMItem	pmi = g_PMEdit;
			// for(int j = 0; j < number - 1; j++)
				// pmi++;
			// pmi->id = GetMenuItemID(hSize, i);
			// pmi->xPos = -1;
			// pmi->yPos = -1;
			// pmi->xCheck = 0;
			// pmi->yCheck = 0;
			// pmi->type = MT_REGULARITEM;
			// _itow(pmi->id - IDI_SIZE_START, szKey, 10);
			// wcscpy(pmi->szText, szKey);
		// }
	// }
	// SetMenuOwnerDraw(hFColors);
	// SetMenuOwnerDraw(hHighlight);
}

void SetScheduleForPrevious(HWND hwnd, wchar_t * lpID, wchar_t * lpFile){
	P_SCHEDULE_TYPE		pS;
	P_NOTE_RTHANDLES	pH;

	//get scheduling (if any)
	pS = (P_SCHEDULE_TYPE)GetWindowLongW(hwnd, OFFSET_SCHEDULE);
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	if(!GetPrivateProfileStructW(lpID, S_SCHEDULE, pS, sizeof(SCHEDULE_TYPE), lpFile)){
		//check for version previous to 4.5
		GetPrivateProfileStructW(lpID, S_SCHEDULE, pS, sizeof(SCHEDULE_TYPE) - sizeof(SYSTEMTIME), lpFile);
	}
	if(pS->scType != 0){
		SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
		if(pS->scType == SCH_REP)	//set repeat count according to timer interval
			pH->nRepeat = SetRepeatCount(&pS->scDate);
		SetTimer(hwnd, 1, TIMER_INTERVAL, TimerProc);
		pH->idTimer = 1;
	}
}

static void SetREFontSize(HWND hNote, int size){
	CHARFORMAT2W		chf;
	P_NOTE_FLAGS 		pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
	P_NOTE_APPEARANCE	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hNote, OFFSET_APPEARANCE);
	HDC					hdc;
	HWND				hEdit;

	hEdit = (HWND)GetPropW(hNote, PH_EDIT);
	ZeroMemory(&chf, sizeof(chf));
	chf.cbSize = sizeof(chf);
	chf.dwMask = CFM_SIZE;
	hdc = GetDC(hEdit);
	size = -MulDiv(size, GetDeviceCaps(hdc, LOGPIXELSY), 72);
	chf.yHeight = 20 * -(size * 72) / GetDeviceCaps(hdc, LOGPIXELSY);
	ReleaseDC(hEdit, hdc);
	RichEdit_SetCharFormat(hEdit, SCF_SELECTION, &chf);
	pF->maskChanged |= F_RTFONT;
	pA->fFontSet = TRUE;
}

static void SetREFontFace(HWND hNote, P_FONT_TYPE pft){
	CHARFORMAT2W		chf;
	P_NOTE_FLAGS 		pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
	P_NOTE_APPEARANCE	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hNote, OFFSET_APPEARANCE);

	if(wcslen(pft->szFace)){
		ZeroMemory(&chf, sizeof(chf));
		chf.cbSize = sizeof(chf);
		chf.dwMask = CFM_FACE | CFM_CHARSET;
		wcscpy(chf.szFaceName, pft->szFace);
		chf.bCharSet = (BYTE)g_SelectedFont.nCharSet;
		RichEdit_SetCharFormat((HWND)GetPropW(hNote, PH_EDIT), SCF_SELECTION, &chf);
		pF->maskChanged |= F_RTFONT;
		pA->fFontSet = TRUE;
	}
}

static void SetREHighlight(HWND hNote, COLORREF crColor){
	CHARFORMAT2W		chf;
	P_NOTE_FLAGS 		pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
	P_NOTE_APPEARANCE	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hNote, OFFSET_APPEARANCE);
	P_NOTE_RTHANDLES	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hNote, OFFSET_HANDLES);
	HWND				hEdit;

	//prevent transparency
	if(crColor == pH->crMask)
		crColor--;
	hEdit = (HWND)GetPropW(hNote, PH_EDIT);
	ZeroMemory(&chf, sizeof(chf));
	chf.cbSize = sizeof(chf);
	if(crColor == -1){
		chf.dwMask = CFE_AUTOBACKCOLOR;
		chf.dwEffects = CFE_AUTOBACKCOLOR;
	}
	else{
		chf.dwMask = CFM_BACKCOLOR;
		chf.crBackColor = crColor;
	}
	RichEdit_SetCharFormat(hEdit, SCF_SELECTION, &chf);
	pF->maskChanged |= F_RTFONT;
	pA->fFontSet = TRUE;
}

static void SetREFontColor(HWND hNote, COLORREF crColor){
	CHARFORMAT2W		chf;
	P_NOTE_FLAGS 		pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
	P_NOTE_APPEARANCE	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hNote, OFFSET_APPEARANCE);
	P_NOTE_RTHANDLES	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hNote, OFFSET_HANDLES);
	HWND				hEdit;
	COLORREF			crDef;


	//prevent transparency
	if(crColor == pH->crMask)
		crColor--;
	hEdit = (HWND)GetPropW(hNote, PH_EDIT);
	ZeroMemory(&chf, sizeof(chf));
	chf.cbSize = sizeof(chf);
	chf.dwMask = CFM_COLOR;
	RichEdit_GetCharFormat(hEdit, SCF_DEFAULT, &chf);
	crDef = chf.crTextColor;
	ZeroMemory(&chf, sizeof(chf));
	chf.cbSize = sizeof(chf);
	chf.dwMask = CFM_COLOR;
	if(crColor == -1){
		if((pA->nPrivate & F_RTFONT) == F_RTFONT)
			chf.crTextColor = crDef;
		else
			chf.crTextColor = g_Appearance.crFont;
	}
	else{
		chf.crTextColor = crColor;
	}
	RichEdit_SetCharFormat(hEdit, SCF_SELECTION, &chf);
	pF->maskChanged |= F_RTFONT;
	pA->fFontSet = TRUE;
}

static void SetREFontDecoration(HWND hNote, font_decoration fd){
	CHARFORMAT2W		chf;
	P_NOTE_FLAGS 		pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
	P_NOTE_APPEARANCE	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hNote, OFFSET_APPEARANCE);
	HWND				hEdit;
	int					state;

	hEdit = (HWND)GetPropW(hNote, PH_EDIT);
	ZeroMemory(&chf, sizeof(chf));
	chf.cbSize = sizeof(chf);
	switch(fd){
	case FD_BOLD:
		chf.dwMask = CFM_BOLD;
		state = CFM_BOLD;
		break;
	case FD_ITALIC:
		chf.dwMask = CFM_ITALIC;
		state = CFM_ITALIC;
		break;
	case FD_UNDERLINE:
		chf.dwMask = CFM_UNDERLINE;
		state = CFM_UNDERLINE;
		break;
	case FD_STRIKETHROUGH:
		chf.dwMask = CFM_STRIKEOUT;
		state = CFM_STRIKEOUT;
		break;
	}
	RichEdit_GetCharFormat(hEdit, SCF_SELECTION, &chf);
	if((chf.dwEffects & state) == state)
		chf.dwEffects &= ~state;
	else
		chf.dwEffects |= state;
	
	RichEdit_SetCharFormat(hEdit, SCF_SELECTION, &chf);
	pF->maskChanged |= F_RTFONT;
	pA->fFontSet = TRUE;
}

static void SetREAlignment(HWND hNote, int alignment){
	PARAFORMAT2			pfm;
	P_NOTE_FLAGS 		pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
	P_NOTE_APPEARANCE	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hNote, OFFSET_APPEARANCE);
	HWND				hEdit;

	hEdit = (HWND)GetPropW(hNote, PH_EDIT);
	ZeroMemory(&pfm, sizeof(pfm));
	pfm.cbSize = sizeof(pfm);
	pfm.dwMask = PFM_ALIGNMENT;
	pfm.wAlignment = alignment;
	RichEdit_SetParaFormat(hEdit, &pfm);
	pF->maskChanged |= F_RTFONT;
	pA->fFontSet = TRUE;
}

static void SetREBullets(HWND hNote){
	PARAFORMAT2			pfm;
	P_NOTE_FLAGS 		pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
	P_NOTE_APPEARANCE	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hNote, OFFSET_APPEARANCE);
	HWND				hEdit;
	BOOL				isSet;

	hEdit = (HWND)GetPropW(hNote, PH_EDIT);
	ZeroMemory(&pfm, sizeof(pfm));
	pfm.cbSize = sizeof(pfm);
	pfm.dwMask = PFM_NUMBERING;
	RichEdit_GetParaFormat(hEdit, &pfm);
	if((pfm.wNumbering & PFN_BULLET) == PFN_BULLET)
		isSet = TRUE;
	else
		isSet = FALSE;
	ZeroMemory(&pfm, sizeof(pfm));
	pfm.cbSize = sizeof(pfm);
	pfm.dwMask = PFM_NUMBERING;
	if(!isSet)
		pfm.wNumbering = PFN_BULLET;
	RichEdit_SetParaFormat(hEdit, &pfm);
	pF->maskChanged |= F_RTFONT;
	pA->fFontSet = TRUE;
}

static void SetREFontColorCommon(HWND hEdit, COLORREF crText){
	CHARFORMAT2W		chf;
	
	ZeroMemory(&chf, sizeof(chf));
	chf.cbSize = sizeof(chf);
	chf.dwMask = CFM_COLOR;
	chf.crTextColor = crText;
	RichEdit_SetCharFormat(hEdit, SCF_ALL, &chf);
}

static void SetREFontCommon(HWND hEdit, LPLOGFONTW lf){
	CHARFORMAT2W		chf;
	HDC					hdc;

	ZeroMemory(&chf, sizeof(chf));
	chf.cbSize = sizeof(chf);
	chf.dwMask = CFM_BOLD | CFM_FACE | CFM_ITALIC | CFM_STRIKEOUT | CFM_UNDERLINE | CFM_CHARSET | CFM_SIZE;
	if(lf->lfWeight >= FW_BOLD)
		chf.dwEffects |= CFE_BOLD;
	if(lf->lfItalic)
		chf.dwEffects |= CFE_ITALIC;
	if(lf->lfStrikeOut)
		chf.dwEffects |= CFE_STRIKEOUT;
	if(lf->lfUnderline)
		chf.dwEffects |= CFE_UNDERLINE;
	hdc = GetDC(hEdit);
	chf.yHeight = 20 * -(lf->lfHeight * 72) / GetDeviceCaps(hdc, LOGPIXELSY);
	ReleaseDC(hEdit, hdc);
	chf.bCharSet = lf->lfCharSet;
	chf.bPitchAndFamily = lf->lfPitchAndFamily;
	wcscpy(chf.szFaceName, lf->lfFaceName);
	RichEdit_SetCharFormat(hEdit, SCF_ALL, &chf);
}

void SetNoteMIImageIndex(int id, int index){
	//set menu item index
	SetMIImageIndex(m_NMenus, NELEMS(m_NMenus), id, index);
}

void SetNoteMIText(int id, wchar_t * lpText){
	//set menu item text
	SetMIText(m_NMenus, NELEMS(m_NMenus), id, lpText);
}

void SetEditMIText(int id, wchar_t * lpText){
	//set menu item text
	SetMIText(m_EMenus, NELEMS(m_EMenus), id, lpText);
}

static int SetRepeatCount(LPSYSTEMTIME lst){
	return ((lst->wHour * 3600 + lst->wMinute * 60 + lst->wSecond) * (1000 / TIMER_INTERVAL));
}

static void ColorChanged(HWND hNote, int id){
	PMItem			pmi;
	COLORREF		color;
	MENUITEMINFOW	mi;

	ZeroMemory(&mi, sizeof(mi));
	mi.cbSize = sizeof(mi);
	mi.fMask = MIIM_DATA;
	if(m_CurrColorsChoice == CC_TEXT)
		GetMenuItemInfoW(m_hMenuTColor, id, FALSE, &mi);
	else
		GetMenuItemInfoW(m_hMenuTHighlight, id, FALSE, &mi);

	if(id == IDI_FORMAT_COLOR_START || id == IDI_FORMAT_HIGHLIGHT_START){
		if(m_CurrColorsChoice == CC_TEXT)
			SetREFontColor(hNote, -1);
		else if(m_CurrColorsChoice == CC_HIGHLIGHT)
			SetREHighlight(hNote, -1);
	}
	else{
		pmi = (PMItem)mi.dwItemData;
		if(pmi){
			color = _wtoi(pmi->szText);
			if(m_CurrColorsChoice == CC_TEXT)
				SetREFontColor(hNote, color);
			else if(m_CurrColorsChoice == CC_HIGHLIGHT)
				SetREHighlight(hNote, color);
		}
	}
	m_CurrColorsChoice = NDF;
	SetFocus(hNote);
}

/*-@@+@@------------------------------------------------------------------
 Procedure: SetTooltip
 Created  : Sun May 27 15:19:09 2007
 Modified : Sun May 27 15:24:02 2007

 Synopsys : Creates or updates note tooltip
 Input    : hOwner - note window handle
            hTooltip - tooltip handle
            pD - pointer to NOTE_DATA
            pH - pointer to NOTE_RTHANDLES
            bUpdate - whether tooltip should be created or updated
 Output   : Nothing
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/
static void SetTooltip(HWND hOwner, HWND hTooltip, P_NOTE_DATA pD, P_NOTE_RTHANDLES pH, BOOL bUpdate){
	
	wchar_t			szTooltip[256], szTemp[64];
	TOOLINFOW		ti;

	ZeroMemory(&ti, sizeof(ti));
	ti.cbSize = sizeof(ti);
	ti.hwnd = hOwner;
	ti.uFlags = TTF_SUBCLASS;
	wcscpy(szTooltip, pD->szName);
	wcscat(szTooltip, __T(" - "));
	if(pD->stChanged.wDay != 0){
		//add last saved time if note has been saved
		ConstructDateTimeString(&pD->stChanged, szTemp, FALSE);
		wcscat(szTooltip, szTemp);
	}
	else{
		//note has not been saved yet
		wcscat(szTooltip, __T("Not saved"));
	}
	//set tooltip text
	ti.lpszText = szTooltip;	
	//set tooltip rectangle
	CopyRect(&ti.rect, &pH->rcTooltip);
	if(bUpdate){
		//update tooltip
		SendMessageW(hTooltip, TTM_NEWTOOLRECTW, 0, (LPARAM)&ti);
		SendMessageW(hTooltip, TTM_UPDATETIPTEXTW, 0, (LPARAM)&ti);
	}
	else{
		//add tooltip
		SendMessageW(hTooltip, TTM_ADDTOOLW, 0, (LPARAM)&ti);
	}
}

static void CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime){

	SYSTEMTIME			st;
	P_SCHEDULE_TYPE		pS;
	P_NOTE_RTHANDLES	pH;

	GetLocalTime(&st);
	pS = (P_SCHEDULE_TYPE)GetWindowLongW(hwnd, OFFSET_SCHEDULE);
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);

	if(pS->scType == SCH_ED){
		if((st.wHour < pS->scDate.wHour)
			|| (st.wHour == pS->scDate.wHour && st.wMinute < pS->scDate.wMinute)){
			//continue checking
			SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
			return;
		}
		if(pS->scDate.wHour == st.wHour && pS->scDate.wMinute == st.wMinute){
			if((BOOL)GetPropW(hwnd, PH_ALARM)){
				//stop checking and alarm
				SetPropW(hwnd, PH_ALARM, (HANDLE)FALSE);
				DoAlarm(hwnd);
			}
		}
	}
	else if(pS->scType == SCH_ONCE){
		if((st.wYear < pS->scDate.wYear) || (st.wYear == pS->scDate.wYear && st.wMonth < pS->scDate.wMonth)
			|| (st.wYear == pS->scDate.wYear && st.wMonth == pS->scDate.wMonth && st.wDay < pS->scDate.wDay)
			|| (st.wYear == pS->scDate.wYear && st.wMonth == pS->scDate.wMonth && st.wDay == pS->scDate.wDay && st.wHour < pS->scDate.wHour)
			|| (st.wYear == pS->scDate.wYear && st.wMonth == pS->scDate.wMonth && st.wDay == pS->scDate.wDay && st.wHour == pS->scDate.wHour && st.wMinute < pS->scDate.wMinute)){
			//continue checking
			SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
			return;
		}
		if(pS->scDate.wYear == st.wYear && pS->scDate.wMonth == st.wMonth 
			&& pS->scDate.wDay == st.wDay && pS->scDate.wHour == st.wHour 
			&& pS->scDate.wMinute == st.wMinute){

			if((BOOL)GetPropW(hwnd, PH_ALARM)){
				//stop checking and alarm
				SetPropW(hwnd, PH_ALARM, (HANDLE)FALSE);
				DoAlarm(hwnd);
			}
		}
	}
	else if(pS->scType == SCH_REP){
		if(pH->nRepeat != 0){
			pH->nRepeat--;
			if(pH->nRepeat == 0){
				DoAlarm(hwnd);
				pH->nRepeat = SetRepeatCount(&pS->scDate);
			}
		}
	}
	else if(pS->scType == SCH_PER_WD){
		int dof = RealDayOfWeek(st.wDayOfWeek);
		if(!IsBitOn(pS->scDate.wDayOfWeek, dof + 1)){
		// if(pS->scDate.wDayOfWeek != st.wDayOfWeek){
			//continue checking
			SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
			return;
		}
		else{
			if(pS->scDate.wHour == st.wHour && pS->scDate.wMinute == st.wMinute){
				if((BOOL)GetPropW(hwnd, PH_ALARM)){
					//stop checking and alarm
					SetPropW(hwnd, PH_ALARM, (HANDLE)FALSE);
					DoAlarm(hwnd);
				}
			}
			else{
				//continue checking
				SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
				return;
			}
		}
	}
	else if(pS->scType == SCH_AFTER){
		if((BOOL)GetPropW(hwnd, PH_ALARM)){
			if(IsAfterMatch(pS, &st)){
				//stop checking and alarm
				SetPropW(hwnd, PH_ALARM, (HANDLE)FALSE);
				DoAlarm(hwnd);
			}
			else{
				//continue checking
				SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
				return;
			}
		}
	}
}

static short RealDayOfWeek(int dof){
	wchar_t			szBuffer[128];

	GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_IFIRSTDAYOFWEEK, szBuffer, 128);
	switch(_wtoi(szBuffer)){
		case 0:		//Monday
			switch(dof){
				case 0:
					return 6;
				default:
					return dof - 1;
			}
		case 1:		//Tuesday
			switch(dof){
				case 0:
					return 5;
				case 1:
					return 6;
				default:
					return dof - 1;
			}
		case 2:		//Wednesday
			switch(dof){
				case 0:
					return 4;
				case 1:
					return 5;
				case 2:
					return 6;
				default:
					return dof - 1;
			}
		case 3:		//Thursday
			switch(dof){
				case 0:
					return 3;
				case 1:
					return 4;
				case 2:
					return 5;
				case 3:
					return 6;
				default:
					return dof - 1;
			}
		case 4:		//Friday
			switch(dof){
				case 5:
					return 0;
				case 6:
					return 1;
				default:
					return dof + 2;
			}
		case 5:		//Saturday
			switch(dof){
				case 6:
					return 0;
				default:
					return dof + 1;
			}
		default:		//6 = Sunday
			return dof;
	}
}

static BOOL IsLeapYear(int year){
	if(((year % 4 == 0 && year % 100 != 0) || (year % 400 == 0)))
		return TRUE;
	return FALSE;
}

static BOOL IsAfterMatch(P_SCHEDULE_TYPE pS, LPSYSTEMTIME lpst){
	TIMEUNION 				ft1;
	TIMEUNION 				ft2;
	unsigned long long		res1, res2;

	//return false if starting date is in the future
	if(lpst->wYear < pS->scStart.wYear)
		return FALSE;
	if(lpst->wYear == pS->scStart.wYear && lpst->wMonth < pS->scStart.wMonth)
		return FALSE;
	if(lpst->wYear == pS->scStart.wYear && lpst->wMonth == pS->scStart.wMonth && lpst->wDay < pS->scStart.wDay)
		return FALSE;
	if(lpst->wYear == pS->scStart.wYear && lpst->wMonth == pS->scStart.wMonth && lpst->wDay == pS->scStart.wDay && lpst->wHour < pS->scStart.wHour)
		return FALSE;
	if(lpst->wYear == pS->scStart.wYear && lpst->wMonth == pS->scStart.wMonth && lpst->wDay == pS->scStart.wDay && lpst->wHour == pS->scStart.wHour && lpst->wMinute < pS->scStart.wMinute)
		return FALSE;
	if(lpst->wYear == pS->scStart.wYear && lpst->wMonth == pS->scStart.wMonth && lpst->wDay == pS->scStart.wDay && lpst->wHour == pS->scStart.wHour && lpst->wMinute == pS->scStart.wMinute && lpst->wSecond < pS->scStart.wSecond)
		return FALSE;
	SystemTimeToFileTime(lpst, &ft1.fileTime);
	SystemTimeToFileTime(&pS->scStart, &ft2.fileTime);
	res1 = (ft1.ul.QuadPart - ft2.ul.QuadPart) / 10000000;
	res2 = pS->scDate.wSecond;
	res2 += pS->scDate.wMinute * 60;
	res2 += pS->scDate.wHour * 60 * 60;
	res2 += pS->scDate.wDay * 60 * 60 * 24;
	res2 += pS->scDate.wDayOfWeek * 60 * 60 * 24 * 7;
	for(int i = 0, j = pS->scStart.wMonth, y = pS->scStart.wYear; i < pS->scDate.wMonth; i++){
		switch(j){
		case 1:
		case 3:
		case 5:
		case 7:
		case 8:
		case 10:
		case 12:
			res2 += 31 * 60 * 60 * 24;
			break;
		case 4:
		case 6:
		case 9:
		case 11:
			res2 += 30 * 60 * 60 * 24;
			break;
		case 2:
			if(IsLeapYear(y))
				res2 += 29 * 60 * 60 * 24;
			else
				res2 += 28 * 60 * 60 * 24;
			break;
		}
		if(j < 12)
			j++;
		else{
			j = 1;
			y++;
		}
	}
	for(int i = 0, j = pS->scStart.wYear; i < pS->scDate.wYear; i++, j++){
		if(IsLeapYear(j) || IsLeapYear(j + 1))
			res2 += 366 * 60 * 60 * 24;
		else
			res2 += 365 * 60 * 60 * 24;
	}
	if(res1 == res2)
		return TRUE;
	return FALSE;
}

static void ShiftNotes(int pos){
	PDOCKITEM		pItem;
	PDOCKHEADER 	pHeader;
	int				offset, dockType;

	switch(pos){
	case DA_LEFT_UP:
		pHeader = DHeader(DOCK_LEFT);
		dockType = DOCK_LEFT;
		offset = 1;
		break;
	case DA_LEFT_DOWN:
		pHeader = DHeader(DOCK_LEFT);
		dockType = DOCK_LEFT;
		offset = -1;
		break;
	case DA_TOP_LEFT:
		pHeader = DHeader(DOCK_TOP);
		dockType = DOCK_TOP;
		offset = 1;
		break;
	case DA_TOP_RIGHT:
		pHeader = DHeader(DOCK_TOP);
		dockType = DOCK_TOP;
		offset = -1;
		break;
	case DA_RIGHT_UP:
		pHeader = DHeader(DOCK_RIGHT);
		dockType = DOCK_RIGHT;
		offset = 1;
		break;
	case DA_RIGHT_DOWN:
		pHeader = DHeader(DOCK_RIGHT);
		dockType = DOCK_RIGHT;
		offset = -1;
		break;
	case DA_BOTTOM_LEFT:
		pHeader = DHeader(DOCK_BOTTOM);
		dockType = DOCK_BOTTOM;
		offset = 1;
		break;
	case DA_BOTTOM_RIGHT:
		pHeader = DHeader(DOCK_BOTTOM);
		dockType = DOCK_BOTTOM;
		offset = -1;
		break;
	}
	pItem = pHeader->pNext;
	while(pItem){
		pItem->index += offset;
		MoveDockWindow(pItem->hwnd, dockType, pItem->index);
		pItem = pItem->pNext;
	}
}

static int CheckDockPositionForAlarm(HWND hwnd){
	P_NOTE_DATA			pD;
	int					result;

	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	result = ArrowRequired(hwnd, DockType(pD->dockData), pD->dockData);
	if(result == -1){
		return 0;
	}
	else{
		ShiftNotes(result);
		return 1;
	}
}

static void DoAlarm(HWND hwnd){
	wchar_t				szPath[MAX_PATH];
	RECT				rc;
	HWND				hBell;
	P_NOTE_RTHANDLES	pH;
	P_NOTE_DATA			pD;
	P_SCHEDULE_TYPE		pS;
	P_NOTE_FLAGS 		pF;

	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pS = (P_SCHEDULE_TYPE)GetWindowLongW(hwnd, OFFSET_SCHEDULE);
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);

	if(DockType(pD->dockData) == DOCK_NONE){
		SendMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_SHOW_CENTER, 0), 0);
	}
	else{
		while(CheckDockPositionForAlarm(hwnd) == 1){
			;
		}
	}
	if(g_NoteSettings.visualAlert){
		hBell = (HWND)GetPropW(hwnd, PH_BELL);
		if(hBell && IsWindow(hBell)){
			DestroyWindow(hBell);
		}
		CopyRect(&rc, &pH->rcTooltip);
		MapWindowPoints(hwnd, HWND_DESKTOP, (LPPOINT)&rc, 2);
		if(!pH->hbSkin)
			OffsetRect(&rc, -16, -16);
		SetPropW(hwnd, PH_BELL, (HANDLE)CreateBell(hwnd, (HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), rc.left, rc.top, pD->onTop));
	}
	if(g_Sound.allowSound){
		if(g_Sound.defSound == 0)
			PlaySoundW(__T("Notify"), 0, SND_ALIAS | SND_ASYNC);
		else{
			wcscpy(szPath, g_NotePaths.SoundDir);
			wcscat(szPath, g_Sound.custSound);
			PlaySoundW(szPath, 0, SND_FILENAME | SND_ASYNC);
		}
	}
	if(pS->scType == SCH_ONCE || pS->scType == SCH_AFTER){
		//stop timer and remove scheduling because alarm should fire only once
		pS->scType = SCH_NO;
		KillTimer(hwnd, pH->idTimer);
		WritePrivateProfileStructW(pF->id, S_SCHEDULE, pS, sizeof(SCHEDULE_TYPE), g_NotePaths.DataFile);
		RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);
	}
}

static BOOL CALLBACK PropEnumProc(HWND hwnd, LPCWSTR lpszString, HANDLE hData){
	if((DWORD)(VOID *)hData != 0xffffffff){
		RemovePropW(hwnd, lpszString);
	}
	return TRUE;
}

static void FormatNewNoteName(wchar_t * lpID){
	
	SYSTEMTIME		st;

	GetLocalTime(&st);
	ConstructDateTimeString(&st, lpID, TRUE);
}

static void FormatNewNoteID(wchar_t * lpID){
	SYSTEMTIME		st;
	wchar_t			buff[8];

	GetLocalTime(&st);
	_itow(st.wYear, buff, 10);
	wcscpy(lpID, buff);
	_itow(st.wMonth, buff, 10);
	wcscat(lpID, buff);
	_itow(st.wDay, buff, 10);
	wcscat(lpID, buff);
	_itow(st.wHour, buff, 10);
	wcscat(lpID, buff);
	_itow(st.wMinute, buff, 10);
	wcscat(lpID, buff);
	_itow(st.wSecond, buff, 10);
	wcscat(lpID, buff);
	_itow(st.wMilliseconds, buff, 10);
	wcscat(lpID, buff);
}

static void AutosaveNote(HWND hwnd, P_NOTE_FLAGS pF){
	P_NOTE_APPEARANCE	pA;
	P_SCHEDULE_TYPE		pS;
	P_NOTE_DATA			pD;
	P_NOTE_RTHANDLES	pH;
	wchar_t				szPath[MAX_PATH], szDBFile[MAX_PATH];
	HANDLE				hFile;
	EDITSTREAM			esm;
	RECT				rcNote;
	NOTE_REL_POSITION	nrp;
	SIZE				sz;

	if(!NotesDirExists(g_NotePaths.DataDir))
		NotesDirCreate(g_NotePaths.DataDir);
	wcscpy(szDBFile, g_NotePaths.DataDir);
	wcscat(szDBFile, AUTOSAVE_INDEX_FILE);
	if(!PathFileExistsW(szDBFile)){
		//create file with "Hidden" attribute
		hFile = CreateFileW(szDBFile, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_HIDDEN, NULL);
		if(hFile == INVALID_HANDLE_VALUE)
			return;
		CloseHandle(hFile);
	}
	//get runtime handles
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	//get note data
	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	//get note appearance
	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hwnd, OFFSET_APPEARANCE);
	//get scheduling
	pS = (P_SCHEDULE_TYPE)GetWindowLongW(hwnd, OFFSET_SCHEDULE);
	if(!pF->fromDB){
		GetTemporarySaveName(hwnd, pD->szName, 128);
	}
	//save window position and size
	if(DockType(pD->dockData) == DOCK_NONE){
		GetWindowRect(hwnd, &rcNote);
		//save current absolute position
		if(pH->rolled)
			rcNote.bottom = rcNote.top + (pH->rcSize.bottom - pH->rcSize.top);
		CopyRect(&pD->rcp, &rcNote);
		CopyRect(&pD->rcp, &rcNote);
		//save rolled status
		pD->rolled = pH->rolled;
		//save current relational position
		sz = GetScreenMetrics();
		nrp.left = (double)rcNote.left / (double)sz.cx;
		nrp.top = (double)rcNote.top / (double)sz.cy;
		nrp.width = rcNote.right - rcNote.left;
		nrp.height = rcNote.bottom - rcNote.top;
		WritePrivateProfileStructW(pF->id, IK_RELPOSITION, &nrp, sizeof(nrp), szDBFile);
	}
	//save note
	wcscpy(szPath, g_NotePaths.DataDir);
	wcscat(szPath, L"~");
	wcscat(szPath, pF->id);
	wcscat(szPath, L".no~");
	hFile = CreateFileW(szPath, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_HIDDEN, NULL);
	if(hFile != INVALID_HANDLE_VALUE){
		esm.dwCookie = (DWORD)hFile;
		esm.dwError = 0;
		esm.pfnCallback = OutStreamCallback;
		RichEdit_StreamOut((HWND)GetPropW(hwnd, PH_EDIT), SF_RTF | SF_UNICODE, &esm);
		CloseHandle(hFile);
	}
	if((pF->maskChanged & F_SKIN) == F_SKIN 
	|| (pF->maskChanged & F_RTFONT) == F_RTFONT 
	|| (pF->maskChanged & F_C_COLOR) == F_C_COLOR
	|| (pF->maskChanged & F_C_FONT) == F_C_FONT
	|| (pF->maskChanged & F_B_COLOR) == F_B_COLOR){
		if((pF->maskChanged & F_SKIN) == F_SKIN)
			pA->nPrivate |= F_SKIN;
		if((pF->maskChanged & F_RTFONT) == F_RTFONT)
			pA->nPrivate |= F_RTFONT;
		if((pF->maskChanged & F_C_COLOR) == F_C_COLOR)
			pA->nPrivate |= F_C_COLOR;
		if((pF->maskChanged & F_C_FONT) == F_C_FONT)
			pA->nPrivate |= F_C_FONT;
		if((pF->maskChanged & F_B_COLOR) == F_B_COLOR)
			pA->nPrivate |= F_B_COLOR;
		WritePrivateProfileStructW(pF->id, S_APPEARANCE, pA, sizeof(NOTE_APPEARANCE), szDBFile);
	}
	if((pF->maskChanged & F_SCHEDULE) == F_SCHEDULE)
		WritePrivateProfileStructW(pF->id, S_SCHEDULE, pS, sizeof(SCHEDULE_TYPE), szDBFile);
	GetLocalTime(&pD->stChanged);
	WritePrivateProfileStructW(pF->id, S_DATA, pD, sizeof(NOTE_DATA), szDBFile);
}

static void SaveNote(HWND hwnd, P_NOTE_FLAGS pF){
	
	wchar_t				szPath[MAX_PATH];
	P_NOTE_APPEARANCE	pA;
	P_SCHEDULE_TYPE		pS;
	P_NOTE_DATA			pD;
	P_NOTE_RTHANDLES	pH;
	HANDLE				hFile;
	EDITSTREAM			esm;

	//get runtime handles
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	//get note data
	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	//get note appearance
	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hwnd, OFFSET_APPEARANCE);
	//get scheduling
	pS = (P_SCHEDULE_TYPE)GetWindowLongW(hwnd, OFFSET_SCHEDULE);
	//save window position and size
	if(DockType(pD->dockData) == DOCK_NONE){
		WriteNotePlacement(hwnd, TRUE);
	}
	//save text
	if((pF->maskChanged & F_TEXT) == F_TEXT){	
		if(!NotesDirExists(g_NotePaths.DataDir))
			NotesDirCreate(g_NotePaths.DataDir);
		wcscpy(szPath, g_NotePaths.DataDir);
		wcscat(szPath, pF->id);
		wcscat(szPath, NOTE_EXTENTION);
		hFile = CreateFileW(szPath, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if(hFile != INVALID_HANDLE_VALUE){
			esm.dwCookie = (DWORD)hFile;
			esm.dwError = 0;
			esm.pfnCallback = OutStreamCallback;
			RichEdit_StreamOut((HWND)GetPropW(hwnd, PH_EDIT), SF_RTF | SF_UNICODE, &esm);
			CloseHandle(hFile);
			DeleteAutoSavedFile(pF->id);
		}
	}
	if((pF->maskChanged & F_SKIN) == F_SKIN 
	|| (pF->maskChanged & F_RTFONT) == F_RTFONT 
	|| (pF->maskChanged & F_C_COLOR) == F_C_COLOR
	|| (pF->maskChanged & F_C_FONT) == F_C_FONT
	|| (pF->maskChanged & F_B_COLOR) == F_B_COLOR){
		if((pF->maskChanged & F_SKIN) == F_SKIN)
			pA->nPrivate |= F_SKIN;
		if((pF->maskChanged & F_RTFONT) == F_RTFONT)
			pA->nPrivate |= F_RTFONT;
		if((pF->maskChanged & F_C_COLOR) == F_C_COLOR)
			pA->nPrivate |= F_C_COLOR;
		if((pF->maskChanged & F_C_FONT) == F_C_FONT)
			pA->nPrivate |= F_C_FONT;
		if((pF->maskChanged & F_B_COLOR) == F_B_COLOR)
			pA->nPrivate |= F_B_COLOR;
		WritePrivateProfileStructW(pF->id, S_APPEARANCE, pA, sizeof(NOTE_APPEARANCE), g_NotePaths.DataFile);
	}
	if((pF->maskChanged & F_SCHEDULE) == F_SCHEDULE)
		WritePrivateProfileStructW(pF->id, S_SCHEDULE, pS, sizeof(SCHEDULE_TYPE), g_NotePaths.DataFile);

	GetLocalTime(&pD->stChanged);
	WritePrivateProfileStructW(pF->id, S_DATA, pD, sizeof(NOTE_DATA), g_NotePaths.DataFile);

	pF->saved = TRUE;
	pF->fromDB = TRUE;

	RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);
	SetTooltip(hwnd, pH->hTooltip, pD, pH, TRUE);

	if(g_hCPDialog)
		SendMessageW(g_hCPDialog, CHM_CTRL_UPD, 0, (LPARAM)hwnd);
}

static BOOL CALLBACK SaveAs_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	HANDLE_MSG (hwnd, WM_INITDIALOG, SaveAs_OnInitDialog);
	HANDLE_MSG (hwnd, WM_COMMAND, SaveAs_OnCommand);
	HANDLE_MSG (hwnd, WM_CLOSE, SaveAs_OnClose);

	default: return FALSE;
	}
}

static BOOL SaveAs_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	P_NOTE_DATA		pD;
	P_NOTE_FLAGS 	pF;
	wchar_t 		szBuffer[256], szFile[MAX_PATH];

	wcscpy(szFile, g_NotePaths.LangDir);
	wcscat(szFile, g_NotePaths.LangFile);

	//set dialog and controls text
	GetPrivateProfileStringW(S_OPTIONS, __T("1002"), __T("Save note as..."), szBuffer, 256, szFile);
	SetWindowTextW(hwnd, szBuffer);
	SetDlgCtlText(hwnd, IDOK, szFile, DS_OK);
	SetDlgCtlText(hwnd, IDCANCEL, szFile, DS_CANCEL);
	SetDlgCtlText(hwnd, IDC_ST_SAVE_AS, szFile, __T("Note name"));
	SendDlgItemMessageW(hwnd, IDC_EDT_SAVE_AS, EM_SETLIMITTEXT, 127, 0);
	//save note handle
	SetWindowLongW(hwnd, GWL_USERDATA, (LONG)lParam);
	pD = (P_NOTE_DATA)GetWindowLongW((HWND)lParam, OFFSET_DATA);
	pF = (P_NOTE_FLAGS)GetWindowLongW((HWND)lParam, OFFSET_FLAGS);
	if(pF->fromDB || pF->saved)
		wcscpy(szBuffer, pD->szName);
	else
		if(GetTemporarySaveName((HWND)lParam, szBuffer, 128) == 0)
			wcscpy(szBuffer, pD->szName);
	SetDlgItemTextW(hwnd, IDC_EDT_SAVE_AS, szBuffer);
	return FALSE;
}

static void SaveAs_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	P_NOTE_DATA			pD;
	HWND				hNote;

	switch(id){
		case IDCANCEL:
			EndDialog(hwnd, IDCANCEL);
			break;
		case IDOK:
			hNote = (HWND)GetWindowLongW(hwnd, GWL_USERDATA);
			pD = (P_NOTE_DATA)GetWindowLongW(hNote, OFFSET_DATA);
			GetDlgItemTextW(hwnd, IDC_EDT_SAVE_AS, pD->szName, 128);
			SetNoteCaption(hNote);
			EndDialog(hwnd, IDOK);
			break;
		case IDC_EDT_SAVE_AS:
			if(codeNotify == EN_CHANGE){
				if(GetWindowTextLengthW(GetDlgItem(hwnd, IDC_EDT_SAVE_AS)))
					EnableWindow(GetDlgItem(hwnd ,IDOK), TRUE);
				else
					EnableWindow(GetDlgItem(hwnd ,IDOK), FALSE);
			}
			break;
	}
}

static void SaveAs_OnClose(HWND hwnd)
{
	EndDialog(hwnd, IDCANCEL);
}

static LRESULT CALLBACK Note_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam){

	P_NOTE_RTHANDLES	pH;
	P_NOTE_DATA			pD;
	P_NOTE_FLAGS 		pF;
	P_NOTE_APPEARANCE	pA;
	P_SCHEDULE_TYPE		pS;
	NOTE_DATA			data;
	int					groupTemp;

	switch(msg){
		HANDLE_MSG (hwnd, WM_CREATE, Note_OnCreate);
		HANDLE_MSG (hwnd, WM_PAINT, Note_OnPaint);
		HANDLE_MSG (hwnd, WM_MOUSEMOVE, Note_OnMouseMove);
		HANDLE_MSG (hwnd, WM_CLOSE, Note_OnClose);
		HANDLE_MSG (hwnd, WM_COMMAND, Note_OnCommand);
		HANDLE_MSG (hwnd, WM_SETFOCUS, Note_OnSetFocus);
		HANDLE_MSG (hwnd, WM_LBUTTONDOWN, Note_OnLButtonDown);
		HANDLE_MSG (hwnd, WM_RBUTTONUP, Note_OnRButtonUp);
		HANDLE_MSG (hwnd, WM_KILLFOCUS, Note_OnKillFocus);
		HANDLE_MSG (hwnd, WM_DRAWITEM, Note_OnDrawItem);
		HANDLE_MSG (hwnd, WM_MEASUREITEM, Note_OnMeasureItem);
		HANDLE_MSG (hwnd, WM_SIZE, Note_OnSize);
		HANDLE_MSG (hwnd, WM_DROPFILES, Note_OnDropFiles);
		HANDLE_MSG (hwnd, WM_WINDOWPOSCHANGING, Note_OnWindowPosChanging);
		HANDLE_MSG (hwnd, WM_RBUTTONDOWN, Note_OnRButtonDown);
		HANDLE_MSG (hwnd, WM_LBUTTONDBLCLK, Note_OnLButtonDblClk);
		HANDLE_MSG (hwnd, WM_MOUSELEAVE, Note_OnMouseLeave);
		HANDLE_MSG (hwnd, WM_INITMENUPOPUP, Note_OnInitMenuPopup);
		case WM_SYSCOMMAND:
			//hide note window instead of closing when Alt+F4 is pressed
			if((wParam & SC_CLOSE) == SC_CLOSE){
				SendMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_HIDE, 0), 0);
				return FALSE;
			}
			return DefWindowProcW (hwnd, msg, wParam, lParam);
		case CHM_RESTORE_BIN:
			pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
			pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
			pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
			pS = (P_SCHEDULE_TYPE)GetWindowLongW(hwnd, OFFSET_SCHEDULE);
			if(wParam == -1)
				groupTemp = pD->idPrevGroup;
			else
				//in case of grag onto different group
				groupTemp = wParam;
			//check whether group exists
			if(GetGroup(groupTemp) == NULL)
				groupTemp = 0;
			pD->idGroup = groupTemp;
			if(IsBitOn(pD->res1, NB_PREV_VISIBLE)){
				ShowWindow(hwnd, SW_SHOW);
				pD->visible = TRUE;
				SaveVisibility(hwnd, TRUE);
			}
			if(pF->fromDB){
				if(GetPrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile)){
					data.idGroup = pD->idGroup;
					WritePrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile);
				}
			}
			if(pS->scType != 0){
				SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
				if(pS->scType == SCH_REP)	//set repeat count according to timer interval
					pH->nRepeat = SetRepeatCount(&pS->scDate);
				SetTimer(hwnd, 1, TIMER_INTERVAL, TimerProc);
				pH->idTimer = 1;
			}
			return FALSE;
		case CHM_DELETE_BIN:
			pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
			pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
			if(wParam == -1)
				pD->idPrevGroup = pD->idGroup;
			else
				pD->idPrevGroup = 0;
			pD->idGroup = GROUP_RECYCLE;
			pD->visible = FALSE;
			if(pF->fromDB){
				if(GetPrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile)){
					data.idGroup = pD->idGroup;
					data.idPrevGroup = pD->idPrevGroup;
					WritePrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile);
				}
			}
			if(IsWindowVisible(hwnd)){
				BitOn(&pD->res1, NB_PREV_VISIBLE);
			}
			else{
				BitOff(&pD->res1, NB_PREV_VISIBLE);
			}
			ShowWindow(hwnd, SW_HIDE);
			SaveVisibility(hwnd, FALSE);
			if(DockType(pD->dockData) != DOCK_NONE){
				SendMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_DOCK_NONE, 0), 0);
			}
			return FALSE;
		case CHM_DELETE_COMPLETE:
			pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
			pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
			WritePrivateProfileSectionW(pF->id, NULL, g_NotePaths.DataFile);
			DeleteRTFile(pF->id);
			if(DockType(pD->dockData) != DOCK_NONE){
				SendMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_DOCK_NONE, 0), 0);
			}
			SendMessageW(hwnd, WM_CLOSE, 0, 0);
			return FALSE;
		case CHM_RESIZE:
			OnResizeNote(hwnd, (LPRECT)wParam);
			return FALSE;
		case CHM_SAVE:
			SaveNote(hwnd, (P_NOTE_FLAGS)wParam);
			return FALSE;
		case CHM_SAVE_ON_SHUTDOWN:{
			wchar_t szBuffer[128];
			pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
			if(GetTemporarySaveName(hwnd, szBuffer, 128) != 0)
				wcscpy(pD->szName, szBuffer);
			SaveNote(hwnd, (P_NOTE_FLAGS)wParam);
			return FALSE;
		}
		case CHM_AUTOSAVE:
			AutosaveNote(hwnd, (P_NOTE_FLAGS)wParam);
			return FALSE;
		case CHM_TT_UPDATE:
			if(g_RTHandles.hbSkin){
				if(g_DockSettings.fCustSkin){
					pH = &g_DRTHandles;
				}
				else{
					pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
				}
			}
			else{
				pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
			}
			pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
			SetTooltip(hwnd, pH->hTooltip, pD, pH, TRUE);
			return FALSE;
		case CHM_CHANGE_FONT:
			SetREFontCommon((HWND)GetPropW(hwnd, PH_EDIT), (LPLOGFONTW)wParam);
			return FALSE;
		case CHM_CHANGE_FONT_COLOR:
			SetREFontColorCommon((HWND)GetPropW(hwnd, PH_EDIT), (COLORREF)wParam);
			return FALSE;
		case CHM_NEW_LANGUAGE:
			SetCommandsTooltips(hwnd);
			return FALSE;
		case CHM_MOVE_DOCK_NOTE:
			pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
			if(DockType(pD->dockData) != DOCK_NONE){
				HWND hTray = MoveDockWindow(hwnd, DockType(pD->dockData), DockIndex(pD->dockData));
				if(hTray)
					SetWindowPos(hTray, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_SHOWWINDOW);
			}
			return FALSE;
		case CHM_NEW_BGCOLOR:
			pA = (P_NOTE_APPEARANCE)GetWindowLongW(hwnd, OFFSET_APPEARANCE);
			pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
			RichEdit_SetBkgndColor(GetPropW(hwnd, PH_EDIT), 0, pA->crWindow);
			CreateSkinlessToolbarBitmap(hwnd, pA, pH, &pH->hbCommand);
			CreateSkinlessSysBitmap(hwnd, pA, pH, &pH->hbDelHide);
			RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);	
			return FALSE;
		case CHM_APPLY_NEW_SKIN:
			ApplyNewSkin(hwnd, (P_NOTE_RTHANDLES)wParam);
			return FALSE;
		case WM_NOTIFY:{
			LPNMHDR pnmh = (LPNMHDR)lParam;
			if(pnmh->code == EN_MSGFILTER){
				MSGFILTER * lpmf = (MSGFILTER *)lParam;
				if(lpmf->msg == WM_CHAR || lpmf->msg == WM_KEYDOWN){
					pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
					pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
					if(pF->transparent){
						//draw window opaque while typing
						SetLayeredWindowAttributes(hwnd, pH->crMask, 255, LWA_COLORKEY);
						pF->transparent = FALSE;
					}
					if(lpmf->msg == WM_KEYDOWN && (GetKeyState(VK_CONTROL) & 0x8000) == 0x8000){
						if((GetKeyState(VK_F) & 0x8000) == 0x8000)
							PostMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_SEARCH_NOTE, 0), 0);
						else if((GetKeyState(VK_H) & 0x8000) == 0x8000)
							PostMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_REPLACE_NOTE, 0), 0);
						else if((GetKeyState(VK_S) & 0x8000) == 0x8000)
							PostMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_SAVE, 0), 0);
					}
				}
			}
			else if(pnmh->code == EN_LINK){
				ENLINK * lpel = (ENLINK *)lParam;
				if(lpel->msg == WM_LBUTTONUP){
					wchar_t 	szBuffer[256];
					TEXTRANGEW	tr;
					tr.chrg.cpMax = lpel->chrg.cpMax;
					tr.chrg.cpMin = lpel->chrg.cpMin;
					tr.lpstrText = szBuffer;
					SendMessageW((HWND)GetPropW(hwnd, PH_EDIT), EM_GETTEXTRANGE, 0, (LPARAM)&tr);
					
					if(_wcsistr(szBuffer, L"http:") == szBuffer || _wcsistr(szBuffer, L"ftp:") == szBuffer || _wcsistr(szBuffer, L"https:") == szBuffer || _wcsistr(szBuffer, L"gopher:") == szBuffer || _wcsistr(szBuffer, L"www.") == szBuffer){
						if(wcslen(g_DefBrowser) == 0)
							ShellExecuteW(hwnd, L"open", szBuffer, NULL, NULL, SW_SHOWDEFAULT);
						else{
							if((int)ShellExecuteW(hwnd, NULL, g_DefBrowser, szBuffer, NULL, SW_SHOWDEFAULT) <= 32){
								ShellExecuteW(hwnd, L"open", szBuffer, NULL, NULL, SW_SHOWDEFAULT);
							}
						}
					}
					else{
						ShellExecuteW(hwnd, L"open", szBuffer, NULL, NULL, SW_SHOWDEFAULT);
					}
				}
			}
			return FALSE;
		}
		default: 
			return DefWindowProcW (hwnd, msg, wParam, lParam);
	}
}

static void Note_OnInitMenuPopup(HWND hwnd, HMENU hMenu, UINT item, BOOL fSystemMenu)
{
	PrepareMenuGradientColors(GetSysColor(COLOR_BTNFACE));
}

static void Note_OnMouseLeave(HWND hwnd){
	P_NOTE_FLAGS		pF;
	P_NOTE_DATA			pD;
	POINT				pt;
	RECT				rc;

	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	GetCursorPos(&pt);
	GetWindowRect(hwnd, &rc);
	pF->mouseIn = FALSE;
	if(!PtInRect(&rc, pt)){
		dock_type dockType = DockType(pD->dockData);
		switch(dockType){
		case DOCK_LEFT:
			ShowWindow(g_DArrows.hlu, SW_HIDE);
			ShowWindow(g_DArrows.hld, SW_HIDE);
			break;
		case DOCK_TOP:
			ShowWindow(g_DArrows.htl, SW_HIDE);
			ShowWindow(g_DArrows.htr, SW_HIDE);
			break;
		case DOCK_RIGHT:
			ShowWindow(g_DArrows.hru, SW_HIDE);
			ShowWindow(g_DArrows.hrd, SW_HIDE);
			break;
		case DOCK_BOTTOM:
			ShowWindow(g_DArrows.hbl, SW_HIDE);
			ShowWindow(g_DArrows.hbr, SW_HIDE);
			break;
		}
	}
}

static void Note_OnLButtonDblClk(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
	P_NOTE_DATA			pD;
	P_NOTE_RTHANDLES	pH;
	POINT				pt;
	RECT				rc;
	
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pt.x = x;
	pt.y = y;
	if(g_NoteSettings.rollOnDblClick){
		if(!pH->hbSkin && DockType(pD->dockData) == DOCK_NONE){
			if(PtInRect(&pH->rcCaption, pt)){
				GetWindowRect(hwnd, &rc);
				if(pH->rolled){
					MoveWindow(hwnd, rc.left, rc.top, rc.right - rc.left, pH->rcSize.bottom - pH->rcSize.top, TRUE);
					pH->rolled = FALSE;
				}
				else{
					m_DblClickSize = TRUE;
					CopyRect(&pH->rcSize, &rc);
					CopyRect(&pD->rcp, &rc);
					MoveWindow(hwnd, rc.left, rc.top, rc.right - rc.left, pH->rcCaption.bottom - pH->rcCaption.top, TRUE);
					pH->rolled = TRUE;
					m_DblClickSize = FALSE;
				}
			}
		}
	}
}

static void Note_OnDropFiles(HWND hwnd, HDROP hdrop)
{
	P_NOTE_FLAGS	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	P_NOTE_DATA		pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	HANDLE			hFile;
	wchar_t			szFile[MAX_PATH], szMessage[256];
	int 			count = DragQueryFileW(hdrop, 0xffffffff, NULL, 0);
	int				result = IDYES;
	BYTE			bytes[4];
	int				flags = 0;
	ULONG			read;

	if(count > 1){
		MessageBoxW(hwnd, g_Strings.TooManyFiles, PROG_NAME, MB_OK | MB_ICONERROR);
	}
	else{
		if(!pF->saved){
			ConstructSaveMessage(pD->szName, szMessage);
			result = MessageBoxW(hwnd, szMessage, g_Strings.SCaption, MB_YESNOCANCEL | MB_ICONQUESTION);
			if(result == IDYES)
				SaveNote(hwnd, pF);
		}
		if(result != IDCANCEL){
			DragQueryFileW(hdrop, 0, szFile, MAX_PATH);
			hFile = CreateFileW(szFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
			if(hFile != INVALID_HANDLE_VALUE){
				//check which file is it
				ReadFile(hFile, bytes, 5, &read, NULL);
				if(bytes[2] == 'r' && bytes[3] == 't' && bytes[4] == 'f')
					flags = SF_RTF;
				if((bytes[0] == 0x0ff && bytes[1] == 0x0fe) || (bytes[0] == 0x0fe && bytes[1] == 0x0ff)){
					//unicode or unicode BE
					flags |= SF_TEXT | SF_UNICODE;
					SetFilePointer(hFile, -3, 0, FILE_CURRENT);
				}
				else if(bytes[0] == 0x0ef && bytes[1] == 0x0bb && bytes[2] == 0x0bf){
					//utf8
					flags |= SF_TEXT;
				}
				else{
					flags |= SF_TEXT;
					SetFilePointer(hFile, -5, 0, FILE_CURRENT);
				}
				EDITSTREAM			esm;
				esm.dwCookie = (DWORD)hFile;
				esm.dwError = 0;
				esm.pfnCallback = InStreamCallback;
				RichEdit_StreamIn((HWND)GetPropW(hwnd, PH_EDIT), flags, &esm);
				CloseHandle(hFile);
			}		
		}
	}
}

static BOOL Note_OnWindowPosChanging(HWND hwnd, LPWINDOWPOS lpwpos)
{
	P_NOTE_RTHANDLES	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);

	if(pH){
		if(!pH->hbSkin){
			if(lpwpos->cx < DEF_SIZE_MIN){
				lpwpos->cx = DEF_SIZE_MIN;
			}
			if(m_DblClickSize){
				if(lpwpos->cy < (pH->rcCaption.bottom - pH->rcCaption.top)){
					lpwpos->cy = pH->rcCaption.bottom - pH->rcCaption.top;
				}
			}
			else{
				if(!pH->rolled){
					if(lpwpos->cy < (pH->rcCaption.bottom - pH->rcCaption.top) + (pH->rcGrip.bottom - pH->rcGrip.top)){
						lpwpos->cy = (pH->rcCaption.bottom - pH->rcCaption.top) + (pH->rcGrip.bottom - pH->rcGrip.top);
					}
				}
				else{
					if(lpwpos->cy < (pH->rcCaption.bottom - pH->rcCaption.top)){
						lpwpos->cy = pH->rcCaption.bottom - pH->rcCaption.top;
					}
				}
			}
		}
	}
	return FALSE;
}

static void Note_OnSize(HWND hwnd, UINT state, int cx, int cy)
{
	P_NOTE_RTHANDLES	pH;
	HWND				hEdit, hGrip;
	P_NOTE_DATA			pD;

	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);

	if(pH && pH->hbSkin){
		if(DockType(pD->dockData) != DOCK_NONE){
			pH = &g_DRTHandles;
		}
	}

	if(pH && !pH->hbSkin){
		if(!m_DblClickSize){
			//save note size if the resize event was not raised from double click
			GetWindowRect(hwnd, &pH->rcSize);
			MeasureSkinlessNoteRectangles(pH, cx, cy);
			hEdit = (HWND)GetPropW(hwnd, PH_EDIT);
			hGrip = (HWND)GetPropW(hwnd, PH_GRIP);
			MoveWindow(hEdit, 1, pH->rcCaption.bottom + 1, cx - 2, pH->rcGrip.top - (pH->rcCaption.bottom - pH->rcCaption.top) - 1, TRUE);
			MoveWindow(hGrip, pH->rcGrip.left, pH->rcGrip.top, pH->rcGrip.right - pH->rcGrip.left, pH->rcGrip.bottom - pH->rcGrip.top, TRUE);
			SetNoteCommandsBar(hwnd, pH);
			pH->rolled = FALSE;
		}
		else{
			;
		}
		m_DblClickSize = FALSE;
	}
}

static void Note_OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT * lpDrawItem)
{
	if(lpDrawItem->CtlType == ODT_MENU)
		DrawMItem(lpDrawItem, g_hBmpMenuNormal, g_hBmpMenuGray, CLR_MASK);
}

static void Note_OnMeasureItem(HWND hwnd, MEASUREITEMSTRUCT * lpMeasureItem)
{
	MeasureMItem(g_hMenuFont, lpMeasureItem);
}

static void Note_OnKillFocus(HWND hwnd, HWND hwndNewFocus)
{
	P_NOTE_FLAGS		pF;
	P_NOTE_RTHANDLES	pH;

	if(hwndNewFocus != (HWND)GetPropW(hwnd, PH_EDIT)){
		pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
		if(g_NoteSettings.transAllow){
			pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
			SetLayeredWindowAttributes(hwnd, pH->crMask, g_NoteSettings.transValue, LWA_COLORKEY | LWA_ALPHA);
			pF->transparent = TRUE;
		}
	}
}

static void Note_OnRButtonUp(HWND hwnd, int x, int y, UINT flags)
{
	P_NOTE_FLAGS 	pF;
	P_NOTE_DATA		pD;
	HMENU			hMenu = g_hNotePopUp;
	dock_type		dockType;

	m_hTempMenu = hMenu;
	
	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	dockType = DockType(pD->dockData);
	if(pF->maskChanged && !pF->saved)
		EnableMenuItem(hMenu, IDM_SAVE, MF_BYCOMMAND | MF_ENABLED);
	else
		EnableMenuItem(hMenu, IDM_SAVE, MF_BYCOMMAND | MF_GRAYED);

	if(g_hAdjust == NULL)
		EnableMenuItem(hMenu, IDM_ADJUST, MF_BYCOMMAND | MF_ENABLED);
	else
		EnableMenuItem(hMenu, IDM_ADJUST, MF_BYCOMMAND | MF_GRAYED);

	PrepareDockMenu(hwnd, hMenu, GetMenuPosition(hMenu, IDM_DOCK));
	AddGroupsMenuItems(hwnd, hMenu);

	if(pD->onTop || dockType != DOCK_NONE)
		CheckMenuItem(hMenu, IDM_ON_TOP, MF_CHECKED | MF_BYCOMMAND);
	else
		CheckMenuItem(hMenu, IDM_ON_TOP, MF_UNCHECKED | MF_BYCOMMAND);

	if(dockType != DOCK_NONE){
		EnableMenuItem(hMenu, IDM_ON_TOP, MF_BYCOMMAND | MF_GRAYED);
	}
	else{
		EnableMenuItem(hMenu, IDM_ON_TOP, MF_BYCOMMAND | MF_ENABLED);
	}

	ShowPopUp(hwnd, hMenu);
}

static void SetDockType(int * dockData, dock_type dockType){
	short		th = HIWORD(*dockData);

	*dockData = MAKELONG(dockType, th);
}

static void UndockNote(HWND hwnd, P_NOTE_DATA pD){

	dock_type			dockType;
	PDOCKITEM			pRemain;
	P_NOTE_DATA			ptd;
	P_NOTE_RTHANDLES	pH;
	P_NOTE_APPEARANCE	pA;
	HWND				hTray = NULL;
	BOOL				rolled;

	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	rolled = pH->rolled;

	dockType = DockType(pD->dockData);
	pRemain = NextDockItem(DHeader(dockType), hwnd);
	SetDockType(&pD->dockData, DOCK_NONE);
	SetDockIndex(&pD->dockData, -777);
	SaveNewDockData(hwnd, pD->dockData);
	RemoveDockItem(DHeader(dockType), hwnd);
	
	MoveWindow(hwnd, pD->rcp.left, pD->rcp.top, pD->rcp.right - pD->rcp.left, pD->rcp.bottom - pD->rcp.top, TRUE);

	if(!pD->prevOnTop){
		SetWindowPos(hwnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	}

	
	if(g_RTHandles.hbSkin){
		// if(g_DockSettings.fCustSkin){
			ApplyNewSkin(hwnd, pH);
		// }
	}
	else{
		if(g_DockSettings.fCustColor){
			pA = (P_NOTE_APPEARANCE)GetWindowLongW(hwnd, OFFSET_APPEARANCE);
			CreateSkinlessToolbarBitmap(hwnd, pA, pH, &pH->hbCommand);
			CreateSkinlessSysBitmap(hwnd, pA, pH, &pH->hbDelHide);
			RichEdit_SetBkgndColor(GetPropW(hwnd, PH_EDIT), 0, pA->crWindow);
		}
		RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);
	}

	while(pRemain){
		ptd = (P_NOTE_DATA)GetWindowLongW(pRemain->hwnd, OFFSET_DATA);
		// itmp = DockIndex(ptd->dockData);
		// itmp--;
		SetDockIndex(&ptd->dockData, pRemain->index);
		hTray = MoveDockWindow(pRemain->hwnd, dockType, pRemain->index);
		pRemain = pRemain->pNext;
	}

	if(rolled){
		pH->rolled = FALSE;
		SendMessageW(hwnd, WM_LBUTTONDBLCLK, MK_LBUTTON, MAKELPARAM(pH->rcCaption.left + 1, pH->rcCaption.top + 1));
	}

	if(hTray)
		SetWindowPos(hTray, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_SHOWWINDOW);
}

static void DockNote(HWND hwnd, dock_type dockType, P_NOTE_DATA pD){
	dock_type			dtPrev;
	PDOCKITEM			pRemain, pTemp;
	HWND				hTray = NULL;
	BOOL				rolled;
	P_NOTE_RTHANDLES	pH;

	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	rolled = pH->rolled;

	dtPrev = DockType(pD->dockData);

	if(dtPrev != DOCK_NONE){
		pRemain = NextDockItem(DHeader(dtPrev), hwnd);
	}

	SetDockType(&pD->dockData, dockType);
	pTemp = AddDockItem(DHeader(dockType), hwnd, -1);
	SetDockIndex(&pD->dockData, pTemp->index);
	SaveNewDockData(hwnd, pD->dockData);
	hTray = MoveDockWindow(hwnd, dockType, pTemp->index);
	if(hTray)
		SetWindowPos(hTray, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_SHOWWINDOW);
	if(g_RTHandles.hbSkin){
		if(g_DockSettings.fCustSkin){
			ApplyNewSkin(hwnd, &g_DRTHandles);
		}
	}
	else{
		if(g_DockSettings.fCustColor || g_DockSettings.fCustCaption){
			ApplyDockColors(hwnd);
		}
		RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);
	}
	
	if(!pD->onTop){
		SendMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_ON_TOP, 1), 0);
	}

	if(dtPrev != DOCK_NONE){
		P_NOTE_DATA		ptd;
		RemoveDockItem(DHeader(dtPrev), hwnd);
		while(pRemain){
			ptd = (P_NOTE_DATA)GetWindowLongW(pRemain->hwnd, OFFSET_DATA);
			SetDockIndex(&ptd->dockData, pRemain->index);
			hTray = MoveDockWindow(pRemain->hwnd, dtPrev, pRemain->index);
			pRemain = pRemain->pNext;
		}
		if(hTray)
			SetWindowPos(hTray, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_SHOWWINDOW);
	}

	pH->rolled = rolled;
}

static void AddGroupsMenuItems(HWND hwnd, HMENU hMenu){
	HMENU				hGroups;
	P_PNGROUP			ppg;
	MENUITEMINFOW		mi;
	int					count, pos = 0;
	P_NOTE_DATA			pD;
	P_NOTE_FLAGS 		pF;
	wchar_t				szBuffer[256], szFile[MAX_PATH];
	MItem				mit, *pmi;

	ZeroMemory(&mit, sizeof(mit));
	mit.xPos = -1;
	mit.yPos = -1;
	mit.xCheck = 22;

	wcscpy(szFile, g_NotePaths.LangDir);
	wcscat(szFile, g_NotePaths.LangFile);
	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	hGroups = GetSubMenu(hMenu, GetMenuPosition(hMenu, IDM_NOTE_GROUPS));
	count = GetMenuItemCount(hGroups);
	FreeMenus(hGroups);
	for(int i = count -1; i >= 0; i--){
		DeleteMenu(hGroups, i, MF_BYPOSITION);
	}

	ZeroMemory(&mi, sizeof(mi));
	mi.cbSize = sizeof(mi);
	mi.fMask = MIIM_DATA | MIIM_ID;
	mi.wID = IDM_ADD_TO_FAVORITES;
	GetMenuItemInfoW(g_hNotePopUp, IDM_ADD_TO_FAVORITES, FALSE, &mi);
	pmi = (PMItem)mi.dwItemData;
	if(pmi){
		GetPrivateProfileStringW(S_FAVORITES, pF->id, NULL, szBuffer, 256, g_NotePaths.INIFile);
		if(wcslen(szBuffer) > 0){
			SetMenuText(IDM_REMOVE_FAVORITES, S_MENU, szFile, L"Remove From Favorites", pmi->szText);
			pmi->xPos = 69;
		}
		else{
			SetMenuText(IDM_ADD_TO_FAVORITES, S_MENU, szFile, L"Add To Favorites", pmi->szText);
			pmi->xPos = 68;
		}
	}
	ppg = GetGroups();
	if(ppg){
		while(ppg){
			mit.id = ppg->id + GROUP_ADDITION;
			wcscpy(mit.szText, ppg->szName);
			if(ppg->id == pD->idGroup){
				AppendMenuW(hGroups, MF_STRING | MF_CHECKED, mit.id, mit.szText);
			}
			else{
				AppendMenuW(hGroups, MF_STRING , mit.id, mit.szText);
			}
			SetMenuItemProperties(&mit, hGroups, pos++);
			//move next
			ppg = ppg->next;
		}
	}
}

static void Note_OnRButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
	RemoveBell(hwnd);
}

static void Note_OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
	P_NOTE_RTHANDLES	pH;
	P_NOTE_FLAGS		pF;
	POINT				pt;
	
	RemoveBell(hwnd);

	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	pt.x = x;
	pt.y = y;
	if(pH->hbSkin){
		pF->inDrag = TRUE;
		SystemParametersInfoW(SPI_GETDRAGFULLWINDOWS, 0, &m_fFullDrag, 0);
	}
	else{
		if(PtInRect(&pH->rcCaption, pt)){
			pF->inDrag = TRUE;
			SystemParametersInfoW(SPI_GETDRAGFULLWINDOWS, 0, &m_fFullDrag, 0);
		}
		else
			pF->inDrag = FALSE;
	}
}

static void Note_OnSetFocus(HWND hwnd, HWND hwndOldFocus)
{
	SetFocus((HWND)GetPropW(hwnd, PH_EDIT));
}

static void Note_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	P_NOTE_FLAGS		pF;
	P_NOTE_DATA			pD;
	P_NOTE_RTHANDLES	pH;
	int					result = 0;
	BOOL				delete = FALSE, changed = FALSE, in_favorites = FALSE;
	wchar_t				szBuffer[256];

	switch(id){
	case IDM_ADJUST:
		pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
		m_Reload = FALSE;
		DialogBoxParamW((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), MAKEINTRESOURCEW(DLG_ADJUST), hwnd, Adjust_DlgProc, (LPARAM)hwnd);
		if(!m_Reload && !g_IsClosing){
			//make note opaque
			pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
			SetLayeredWindowAttributes(hwnd, pH->crMask, 255, LWA_COLORKEY);
			pF->transparent = FALSE;
		}
		g_hAdjust = NULL;
		break;
	case IDM_SAVE:
		pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
		pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
		if(pF->fromDB)
			SaveNote(hwnd, pF);
		else{
			//check whether note was in favorites
			GetPrivateProfileStringW(S_FAVORITES, pF->id, NULL, szBuffer, 256, g_NotePaths.INIFile);
			if(wcslen(szBuffer) > 0){
				//in such case delete its name from favorites
				in_favorites = TRUE;
				WritePrivateProfileStringW(S_FAVORITES, pF->id, NULL, g_NotePaths.INIFile);
			}
			result = DialogBoxParamW((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), MAKEINTRESOURCEW(DLG_SAVE_AS), hwnd, SaveAs_DlgProc, (LPARAM)hwnd);
			if(result == IDOK){
				SaveNote(hwnd, pF);
				if(in_favorites){
					//if note was previously in favorites - save it in favorites with new name
					pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
					WritePrivateProfileStringW(S_FAVORITES, pF->id, pD->szName, g_NotePaths.INIFile);
				}
			}
		}
		//make note opaque
		if(g_NoteSettings.transAllow){
			if(GetFocus() == (HWND)GetPropW(hwnd, PH_EDIT)){
				SetLayeredWindowAttributes(hwnd, pH->crMask, 255, LWA_COLORKEY);
				pF->transparent = FALSE;
			}
		}
		break;
	case IDM_ATTACHMENT:{
		wchar_t			szPath[MAX_PATH + 128], szTemp[MAX_PATH + 128];

		pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
		if(!pF->saved){
			if(MessageBoxW(hwnd, g_Strings.SaveBeforeSend, g_Strings.SCaption, MB_YESNO | MB_ICONQUESTION) == IDNO){
				break;
			}
			SendMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_SAVE, 0), 0);
		}
		wcscpy(szPath, g_NotePaths.DataDir);
		wcscat(szPath, pF->id);
		wcscat(szPath, L".pnote");
		SendAsAttachment(g_hMain, szPath, DEF_MAIL_SUBJECT, szTemp);
		break;
	}
	case IDM_SAVE_AS:
		pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
		pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
		//check whether note was in favorites
		GetPrivateProfileStringW(S_FAVORITES, pF->id, NULL, szBuffer, 256, g_NotePaths.INIFile);
		if(wcslen(szBuffer) > 0){
			//in such case delete its name from favorites
			in_favorites = TRUE;
			WritePrivateProfileStringW(S_FAVORITES, pF->id, NULL, g_NotePaths.INIFile);
		}
		result = DialogBoxParamW((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), MAKEINTRESOURCEW(DLG_SAVE_AS), hwnd, SaveAs_DlgProc, (LPARAM)hwnd);
		if(result == IDOK){
			SaveNote(hwnd, pF);
			if(in_favorites){
				//if note was previously in favorites - save it in favorites with new name
				pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
				WritePrivateProfileStringW(S_FAVORITES, pF->id, pD->szName, g_NotePaths.INIFile);
			}
		}
		//make note opaque
		if(g_NoteSettings.transAllow){
			if(GetFocus() == (HWND)GetPropW(hwnd, PH_EDIT)){
				SetLayeredWindowAttributes(hwnd, pH->crMask, 255, LWA_COLORKEY);
				pF->transparent = FALSE;
			}
		}
		break;
	case IDM_SHOW:
		ShowWindow(hwnd, SW_SHOW);
		if(g_hCPDialog)
			SendMessageW(g_hCPDialog, CHM_CTRL_UPD, 0, (LPARAM)hwnd);
		if(g_hSearchDialog)
			SendMessageW(g_hSearchDialog, CHM_SHOW_AFTER_SEARCH, (WPARAM)GetPropW(hwnd, PH_EDIT), 0);

		SaveVisibility(hwnd, TRUE);
		break;
	case IDM_HIDE:
		pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
		if(DockType(pD->dockData) != DOCK_NONE){
			SendMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_DOCK_NONE, 0), 0);
		}
		ShowWindow(hwnd, SW_HIDE);
		if(g_hCPDialog)
			SendMessageW(g_hCPDialog, CHM_CTRL_UPD, 0, (LPARAM)hwnd);
		if(g_hSearchDialog)
			SendMessageW(g_hSearchDialog, CHM_BECOMES_HIDDEN, (WPARAM)GetPropW(hwnd, PH_EDIT), 0);

		SaveVisibility(hwnd, FALSE);
		
		break;
	case IDM_SHOW_CENTER:
	case IDM_RESTORE_PLACEMENT:
		pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
		if(pD->idGroup > GROUP_RECYCLE){
			if(DockType(pD->dockData) != DOCK_NONE){
				SendMessageW(hwnd, WM_COMMAND, MAKEWPARAM(IDM_DOCK_NONE, 0), 0);
			}
			if(id == IDM_SHOW_CENTER)
				CenterWindow(hwnd, FALSE);
			else
				CenterWindow(hwnd, TRUE);
			ShowWindow(hwnd, SW_SHOW);
			SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
			if(!pD->onTop){
				SetWindowPos(hwnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
			}
			if(g_hCPDialog)
				SendMessageW(g_hCPDialog, CHM_CTRL_UPD, 0, (LPARAM)hwnd);
		}
		break;
	case IDM_DEL:{
		BOOL		delComplete = FALSE;
		wchar_t		szMessage[256];

		pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);

		WritePrivateProfileStringW(S_FAVORITES, pF->id, NULL, g_NotePaths.INIFile);

		if((GetKeyState(VK_LSHIFT) & 0x8000) == 0x8000){
			delComplete = TRUE;
			wcscpy(szMessage, g_Strings.DMessageComplete);
		}
		else{
			wcscpy(szMessage, g_Strings.DMessageBin);
		}
		if(codeNotify == CHN_DEL_WITH_GROUP){
			//do not ask question when delete with group
			delete = TRUE;
		}
		else{
			if(g_NoteSettings.confirmDelete){
				result = MessageBoxW(hwnd, szMessage, g_Strings.DCaption, MB_ICONQUESTION | MB_YESNO);
				if (result == IDYES){
					delete = TRUE;
				}
			}
			else
				delete = TRUE;
		}

		if(delete){
			//remove from favorites
			WritePrivateProfileStringW(S_FAVORITES, pF->id, NULL, g_NotePaths.INIFile);
			if(delComplete){
				SendMessageW(hwnd, CHM_DELETE_COMPLETE, 0, 0);
			}
			else{
				if(codeNotify == CHN_DEL_WITH_GROUP)
					SendMessageW(hwnd, CHM_DELETE_BIN, 0, 0);
				else
					SendMessageW(hwnd, CHM_DELETE_BIN, -1, 0);
			}
			if(g_hCPDialog){
				if(codeNotify != CHN_DEL_WITH_GROUP){
					//refresh control panel only when delete note (not group in whole)
					SendMessageW(g_hCPDialog, CHM_CTRL_DEL, 0, (LPARAM)hwnd);
					SendMessageW(g_hCPDialog, CHM_RELOAD, 0, 0);
				}
				SendMessageW(g_hCPDialog, CHM_UPDATE_CTRL_STATUS, 0, 0);
			}
		}
		break;
	}
	case IDM_EMAIL:
		SendNoteByEmail(hwnd);
		break;
	case IDM_ON_TOP:
		pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
		pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
		
		if(codeNotify == 0){
			result = GetMenuState(g_hNotePopUp, IDM_ON_TOP, MF_BYCOMMAND);
			if((result & MF_CHECKED) != MF_CHECKED){
				changed = TRUE;
				CheckMenuItem(g_hNotePopUp, IDM_ON_TOP, MF_CHECKED | MF_BYCOMMAND);
				pD->onTop = TRUE;
				pD->prevOnTop = TRUE;
				SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
			}
			else{
				changed = TRUE;
				CheckMenuItem(g_hNotePopUp, IDM_ON_TOP, MF_UNCHECKED | MF_BYCOMMAND);
				pD->onTop = FALSE;
				pD->prevOnTop = FALSE;
				SetWindowPos(hwnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
				SendMessageW(g_hMain, CHM_ON_TOP_CHANGED, 0, 0);
			}
		}
		else{
			CheckMenuItem(g_hNotePopUp, IDM_ON_TOP, MF_CHECKED | MF_BYCOMMAND);
			// pD->onTop = TRUE;
			SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
		}

		if(changed && codeNotify == 0){
			pF->saved = FALSE;
			pF->maskChanged |= F_ONTOP;
			RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);
			if(g_hCPDialog)
				SendMessageW(g_hCPDialog, CHM_CTRL_UPD, 0, (LPARAM)hwnd);
		}
		break;
	case IDC_NOTE_EDIT:
		if(codeNotify == EN_CHANGE && !m_JustCreated){
			pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);			
			pF->maskChanged |= F_TEXT;
			if(pF->saved){
				pF->saved = FALSE;
				RedrawWindow(hwnd, NULL, NULL, RDW_INVALIDATE);
				if(g_hCPDialog)
					SendMessageW(g_hCPDialog, CHM_CTRL_UPD, 0, (LPARAM)hwnd);
			}
		}
		break;
	case IDM_UNDO:
		SendMessageW((HWND)GetPropW(hwnd, PH_EDIT), EM_UNDO, 0, 0);
		break;
	case IDM_REDO:
		SendMessageW((HWND)GetPropW(hwnd, PH_EDIT), EM_REDO, 0, 0);
		break;
	case IDM_CUT:
		SendMessageW((HWND)GetPropW(hwnd, PH_EDIT), WM_CUT, 0, 0);
		break;
	case IDM_COPY:
		SendMessageW((HWND)GetPropW(hwnd, PH_EDIT), WM_COPY, 0, 0);
		break;
	case IDM_PASTE:
		SendMessageW((HWND)GetPropW(hwnd, PH_EDIT), WM_PASTE, 0, 0);
		break;
	case IDM_SELECT_ALL:{
		CHARRANGE	chr;
		chr.cpMin = 0;
		chr.cpMax = -1;
		SendMessageW((HWND)GetPropW(hwnd, PH_EDIT), EM_EXSETSEL, 0, (LPARAM)&chr);
		break;
	}
	case IDM_SEARCH_NOTE:
	case IDM_REPLACE_NOTE:{
		NFIND nf;
		nf.hNote = hwnd;
		nf.iFindReplace = id;
		if(g_hFindDialog == NULL)
			DialogBoxParamW(g_hInstance, MAKEINTRESOURCEW(DLG_FIND_SINGLE), g_hMain, Find_DlgProc, (LPARAM)&nf);
		else
			SendMessageW(g_hFindDialog, CHM_NEW_FIND_PARENT, 0, (LPARAM)&nf);
		break;
	}
	case IDM_DOCK_LEFT:
		pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
		DockNote(hwnd, DOCK_LEFT, pD);
		break;
	case IDM_DOCK_TOP:
		pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
		DockNote(hwnd, DOCK_TOP, pD);
		break;
	case IDM_DOCK_RIGHT:
		pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
		DockNote(hwnd, DOCK_RIGHT, pD);
		break;
	case IDM_DOCK_BOTTOM:
		pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
		DockNote(hwnd, DOCK_BOTTOM, pD);
		break;
	case IDM_DOCK_NONE:
		pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
		UndockNote(hwnd, pD);
		break;
	case IDM_ADD_TO_FAVORITES:
		AddToFavorites(hwnd);
		break;
	case IDM_FORMAT_FONT_FAMILY:{
		P_NCMD_BAR			pC;
		CBCREATION 			cbr;

		pC = (P_NCMD_BAR)GetWindowLongW(hwnd, OFFSET_COMMANDS);
		cbr.hParent = g_hMain;
		cbr.hButton = pC->hCmd;
		cbr.hEdit = (HWND)GetPropW(hwnd, PH_EDIT);
		SetCurrentFontFace((HWND)GetPropW(hwnd, PH_EDIT));
		CreateComboDialog(&cbr, g_hInstance);
		break;
	}
	case IDM_FORMAT_FONT_BOLD:
	case IDM_FORMAT_FONT_ITALIC:
	case IDM_FORMAT_FONT_UNDER:
	case IDM_FORMAT_FONT_STRIKE:
	case IDM_FORMAT_FONT_HILIT:
	case IDM_FORMAT_ALIGN_LEFT:
	case IDM_FORMAT_ALIGN_CENTER:
	case IDM_FORMAT_ALIGN_RIGHT:
	case IDM_FORMAT_BULLETS:
		
		SendMessageW((HWND)GetPropW(hwnd, PH_COMMANDS), WM_COMMAND, MAKEWPARAM(id, 0), 0);
		break;
	default:
		if(codeNotify == 0 && hwndCtl == 0){	
			if(id >= IDI_FORMAT_COLOR_START && id <= IDI_FORMAT_COLOR_END){
				m_CurrColorsChoice = CC_TEXT;
				ColorChanged(hwnd, id);
			}
			else if(id >= IDI_FORMAT_HIGHLIGHT_START && id <= IDI_FORMAT_HIGHLIGHT_END){
				m_CurrColorsChoice = CC_HIGHLIGHT;
				ColorChanged(hwnd, id);
			}
			else if(id > IDI_SIZE_START && id <= IDI_SIZE_START + MAX_FONT_SIZE){
				//size menu
				SetREFontSize(hwnd, id - IDI_SIZE_START);
				SetFocus(hwnd);
				//if(id - IDI_SIZE_START != GetCFSize()){
					//SetREFontSize(hwnd, id - IDI_SIZE_START);
					//SetFocus(hwnd);
				//}
			}
			else{	//from groups menu
				HMENU			hGroups;
				NOTE_DATA		data;

				pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
				pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
				hGroups = GetSubMenu(g_hNotePopUp, GetMenuPosition(g_hNotePopUp, IDM_NOTE_GROUPS));	
				if((GetMenuState(hGroups, id, MF_BYCOMMAND) & MFS_CHECKED) != MFS_CHECKED){
					pD->idGroup = id - GROUP_ADDITION;
					if(pF->fromDB){
						if(GetPrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile)){
							data.idGroup = pD->idGroup;
							WritePrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile);
						}
					}
					if(g_hCPDialog){
						SendMessageW(g_hCPDialog, CHM_RELOAD, 0, 0);
					}
				}
			}
		}
		else{
			DefWindowProcW(hwnd, WM_COMMAND, MAKEWPARAM(codeNotify, id), (LPARAM)hwndCtl);
		}
	}
}

static void Note_OnClose(HWND hwnd)
{
	P_NOTE_RTHANDLES	pH;
	P_NOTE_APPEARANCE	pA;

	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hwnd, OFFSET_APPEARANCE);

	if(pH->idTimer != 0){
		KillTimer(hwnd, pH->idTimer);
	}
	if(pH->hbBackground)
		DeleteObject(pH->hbBackground);
	if((pA->nPrivate & F_SKIN) == F_SKIN){
		DeleteObject(pH->hbSkin);
		DeleteObject(pH->hbSys);
		DeleteObject(pH->hbPattern);
		DeleteObject(pH->hbDelHide);
	}
	if((pA->nPrivate & F_C_FONT) == F_C_FONT)
		DeleteObject(pH->hFCaption);
	//delete commands bitmap for skinless note
	if(!pH->hbSkin){
		DeleteObject(pH->hbCommand);
		DeleteObject(pH->hbDelHide);
	}
	free(pH);
	free(pA);
	free((P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS));
	free((P_SCHEDULE_TYPE)GetWindowLongW(hwnd, OFFSET_SCHEDULE));
	free((P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA));
	EnumPropsW(hwnd, PropEnumProc);
	free((P_NCMD_BAR)GetWindowLongW(hwnd, OFFSET_COMMANDS));
	free((P_NCMD_BAR)GetWindowLongW(hwnd, OFFSET_DH));
	DestroyWindow(hwnd);
}

static BOOL Note_OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct)
{
	CenterWindow(hwnd, FALSE);
	return TRUE;
}

static void Note_OnPaint(HWND hwnd)
{
	P_NOTE_RTHANDLES	pH;
	P_NOTE_DATA			pD;
	P_NOTE_APPEARANCE	pA;
	PAINTSTRUCT			ps;

	BeginPaint(hwnd, &ps);
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hwnd, OFFSET_APPEARANCE);
	if(pH->hbSkin){
		if(DockType(pD->dockData) == DOCK_NONE){
			PaintSkinnableWindow(hwnd, ps.hdc, pH);
		}
		else{
			if(g_DockSettings.fCustSkin){
				PaintSkinnableWindow(hwnd, ps.hdc, &g_DRTHandles);
			}
			else{
				if((pA->nPrivate & F_SKIN) != F_SKIN){
					PaintSkinnableWindow(hwnd, ps.hdc, pH);
				}
				else{
					PaintSkinnableWindow(hwnd, ps.hdc, &g_DRTHandles);
				}
			}
		}
	}
	else
		PaintSimpleWindow(hwnd, ps.hdc, pH);
	EndPaint(hwnd, &ps);
}

static void Note_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags)
{
	P_NOTE_FLAGS		pF;
	P_NOTE_RTHANDLES	pH;
	P_NOTE_DATA			pD;
	POINT				pt;
	BOOL				dragIt = FALSE;

	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);

	if(keyFlags == MK_LBUTTON){
		pt.x = x;
		pt.y = y;
		if(pH->hbSkin){
			if(pF->inDrag){
				dragIt = TRUE;
			}
		}
		else{
			if(PtInRect(&pH->rcCaption, pt) && pF->inDrag){
				dragIt = TRUE;
			}
		}
		//drag full window
		if(dragIt && DockType(pD->dockData) == DOCK_NONE){
			ReleaseCapture();
			SystemParametersInfoW(SPI_SETDRAGFULLWINDOWS, TRUE, 0, 0);
			SendMessage(hwnd, WM_NCLBUTTONDOWN, HTCAPTION, 0);
			SystemParametersInfoW(SPI_SETDRAGFULLWINDOWS, m_fFullDrag, 0, 0);
			if(pF->fromDB)
				WriteNotePlacement(hwnd, TRUE);
			else
				WriteNotePlacement(hwnd, FALSE);
		}
	}
	else if(keyFlags == 0 && !pF->mouseIn){
		if(DockType(pD->dockData) != DOCK_NONE){
			int result = ArrowRequired(hwnd, DockType(pD->dockData), pD->dockData);
			if(result != -1){
				HWND hTemp;
				switch(result){
				case DA_LEFT_UP:
					hTemp = g_DArrows.hlu;
					break;
				case DA_LEFT_DOWN:
					hTemp = g_DArrows.hld;
					break;
				case DA_TOP_LEFT:
					hTemp = g_DArrows.htl;
					break;
				case DA_TOP_RIGHT:
					hTemp = g_DArrows.htr;
					break;
				case DA_RIGHT_UP:
					hTemp = g_DArrows.hru;
					break;
				case DA_RIGHT_DOWN:
					hTemp = g_DArrows.hrd;
					break;
				case DA_BOTTOM_LEFT:
					hTemp = g_DArrows.hbl;
					break;
				case DA_BOTTOM_RIGHT:
					hTemp = g_DArrows.hbr;
					break;
				}
				// ShowWindow(hTemp, SW_SHOWNORMAL);
				SetWindowPos(hTemp, HWND_TOPMOST, 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
				pF->mouseIn = TRUE;
				TRACKMOUSEEVENT	tm;
				tm.cbSize = sizeof(tm);
				tm.hwndTrack = hwnd;
				tm.dwHoverTime = 0;
				tm.dwFlags = TME_LEAVE;
				TrackMouseEvent(&tm);

			}
		}
	}
}

static int ArrowRequired(HWND hwnd, dock_type dockType, int dockData){
	RECT			rcDesktop, rc, rcTray, rcReal;
	int				mcount, index;
	short 			count = DHeader(dockType)->count;
	HWND			hTray;

	GetWindowRect(hwnd, &rc);
	GetClientRect(GetDesktopWindow(), &rcDesktop);
	mcount = GetSystemMetrics(SM_CMONITORS);
	if(mcount > 1){
		rcDesktop.right = GetSystemMetrics(SM_CXVIRTUALSCREEN);
		rcDesktop.bottom = GetSystemMetrics(SM_CYVIRTUALSCREEN);
	}
	hTray = FindWindow("Shell_TrayWnd", NULL);
	GetWindowRect(hTray, &rcTray);
	SubtractRect(&rcReal, &rcDesktop, &rcTray);

	index = DockIndex(dockData);
	switch(dockType){
	case DOCK_LEFT:
		if(rc.top < rcReal.top){
			return DA_LEFT_UP;
		}
		else if(rc.bottom > rcReal.bottom){
			return DA_LEFT_DOWN;
		}
		else if(index < count - 1 && rc.bottom == rcReal.bottom){
			return DA_LEFT_DOWN;
		}
		else if(index > 0 && rc.top == rcReal.top){
			return DA_LEFT_UP;
		}
		break;
	case DOCK_TOP:
		if(rc.left < rcReal.left){
			return DA_TOP_LEFT;
		}
		else if(rc.right > rcReal.right){
			return DA_TOP_RIGHT;
		}
		else if(index < count - 1 && rc.right == rcReal.right){
			return DA_TOP_RIGHT;
		}
		else if(index > 0 && rc.left == rcReal.left){
			return DA_TOP_LEFT;
		}
		break;
	case DOCK_RIGHT:
		if(rc.top < rcReal.top){
			return DA_RIGHT_UP;
		}
		else if(rc.bottom > rcReal.bottom){
			return DA_RIGHT_DOWN;
		}
		else if(index < count - 1 && rc.bottom == rcReal.bottom){
			return DA_RIGHT_DOWN;
		}
		else if(index > 0 && rc.top == rcReal.top){
			return DA_RIGHT_UP;
		}
		break;
	case DOCK_BOTTOM:
		if(rc.left < rcReal.left){
			return DA_BOTTOM_LEFT;
		}
		else if(rc.right > rcReal.right){
			return DA_BOTTOM_LEFT;
		}
		else if(index < count - 1 && rc.right == rcReal.right){
			return DA_BOTTOM_RIGHT;
		}
		else if(index > 0 && rc.left == rcReal.left){
			return DA_BOTTOM_LEFT;
		}
		break;
	}
	
	return -1;
}

static void OnResizeNote(HWND hwnd, LPRECT lpRc){

	RECT				rc;
	P_NOTE_FLAGS 		pF;
	P_NOTE_RTHANDLES	pH;
	P_NOTE_DATA			pD;

	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);

	if(pH && pH->hbSkin){
		if(DockType(pD->dockData) != DOCK_NONE){
			if(g_DockSettings.fCustSkin){
				pH = &g_DRTHandles;
			}
		}
	}

	if((pF->maskChanged & F_SKIN) == F_SKIN)
		CopyRect(&rc, &pH->rcEdit);
	else
		CopyRect(&rc, lpRc);
	//resize edit box when skin is changed
	MoveWindow((HWND)GetPropW(hwnd, PH_EDIT), rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
	//move commands bar
	if(pH && pH->hbSkin)
		SetNoteCommandsBar(hwnd, pH);
}

static void SetNoteCommandsBar(HWND hwnd, P_NOTE_RTHANDLES pH){
	P_NCMD_BAR			pC, pDH;
	int					width, height;
	HRGN				rBase;
	HWND				hToolbar;

	hToolbar = (HWND)GetPropW(hwnd, PH_COMMANDS);
	if(pH->hbSkin){
		rBase = RegionFromSkin(pH->hbCommand, pH->crMask);
		SetWindowRgn(hToolbar, rBase, TRUE);
	}
	if(!pH->vertTbr){
		height = pH->rcCommands.bottom - pH->rcCommands.top;
		width = (pH->rcCommands.right - pH->rcCommands.left) / CMD_TBR_BUTTONS;
	}
	else{
		height = (pH->rcCommands.bottom - pH->rcCommands.top) / CMD_TBR_BUTTONS;
		width = pH->rcCommands.right - pH->rcCommands.left;
	}
	if(IsBitOn(g_NoteSettings.reserved1, SB_HIDETOOLBAR))
		MoveWindow(hToolbar, pH->rcCommands.left, pH->rcCommands.top, pH->rcCommands.right - pH->rcCommands.left, pH->rcCommands.bottom - pH->rcCommands.top, FALSE);
	else
		MoveWindow(hToolbar, pH->rcCommands.left, pH->rcCommands.top, pH->rcCommands.right - pH->rcCommands.left, pH->rcCommands.bottom - pH->rcCommands.top, TRUE);
	pC = (P_NCMD_BAR)GetWindowLongW(hwnd, OFFSET_COMMANDS);
	for(int i = CMD_FONT; i < CMD_TBR_BUTTONS; i++){
		HWND hw = pC->hCmd;
		if(!pH->vertTbr)
			MoveWindow(hw, width * i, 0, width, height, TRUE);		
		else
			MoveWindow(hw, 0, height * i, width, height, TRUE);		
		pC++;
	}
	//set delete/hide toolbar
	hToolbar = (HWND)GetPropW(hwnd, PH_DELHIDE);
	if(pH->hbSkin){
		SetWindowRgn(hToolbar, RegionFromSkin(pH->hbDelHide, pH->crMask), TRUE);
	}
	height = pH->rcDH.bottom - pH->rcDH.top;
	width = (pH->rcDH.right - pH->rcDH.left) / 2;
	MoveWindow(hToolbar, pH->rcDH.left, pH->rcDH.top, pH->rcDH.right - pH->rcDH.left, height, TRUE);
	pDH = (P_NCMD_BAR)GetWindowLongW(hwnd, OFFSET_DH);
	for(int i = 0; i < CMD_SYS_BUTTONS; i++){
		MoveWindow(pDH->hCmd, width * i, 0, width, height, TRUE);	
		pDH++;
	}
}

static LRESULT CALLBACK Grip_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam){

	switch(msg){
	HANDLE_MSG (hwnd, WM_MOUSEMOVE, Grip_OnMouseMove);

	case WM_PAINT:
	{
		PAINTSTRUCT			ps;
		P_NOTE_APPEARANCE	pA;
		P_NOTE_DATA			pD;

		BeginPaint(hwnd, &ps);
		pD = (P_NOTE_DATA)GetWindowLongW(GetParent(hwnd), OFFSET_DATA);
		pA = (P_NOTE_APPEARANCE)GetWindowLongW(GetParent(hwnd), OFFSET_APPEARANCE);
		if(DockType(pD->dockData) != DOCK_NONE && g_DockSettings.fCustColor)
			PaintGrip(hwnd, ps.hdc, g_DockSettings.crWindow);
		else
			PaintGrip(hwnd, ps.hdc, pA->crWindow);
		EndPaint(hwnd, &ps);
		return FALSE;
	}
	default:
		return DefWindowProcW(hwnd, msg, wParam, lParam);
	}
}

static void Grip_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags)
{
	P_NOTE_DATA			pD;
	P_NOTE_FLAGS		pF;

	pD = (P_NOTE_DATA)GetWindowLongW(GetParent(hwnd), OFFSET_DATA);
	pF = (P_NOTE_FLAGS)GetWindowLongW(GetParent(hwnd), OFFSET_FLAGS);

	if(keyFlags == MK_LBUTTON && DockType(pD->dockData) == DOCK_NONE){
		ReleaseCapture();
		SendMessageW(GetParent(hwnd), WM_NCLBUTTONDOWN, HTBOTTOMRIGHT, 0);
		if(pF->fromDB)
			WriteNotePlacement(GetParent(hwnd), TRUE);
		else
			WriteNotePlacement(GetParent(hwnd), FALSE);
	}
}

static void ShowHideNoteToolbar(HWND hNote){
	P_NCMD_BAR			pC;

	pC = (P_NCMD_BAR)GetWindowLongW(hNote, OFFSET_COMMANDS);
	if(pC){
		if(!IsBitOn(g_NoteSettings.reserved1, SB_HIDETOOLBAR))
			ShowWindow((HWND)GetPropW(hNote, PH_COMMANDS), SW_SHOW);
	}
}

static LRESULT CALLBACK Edit_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	P_NOTE_FLAGS 		pF;
	P_NOTE_RTHANDLES	pH;
	P_NCMD_BAR			pC;

	switch(msg)
	{
		HANDLE_MSG (hwnd, WM_RBUTTONUP, Edit_OnRButtonUp);

		case WM_KILLFOCUS:
			if((HWND)wParam != GetParent(hwnd)){
				if(g_NoteSettings.transAllow){
					//return transparency
					pF = (P_NOTE_FLAGS)GetWindowLongW(GetParent(hwnd), OFFSET_FLAGS);
					pH = (P_NOTE_RTHANDLES)GetWindowLongW(GetParent(hwnd), OFFSET_HANDLES);
					SetLayeredWindowAttributes(GetParent(hwnd), pH->crMask, g_NoteSettings.transValue, LWA_COLORKEY | LWA_ALPHA);
					pF->transparent = TRUE;
				}
				pC = (P_NCMD_BAR)GetWindowLongW(GetParent(hwnd), OFFSET_COMMANDS);
				if(pC){
					if(!g_hComboDialog || (g_hComboDialog && (HWND)wParam != g_hComboDialog)){
						if(!IsBitOn(g_NoteSettings.reserved1, SB_HIDETOOLBAR))
							ShowWindow((HWND)GetPropW(GetParent(hwnd), PH_COMMANDS), SW_HIDE);
					}
				}
			}
			return CallWindowProcW((WNDPROC)GetWindowLongW(hwnd, GWL_USERDATA), hwnd, msg, wParam, lParam);
		case WM_SETFOCUS:
			RemoveBell(GetParent(hwnd));
			pF = (P_NOTE_FLAGS)GetWindowLongW(GetParent(hwnd), OFFSET_FLAGS);
			pH = (P_NOTE_RTHANDLES)GetWindowLongW(GetParent(hwnd), OFFSET_HANDLES);
			if(pF->transparent){
				//draw window opaque 
				SetLayeredWindowAttributes(GetParent(hwnd), pH->crMask, 255, LWA_COLORKEY);
				pF->transparent = FALSE;
			}
			ShowHideNoteToolbar(GetParent(hwnd));
			// pC = (P_NCMD_BAR)GetWindowLongW(GetParent(hwnd), OFFSET_COMMANDS);
			// if(pC){
				// if(!IsBitOn(g_NoteSettings.reserved1, SB_HIDETOOLBAR))
					// ShowWindow((HWND)GetPropW(GetParent(hwnd), PH_COMMANDS), SW_SHOW);
			// }
			return CallWindowProcW((WNDPROC)GetWindowLongW(hwnd, GWL_USERDATA), hwnd, msg, wParam, lParam);
		case WM_VSCROLL:
		case WM_MOUSEWHEEL:
			pF = (P_NOTE_FLAGS)GetWindowLongW(GetParent(hwnd), OFFSET_FLAGS);
			pH = (P_NOTE_RTHANDLES)GetWindowLongW(GetParent(hwnd), OFFSET_HANDLES);
			if(pF->transparent){
				//draw window opaque 
				SetLayeredWindowAttributes(GetParent(hwnd), pH->crMask, 255, LWA_COLORKEY);
				pF->transparent = FALSE;
			}
			return CallWindowProcW((WNDPROC)GetWindowLongW(hwnd, GWL_USERDATA), hwnd, msg, wParam, lParam);
		default:
			return CallWindowProcW((WNDPROC)GetWindowLongW(hwnd, GWL_USERDATA), hwnd, msg, wParam, lParam);
	}
}

static void Edit_OnRButtonUp(HWND hwnd, int x, int y, UINT keyFlags)
{
	HMENU			hMenu = g_hEditPopUp;
	CHARRANGE		chr;

	m_hTempMenu = hMenu;
	if(SendMessageW(hwnd, EM_CANUNDO, 0, 0))
		EnableMenuItem(hMenu, IDM_UNDO, MF_BYCOMMAND | MF_ENABLED);
	else
		EnableMenuItem(hMenu, IDM_UNDO, MF_BYCOMMAND | MF_GRAYED);

	if(SendMessageW(hwnd, EM_CANREDO, 0, 0))
		EnableMenuItem(hMenu, IDM_REDO, MF_BYCOMMAND | MF_ENABLED);
	else
		EnableMenuItem(hMenu, IDM_REDO, MF_BYCOMMAND | MF_GRAYED);

	ZeroMemory(&chr, sizeof(chr));
	SendMessageW(hwnd, EM_EXGETSEL, 0, (LPARAM)&chr);
	if(chr.cpMin == chr.cpMax){
		EnableMenuItem(hMenu, IDM_CUT, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(hMenu, IDM_COPY, MF_BYCOMMAND | MF_GRAYED);
	}
	else{
		EnableMenuItem(hMenu, IDM_CUT, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(hMenu, IDM_COPY, MF_BYCOMMAND | MF_ENABLED);
	}

	if(SendMessageW(hwnd, EM_CANPASTE, CF_TEXT, 0) || SendMessageW(hwnd, EM_CANPASTE, CF_UNICODETEXT, 0) || SendMessageW(hwnd, EM_CANPASTE, CF_OEMTEXT, 0))
		EnableMenuItem(hMenu, IDM_PASTE, MF_BYCOMMAND | MF_ENABLED);
	else
		EnableMenuItem(hMenu, IDM_PASTE, MF_BYCOMMAND | MF_GRAYED);

	ShowPopUp(GetParent(hwnd), hMenu);
}

static void ApplyNewSkin(HWND hNote, P_NOTE_RTHANDLES pH){

	RECT				rc;
	BITMAP				bm;

	GetWindowRect(hNote, &rc);
	GetObjectW(pH->hbSkin, sizeof(bm), &bm);
	//resize controls on note window
	SendMessage(hNote, CHM_RESIZE, (WPARAM)&pH->rcEdit, 0);
	//resize note window
	MoveWindow(hNote, rc.left, rc.top, bm.bmWidth, bm.bmHeight, TRUE);
	//update tooltip
	SendMessageW(hNote, CHM_TT_UPDATE, 0, 0);
}

static BOOL ApplyNoteChanges(HWND hwnd){
	HWND				hNote = (HWND)GetWindowLongW(hwnd, GWL_USERDATA);
	P_NOTE_FLAGS 		pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
	P_NOTE_APPEARANCE	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hNote, OFFSET_APPEARANCE);
	P_NOTE_RTHANDLES	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hNote, OFFSET_HANDLES);
	P_SCHEDULE_TYPE		pS = (P_SCHEDULE_TYPE)GetWindowLongW(hNote, OFFSET_SCHEDULE);
	int					wChanged = 0;
	BOOL				fReload = FALSE;

	if((pF->maskChanged & F_RTFONT) == F_RTFONT)
		wChanged |= F_RTFONT;
	if((pF->maskChanged & F_SKIN) == F_SKIN)
		wChanged |= F_SKIN;
	if((pF->maskChanged & F_C_FONT) == F_C_FONT)
		wChanged |= F_C_FONT;
	if((pF->maskChanged & F_C_COLOR) == F_C_COLOR)
		wChanged |= F_C_COLOR;
	if((pF->maskChanged & F_B_COLOR) == F_B_COLOR)
		wChanged |= F_B_COLOR;
	if(ScheduleChanged(hNote))
		wChanged |= F_SCHEDULE;
	if(wChanged){
		//if appearance changed
		if((wChanged & F_SKIN) == F_SKIN 
		|| (wChanged & F_RTFONT) == F_RTFONT 
		|| (wChanged & F_C_FONT) == F_C_FONT
		|| (wChanged & F_C_COLOR) == F_C_COLOR
		|| (wChanged & F_B_COLOR) == F_B_COLOR){
			memcpy(pA, &m_TNAppearance, sizeof(NOTE_APPEARANCE));
		}
		if((wChanged & F_SKIN) == F_SKIN){	//skin changed
			if(pH->crMask == m_TNHandles.crMask){
				if((pA->nPrivate & F_SKIN) == F_SKIN)			
					//if previous skin was from separate bitmap
					GetSkinProperties(hwnd, pH, pA->szSkin, TRUE);
				else
					GetSkinProperties(hwnd, pH, pA->szSkin, FALSE);
				pA->nPrivate |= F_SKIN;
				ApplyNewSkin(hNote, pH);
			}
			else{
				fReload = TRUE;
			}
			pF->saved = FALSE;
		}
		
		if((wChanged & F_C_FONT) == F_C_FONT){
			if((pA->nPrivate & F_C_FONT) == F_C_FONT)
				DeleteObject(pH->hFCaption);
			pH->hFCaption = CreateFontIndirectW(&pA->lfCaption);
			pA->nPrivate |= F_C_FONT;
			if(pF->fromDB){
				SaveNewCFont(hNote, &pA->lfCaption);
			}
			else{
				pF->saved = FALSE;
			}
		}

		if((wChanged & F_C_COLOR) == F_C_COLOR){
			pA->nPrivate |= F_C_COLOR;
			if(pF->fromDB){
				SaveNewCColor(hNote, pA->crCaption);
			}
			else{
				pF->saved = FALSE;
			}
		}

		if((wChanged & F_B_COLOR) == F_B_COLOR){
			pA->nPrivate |= F_B_COLOR;
			if(pF->fromDB){
				SaveNewBColor(hNote, pA->crWindow);
			}
			else{
				pF->saved = FALSE;
			}
		}

		if((wChanged & F_SCHEDULE) == F_SCHEDULE){	//schedule changed
			if(wChanged != F_SCHEDULE){
				pF->maskChanged |= F_SCHEDULE;
				pF->saved = FALSE;
			}
			else{
				//just save new schedule without prompting
				WritePrivateProfileStructW(pF->id, S_SCHEDULE, pS, sizeof(SCHEDULE_TYPE), g_NotePaths.DataFile);
			}
		}

		//recreate toolbar for skinless window
		if((wChanged & F_C_FONT) == F_C_FONT || (wChanged & F_C_COLOR) == F_C_COLOR || (wChanged & F_B_COLOR) == F_B_COLOR) {
			CreateSkinlessToolbarBitmap(hNote, pA, pH, &pH->hbCommand);
			CreateSkinlessSysBitmap(hNote, pA, pH, &pH->hbDelHide);
		}
		//set new richedit background color
		if((wChanged & F_B_COLOR) == F_B_COLOR){
			RichEdit_SetBkgndColor(GetPropW(hNote, PH_EDIT), 0, pA->crWindow);					
		}
		RedrawWindow(hNote, NULL, NULL, RDW_INVALIDATE);
		if(g_hCPDialog)
			SendMessageW(g_hCPDialog, CHM_CTRL_UPD, 0, (LPARAM)hNote);
	}
	return fReload;
}

static BOOL CALLBACK Adjust_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	NMHDR		* nm;

	switch(msg)
	{
	HANDLE_MSG (hwnd, WM_CLOSE, Adjust_OnClose);
	HANDLE_MSG (hwnd, WM_INITDIALOG, Adjust_OnInitDialog);
	HANDLE_MSG (hwnd, WM_COMMAND, Adjust_OnCommand);
	case WM_NOTIFY:
		nm = (NMHDR *)lParam;
		if(nm->code == TCN_SELCHANGE){
			//show/hide controls on tabs selection
			switch(SendDlgItemMessageW(hwnd, IDC_TAB_ADJUST, TCM_GETCURSEL, 0, 0)){
				case 0:
					ShowWindow(m_Dlgs.hSchedule, SW_HIDE);
					ShowWindow(m_Dlgs.hAppearance, SW_SHOW);
					break;
				case 1:
					ShowWindow(m_Dlgs.hSchedule, SW_SHOW);
					ShowWindow(m_Dlgs.hAppearance, SW_HIDE);
					break;
			}
		}
		return TRUE;
	default: return FALSE;
	}
}

static void Adjust_OnClose(HWND hwnd)
{
	g_hAdjust = NULL;
	EndDialog(hwnd, 0);
}

static BOOL Adjust_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	P_NOTE_APPEARANCE	pA;
	P_NOTE_RTHANDLES	pH;
	P_NOTE_DATA			pD;
	RECT				rc;
	wchar_t				szCaption[256];

	//store handle of calling window
	SetWindowLongW(hwnd, GWL_USERDATA, lParam);
	
	m_Dlgs.hAppearance = CreateDialogParamW((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), MAKEINTRESOURCEW(DLG_NOTE_APP), hwnd, NApp_DlgProc, 0); 
	m_Dlgs.hSchedule = CreateDialogParamW((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), MAKEINTRESOURCEW(DLG_NOTE_SCHEDULE), hwnd, NSched_DlgProc, 0); 

	//create schedule part dialogs
	m_Dns.hOnce = CreateDialogParamW((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), MAKEINTRESOURCEW(DNS_ONCE), m_Dlgs.hSchedule, Dns_DlgProc, 0); 
	m_Dns.hEveryDay = CreateDialogParamW((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), MAKEINTRESOURCEW(DNS_EVERYDAY), m_Dlgs.hSchedule, Dns_DlgProc, 0); 
	m_Dns.hRepeat = CreateDialogParamW((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), MAKEINTRESOURCEW(DNS_REPEATEVERY), m_Dlgs.hSchedule, Dns_DlgProc, 0); 
	m_Dns.hWeekly = CreateDialogParamW((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), MAKEINTRESOURCEW(DNS_WEEKLY), m_Dlgs.hSchedule, Dns_DlgProc, 0); 
	m_Dns.hAfter = CreateDialogParamW((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), MAKEINTRESOURCEW(DNS_AFTER), m_Dlgs.hSchedule, Dns_DlgProc, 0); 

	//store parameters
	pH = (P_NOTE_RTHANDLES)GetWindowLongW((HWND)lParam, OFFSET_HANDLES);
	pA = (P_NOTE_APPEARANCE)GetWindowLongW((HWND)lParam, OFFSET_APPEARANCE);
	pD = (P_NOTE_DATA)GetWindowLongW((HWND)lParam, OFFSET_DATA);
	memcpy(&m_TNAppearance, pA, sizeof(NOTE_APPEARANCE));
	memcpy(&m_TNHandles, pH, sizeof(NOTE_RTHANDLES));

	AddTabs(hwnd);
	GetClientRect(hwnd, &rc);
	SetSizes(hwnd, rc.right - rc.left, rc.bottom -rc.top);

	if(DockType(pD->dockData) != DOCK_NONE){
		ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_LST_SKIN), SW_HIDE);
		ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_ST_BMP), SW_HIDE);
		ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_CMD_SKNLESS_CFONT), SW_HIDE);
		ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_CMD_SKNLESS_BCOLOR), SW_HIDE);
		ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_ST_ADJUST_DISABLE), SW_SHOW);
	}
	else{
		if(m_TNHandles.hbSkin){
			ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_LST_SKIN), SW_SHOW);
			ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_CMD_SKNLESS_CFONT), SW_HIDE);
			ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_CMD_SKNLESS_BCOLOR), SW_HIDE);
		}
		else{
			ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_LST_SKIN), SW_HIDE);
			ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_CMD_SKNLESS_CFONT), SW_SHOW);
			ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_CMD_SKNLESS_BCOLOR), SW_SHOW);
		}
		ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_ST_BMP), SW_SHOW);
		ShowWindow(GetDlgItem(m_Dlgs.hAppearance, IDC_ST_ADJUST_DISABLE), SW_HIDE);
	}


	InitAppearancePart(m_Dlgs.hAppearance);
	InitSchedulePart(m_Dlgs.hSchedule);

	//enable "Standard view" button
	if((pA->nPrivate & F_SKIN) == F_SKIN 
	|| (pA->nPrivate & F_RTFONT) == F_RTFONT 
	|| (pA->nPrivate & F_C_COLOR) == F_C_COLOR
	|| (pA->nPrivate & F_C_FONT) == F_C_FONT
	|| (pA->nPrivate & F_B_COLOR) == F_B_COLOR){
		if(DockType(pD->dockData) == DOCK_NONE){
			EnableWindow(GetDlgItem(hwnd, IDC_CMD_STD_VIEW), TRUE);
		}
		else{
			EnableWindow(GetDlgItem(hwnd, IDC_CMD_STD_VIEW), FALSE);
		}
	}
	else{
		EnableWindow(GetDlgItem(hwnd, IDC_CMD_STD_VIEW), FALSE);
	}

	GetWindowTextW(hwnd, szCaption, 128);
	
	wcscat(szCaption, L" - ");
	wcscat(szCaption, pD->szName);
	SetWindowTextW(hwnd, szCaption);

	//prevent another window appearance
	g_hAdjust = hwnd;

	return FALSE;
}

static void Adjust_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	HWND				hNote = (HWND)GetWindowLongW(hwnd, GWL_USERDATA);

	switch(id){
		case IDCANCEL:
			ClearOnAdjustClose(hNote);
			SendMessageW(hwnd, WM_CLOSE, 0, 0);
			break;
		case IDOK:
			m_Reload = ApplyNoteChanges(hwnd);
			if(m_Reload){
				//the note should be reloaded
				HWND 			hNote = (HWND)GetWindowLongW(hwnd, GWL_USERDATA);
				HWND			hParent = GetParent(hNote);
				P_NOTE_FLAGS	pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
				wchar_t			szID[28];

				wcscpy(szID, pF->id);
				SaveNote(hNote, pF);
				SendMessageW(hNote, WM_CLOSE, 0, 0);
				CreateNoteWindow((HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE), hParent, &g_RTHandles, szID, FALSE, NULL);
				if(g_hCPDialog)
					SendMessageW(g_hCPDialog, CHM_RELOAD, 0, 0);
			}
			SendMessageW(hwnd, WM_CLOSE, 0, 0);
			break;
		case IDC_CMD_STD_VIEW:
			RestoreStandardView(hNote);
			SendMessageW(hwnd, WM_CLOSE, 0, 0);
			break;
	}
}

static void ClearOnAdjustClose(HWND hNote){

	P_NOTE_RTHANDLES	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hNote, OFFSET_HANDLES);

	if(pH->hFCaption != m_TNHandles.hFCaption)
		DeleteObject(m_TNHandles.hFCaption);
	if(pH->hFont != m_TNHandles.hFont)
		DeleteObject(m_TNHandles.hFont);
	if(pH->hbSkin != m_TNHandles.hbSkin)
		DeleteObject(m_TNHandles.hbSkin);
	if(pH->hbPattern != m_TNHandles.hbPattern)
		DeleteObject(m_TNHandles.hbPattern);
	if(pH->hbDelHide != m_TNHandles.hbDelHide)
		DeleteObject(m_TNHandles.hbDelHide);
	if(pH->hbSys != m_TNHandles.hbSys)
		DeleteObject(m_TNHandles.hbSys);
}

/*-@@+@@------------------------------------------------------------------
 Procedure: RestoreStandardView
 Created  : Sun Sep  9 12:36:00 2007
 Modified : Sun Sep  9 12:36:00 2007

 Synopsys : Restores standard view for note
 Input    : hNote - note handle
 Output   : Nothing
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/
static void RestoreStandardView(HWND hNote){
	P_NOTE_RTHANDLES		pH = (P_NOTE_RTHANDLES)GetWindowLongW(hNote, OFFSET_HANDLES);
	P_NOTE_APPEARANCE		pA = (P_NOTE_APPEARANCE)GetWindowLongW(hNote, OFFSET_APPEARANCE);
	P_NOTE_FLAGS			pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
	P_NOTE_RTHANDLES		pMainHandles;
	RECT					rc;
	BITMAP					bm;
	HWND					hEdit;
	BOOL					fDeletePrevious = FALSE;

	hEdit = (HWND)GetPropW(hNote, PH_EDIT);
	//copy standard appearance
	if((pA->nPrivate & F_SKIN) == F_SKIN)
		fDeletePrevious = TRUE;
	memcpy(pA, &g_Appearance, sizeof(NOTE_APPEARANCE));
	//clear note's flags
	pF->maskChanged &= ~F_SKIN;
	pF->maskChanged &= ~F_RTFONT;
	pF->maskChanged &= ~F_C_FONT;
	pF->maskChanged &= ~F_C_COLOR;
	pF->maskChanged &= ~F_B_COLOR;
	//get standard run-time handles
	pMainHandles = (P_NOTE_RTHANDLES)SendMessageW(GetParent(hNote), CHM_GET_RT_HANDLES, 0, 0);
	//get standard font and skin
	pH->hFont = pMainHandles->hFont;
	pH->hbSkin = pMainHandles->hbSkin;
	//clear "saved" flag, if note's text has not been changed
	if((pF->maskChanged & F_TEXT) != F_TEXT)
		pF->saved = TRUE;
	if(!pH->hbSkin){
		//in case of skinless window - get caption font
		pH->hFCaption = pMainHandles->hFCaption;
		//prepare needed rectangles
		GetClientRect(hNote, &rc);
		DeleteObject(pH->hbCommand);
		CreateSkinlessToolbarBitmap(hNote, pA, pH, &pH->hbCommand);
		CreateSkinlessSysBitmap(hNote, pA, pH, &pH->hbDelHide);
		MeasureSkinlessNoteRectangles(pH, rc.right - rc.left, rc.bottom - rc.top);
		RichEdit_SetBkgndColor(hEdit, FALSE, pA->crWindow);
	}
	else{
		//get standard skin properties
		GetSkinProperties(hNote, pH, pA->szSkin, fDeletePrevious);
		GetWindowRect(hNote, &rc);
		GetObjectW(pH->hbSkin, sizeof(bm), &bm);
		//resize controls on note window
		SendMessage(hNote, CHM_RESIZE, (WPARAM)&pH->rcEdit, 0);
		//resize note window
		MoveWindow(hNote, rc.left, rc.top, bm.bmWidth, bm.bmHeight, TRUE);
		//update tooltip
		SendMessageW(hNote, CHM_TT_UPDATE, 0, 0);
	}
	//clear private appearance
	pA->nPrivate = 0;
	//remove appearance entry from data file
	WritePrivateProfileStructW(pF->id, S_APPEARANCE, NULL, 0, g_NotePaths.DataFile);
	SetREFontCommon(hEdit, &pA->lfFont);
	SetREFontColorCommon(hEdit, pA->crFont);

	//redraw note
	RedrawWindow(hNote, NULL, NULL, RDW_INVALIDATE);
}

/*-@@+@@------------------------------------------------------------------
 Procedure: ScheduleChanged
 Created  : Sun May 27 15:33:17 2007
 Modified : Sun May 27 15:33:17 2007

 Synopsys : Checks whether schedule of note has been changed
 Input    : hwnd - note handle
 Output   : TRUE if changed, otherwise - FALSE
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/

static BOOL ScheduleChanged(HWND hwnd){

	P_SCHEDULE_TYPE		pS;
	P_NOTE_RTHANDLES	pH;
	SYSTEMTIME			st, stSt;
	int					prevType, weekday;
	int					scType;
	BOOL				translated;

	GetSystemTime(&st);
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	pS = (P_SCHEDULE_TYPE)GetWindowLongW(hwnd, OFFSET_SCHEDULE);
	scType = SendDlgItemMessageW(m_Dlgs.hSchedule, IDC_CBO_NOTE_SCHEDULE, CB_GETCURSEL, 0, 0);

	switch(scType){
		case SCH_NO:					//no schedule checked
			if(pS->scType != 0){		//note was scheduled before
				pS->scType = 0;			//remove scheduling
				if(pH->idTimer)			//stop timer
					KillTimer(hwnd, pH->idTimer);
				return TRUE;
			}
			break;
		case SCH_ONCE:					//schedule once checked
			if(pS->scType != SCH_ONCE){		//different schedule type before
				prevType = pS->scType;
				pS->scType = SCH_ONCE;
				//get selected time
				SendDlgItemMessageW(m_Dns.hOnce, IDC_DTP_ONCE, DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
				//copy time
				memcpy(&pS->scDate, &st, sizeof(SYSTEMTIME));
				if(prevType == SCH_NO){		//no timer before - now set timer
					SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
					SetTimer(hwnd, 1, TIMER_INTERVAL, TimerProc);
					pH->idTimer = 1;
				}
				return TRUE;
			}
			else{			//the previous scheduling was of the same type - must check times difference
				//get selected time
				SendDlgItemMessageW(m_Dns.hOnce, IDC_DTP_ONCE, DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
				if(pS->scDate.wDay != st.wDay || pS->scDate.wMonth != st.wMonth
					|| pS->scDate.wYear != st.wYear || pS->scDate.wHour != st.wHour 
					|| pS->scDate.wMinute != st.wMinute){

					//copy time
					memcpy(&pS->scDate, &st, sizeof(SYSTEMTIME));
					return TRUE;
				}
			}
			break;
		case SCH_ED:			//schedule every day checked
			if(pS->scType != SCH_ED){		//different schedule type before
				prevType = pS->scType;
				pS->scType = SCH_ED;
				//get selected time
				SendDlgItemMessageW(m_Dns.hEveryDay, IDC_DTP_EV_DAY, DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
				//copy time
				memcpy(&pS->scDate, &st, sizeof(SYSTEMTIME));
				if(prevType == SCH_NO){		//no timer before - now set timer
					SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
					SetTimer(hwnd, 1, TIMER_INTERVAL, TimerProc);
					pH->idTimer = 1;
				}
				return TRUE;
			}
			else{			//the previous scheduling was of the same type - must check times difference
				//get selected time
				SendDlgItemMessageW(m_Dns.hEveryDay, IDC_DTP_EV_DAY, DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
				if(pS->scDate.wHour != st.wHour || pS->scDate.wMinute != st.wMinute){
					//copy time
					memcpy(&pS->scDate, &st, sizeof(SYSTEMTIME));
					return TRUE;
				}
			}
			break;
		case SCH_REP:					//schedule repeat checked
			if(pS->scType != SCH_REP){		//different schedule before
				prevType = pS->scType;
				pS->scType = SCH_REP;
				//get selected time
				GetPesterPart(&st);
				// SendDlgItemMessageW(hdlg, IDC_DT_REPEAT, DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
				//copy time
				memcpy(&pS->scDate, &st, sizeof(SYSTEMTIME));
				pH->nRepeat = SetRepeatCount(&pS->scDate);		//set repeat timer according to timer interval
				if(prevType == SCH_NO){		//no timer before - now set timer
					SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
					SetTimer(hwnd, 1, TIMER_INTERVAL, TimerProc);
					pH->idTimer = 1;
				}
				return TRUE;
			}
			else{			//the previous scheduling was of the same type - must check times difference
				//get selected time
				GetPesterPart(&st);
				// SendDlgItemMessageW(hdlg, IDC_DT_REPEAT, DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
				if(pS->scDate.wHour != st.wHour || pS->scDate.wMinute != st.wMinute || pS->scDate.wSecond != st.wSecond){
					//copy time
					memcpy(&pS->scDate, &st, sizeof(SYSTEMTIME));
					pH->nRepeat = SetRepeatCount(&pS->scDate);		//set repeat timer according to timer interval
					return TRUE;
				}
			}
			break;
		case SCH_PER_WD:		//weekly schedule checked
			//get weekday
			weekday = SelectedWeekdays();
			if(weekday == 0)
				return TRUE;
			// weekday = SendDlgItemMessageW(hdlg, IDC_CBO_EVERY_WD, CB_GETITEMDATA, SendDlgItemMessageW(hdlg, IDC_CBO_EVERY_WD, CB_GETCURSEL, 0, 0), 0);
			if(pS->scType != SCH_PER_WD){		//different schedule type
				prevType = pS->scType;
				pS->scType = SCH_PER_WD;
				//get selected time
				SendDlgItemMessageW(m_Dns.hWeekly, IDC_DTP_TIME_EVERY_WD, DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
				//store weekday
				st.wDayOfWeek = weekday;
				//copy time
				memcpy(&pS->scDate, &st, sizeof(SYSTEMTIME));
				if(prevType == SCH_NO){		//no timer before - now set timer
					SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
					SetTimer(hwnd, 1, TIMER_INTERVAL, TimerProc);
					pH->idTimer = 1;
				}
				return TRUE;
			}
			else{
				//get selected time
				SendDlgItemMessageW(m_Dns.hWeekly, IDC_DTP_TIME_EVERY_WD, DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
				if(pS->scDate.wDayOfWeek != weekday || pS->scDate.wHour != st.wHour || pS->scDate.wMinute != st.wMinute){
					st.wDayOfWeek = weekday;
					//copy time
					memcpy(&pS->scDate, &st, sizeof(SYSTEMTIME));
					return TRUE;
				}
			}
			break;
		case SCH_AFTER:				//'after' schedule checked
			if(pS->scType != SCH_AFTER){	//different schedule type
				prevType = pS->scType;
				pS->scType = SCH_AFTER;
				//get selected time
				SendDlgItemMessageW(m_Dns.hAfter, IDC_DTP_AFTER, DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
				//copy start time
				memcpy(&pS->scStart, &st, sizeof(SYSTEMTIME));				
				//store date parts - wDayOfWeek is used for weeks count
				pS->scDate.wYear = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_YEARS, &translated, FALSE);
				pS->scDate.wMonth = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_MONTHS, &translated, FALSE);
				pS->scDate.wDayOfWeek = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_WEEKS, &translated, FALSE);
				pS->scDate.wDay = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_DAYS, &translated, FALSE);
				pS->scDate.wHour = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_HOURS, &translated, FALSE);
				pS->scDate.wMinute = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_MINS, &translated, FALSE);
				pS->scDate.wSecond = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_SECS, &translated, FALSE);
				if(prevType == SCH_NO){		//no timer before - now set timer
					SetPropW(hwnd, PH_ALARM, (HANDLE)TRUE);
					SetTimer(hwnd, 1, TIMER_INTERVAL, TimerProc);
					pH->idTimer = 1;
				}
				return TRUE;
			}
			else{
				//get selected time
				SendDlgItemMessageW(m_Dns.hAfter, IDC_DTP_AFTER, DTM_GETSYSTEMTIME, 0, (LPARAM)&st);
				//store date parts in temporary variable - wDayOfWeek is used for weeks count
				stSt.wYear = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_YEARS, &translated, FALSE);
				stSt.wMonth = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_MONTHS, &translated, FALSE);
				stSt.wDayOfWeek = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_WEEKS, &translated, FALSE);
				stSt.wDay = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_DAYS, &translated, FALSE);
				stSt.wHour = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_HOURS, &translated, FALSE);
				stSt.wMinute = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_MINS, &translated, FALSE);
				stSt.wSecond = GetDlgItemInt(m_Dns.hAfter, IDC_EDT_SECS, &translated, FALSE);
				if(pS->scStart.wYear != st.wYear || pS->scStart.wMonth != st.wMonth 
				|| pS->scStart.wDayOfWeek != st.wDayOfWeek || pS->scStart.wDay != st.wDay 
				|| pS->scStart.wHour != st.wHour || pS->scStart.wMinute != st.wMinute 
				|| pS->scStart.wSecond != st.wSecond 
				|| pS->scDate.wYear != stSt.wYear || pS->scDate.wMonth != stSt.wMonth 
				|| pS->scDate.wDayOfWeek != stSt.wDayOfWeek || pS->scDate.wDay != stSt.wDay 
				|| pS->scDate.wHour != stSt.wHour || pS->scDate.wMinute != stSt.wMinute 
				|| pS->scDate.wSecond != stSt.wSecond){
					//copy start time
					memcpy(&pS->scStart, &st, sizeof(SYSTEMTIME));
					//copy dates parts
					memcpy(&pS->scDate, &stSt, sizeof(SYSTEMTIME));
					return TRUE;
				}	
			}
			break;
	}
	return FALSE;
}

static void FillScheduleTypes(HWND hwnd, wchar_t * lpFile){
	wchar_t 		szBuffer[256], szId[12];
	
	_itow(IDC_OPT_NO_SCH, szId, 10);
	GetPrivateProfileStringW(S_OPTIONS, szId, L"No schedule", szBuffer, 256, lpFile);
	SendDlgItemMessageW(hwnd, IDC_CBO_NOTE_SCHEDULE, CB_ADDSTRING, 0, (LPARAM)szBuffer);
	_itow(IDC_OPT_ONCE, szId, 10);
	GetPrivateProfileStringW(S_OPTIONS, szId, L"Once at:", szBuffer, 256, lpFile);
	SendDlgItemMessageW(hwnd, IDC_CBO_NOTE_SCHEDULE, CB_ADDSTRING, 0, (LPARAM)szBuffer);
	_itow(IDC_OPT_EV_DAY, szId, 10);
	GetPrivateProfileStringW(S_OPTIONS, szId, L"Every day at:", szBuffer, 256, lpFile);
	SendDlgItemMessageW(hwnd, IDC_CBO_NOTE_SCHEDULE, CB_ADDSTRING, 0, (LPARAM)szBuffer);
	_itow(IDC_OPT_REPEAT, szId, 10);
	GetPrivateProfileStringW(S_OPTIONS, szId, L"Repeat every:", szBuffer, 256, lpFile);
	SendDlgItemMessageW(hwnd, IDC_CBO_NOTE_SCHEDULE, CB_ADDSTRING, 0, (LPARAM)szBuffer);
	_itow(IDC_OPT_EVERY_WD, szId, 10);
	GetPrivateProfileStringW(S_OPTIONS, szId, L"Weekly on:", szBuffer, 256, lpFile);
	SendDlgItemMessageW(hwnd, IDC_CBO_NOTE_SCHEDULE, CB_ADDSTRING, 0, (LPARAM)szBuffer);
	_itow(IDC_OPT_AFTER, szId, 10);
	GetPrivateProfileStringW(S_OPTIONS, szId, L"After:", szBuffer, 256, lpFile);
	SendDlgItemMessageW(hwnd, IDC_CBO_NOTE_SCHEDULE, CB_ADDSTRING, 0, (LPARAM)szBuffer);
}

static void AddTabs(HWND hwnd){

	TCITEMW		tci;
	wchar_t 	szBuffer[256], szFile[MAX_PATH];

	wcscpy(szFile, g_NotePaths.LangDir);
	wcscat(szFile, g_NotePaths.LangFile);

	//set dialog and controls text
	GetPrivateProfileStringW(S_OPTIONS, __T("1003"), DS_CAP_SINGLE, szBuffer, 256, szFile);
	SetWindowTextW(hwnd, szBuffer);
	SetDlgCtlText(hwnd, IDOK, szFile, DS_OK);
	SetDlgCtlText(hwnd, IDCANCEL, szFile, DS_CANCEL);
	SetDlgCtlText(hwnd, IDC_CMD_STD_VIEW, szFile, L"Standard view");
	//tabs
	ZeroMemory(&tci, sizeof(tci));
	tci.mask = TCIF_TEXT | TCIF_PARAM;
	tci.pszText = szBuffer;
	tci.cchTextMax = 256;
	GetPrivateProfileStringW(__T("tabs"), __T("0"), __T("Skin"), szBuffer, 256, szFile);
	SendDlgItemMessageW(hwnd, IDC_TAB_ADJUST, TCM_INSERTITEMW, 0, (LPARAM)&tci);
	GetPrivateProfileStringW(__T("tabs"), __T("2"), __T("Schedule"), szBuffer, 256, szFile);
	SendDlgItemMessageW(hwnd, IDC_TAB_ADJUST, TCM_INSERTITEMW, 1, (LPARAM)&tci);

	//appearance part
	SetDlgCtlText(m_Dlgs.hAppearance, IDC_ST_ADJUST_DISABLE, szFile, L"Settings are available for not docked notes only");

	//-----------	schedule part	-----------
	//'repeat' dialog
	SetDlgCtlText(m_Dns.hRepeat, IDC_ST_DAYS, szFile, L"Days");
	SetDlgCtlText(m_Dns.hRepeat, IDC_ST_HOURS, szFile, L"Hours");
	SetDlgCtlText(m_Dns.hRepeat, IDC_ST_MINS, szFile, L"Minutes");
	SetDlgCtlText(m_Dns.hRepeat, IDC_ST_SECS, szFile, L"Seconds");
	//'after' dialog
	SetDlgCtlText(m_Dns.hAfter, IDC_ST_DAYS, szFile, L"Days");
	SetDlgCtlText(m_Dns.hAfter, IDC_ST_HOURS, szFile, L"Hours");
	SetDlgCtlText(m_Dns.hAfter, IDC_ST_MINS, szFile, L"Minutes");
	SetDlgCtlText(m_Dns.hAfter, IDC_ST_SECS, szFile, L"Seconds");
	SetDlgCtlText(m_Dns.hAfter, IDC_ST_YEARS, szFile, L"Years");
	SetDlgCtlText(m_Dns.hAfter, IDC_ST_MONTHS, szFile, L"Months");
	SetDlgCtlText(m_Dns.hAfter, IDC_ST_WEEKS, szFile, L"Weeks");
	SetDlgCtlText(m_Dns.hAfter, IDC_ST_STARTING, szFile, L"Starting from:");
	//'weekly' dialog
	SetDlgCtlText(m_Dns.hWeekly, IDC_ST_TIME_EVERY_WD, szFile, L"At:");
	//format datetime pickers
	SendDlgItemMessageW(m_Dns.hOnce, IDC_DTP_ONCE, DTM_SETFORMATW, 0, (LPARAM)g_DTFormats.DateFormat);
	SendDlgItemMessageW(m_Dns.hEveryDay, IDC_DTP_EV_DAY, DTM_SETFORMATW, 0, (LPARAM)g_DTFormats.TimeFormat);
	SendDlgItemMessageW(m_Dns.hWeekly, IDC_DTP_TIME_EVERY_WD, DTM_SETFORMATW, 0, (LPARAM)g_DTFormats.TimeFormat);
	SendDlgItemMessageW(m_Dns.hAfter, IDC_DTP_AFTER, DTM_SETFORMATW, 0, (LPARAM)g_DTFormats.DateFormat);
	//prepare textboxes for 'repeat' dialog
	for(int i = IDC_EDT_DAYS; i <= IDC_EDT_SECS; i++)
		SendDlgItemMessageW(m_Dns.hRepeat, i, EM_LIMITTEXT, 2, 0);
	//prepare textboxes for 'after' dialog
	for(int i = IDC_EDT_DAYS; i <= IDC_EDT_SECS; i++)
		SendDlgItemMessageW(m_Dns.hAfter, i, EM_LIMITTEXT, 2, 0);
	for(int i = IDC_EDT_YEARS; i <= IDC_EDT_WEEKS; i++)
		SendDlgItemMessageW(m_Dns.hAfter, i, EM_LIMITTEXT, 2, 0);
	//prepare days of week
	FillDaysOfWeek(szFile);
	
	FillScheduleTypes(m_Dlgs.hSchedule, szFile);
}

static void FillDaysOfWeek(wchar_t * lpLang){
	wchar_t			szBuffer[128];

	GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_IFIRSTDAYOFWEEK, szBuffer, 128);
	switch(_wtoi(szBuffer)){
		case 0:
			for(int i = IDC_CHK_WD_0, j = 0; i <= IDC_CHK_WD_6; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			break;
		case 1:
			for(int i = IDC_CHK_WD_0, j = 1; i <= IDC_CHK_WD_5; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			SetWeekdayText(IDC_CHK_WD_6, 0, lpLang);
			break;
		case 2:
			for(int i = IDC_CHK_WD_0, j = 2; i <= IDC_CHK_WD_4; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			for(int i = IDC_CHK_WD_5, j = 0; i <= IDC_CHK_WD_6; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			break;
		case 3:
			for(int i = IDC_CHK_WD_0, j = 3; i <= IDC_CHK_WD_3; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			for(int i = IDC_CHK_WD_4, j = 0; i <= IDC_CHK_WD_6; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			break;
		case 4:
			for(int i = IDC_CHK_WD_0, j = 4; i <= IDC_CHK_WD_2; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			for(int i = IDC_CHK_WD_3, j = 0; i <= IDC_CHK_WD_6; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			break;
		case 5:
			for(int i = IDC_CHK_WD_0, j = 5; i <= IDC_CHK_WD_1; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			for(int i = IDC_CHK_WD_2, j = 0; i <= IDC_CHK_WD_6; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			break;
		case 6:
			SetWeekdayText(IDC_CHK_WD_0, 6, lpLang);
			for(int i = IDC_CHK_WD_1, j = 0; i <= IDC_CHK_WD_6; i++, j++){
				SetWeekdayText(i, j, lpLang);
			}
			break;
	}
}

static short SelectedWeekdays(void){
	int			result = 0;

	if(IsDlgButtonChecked(m_Dns.hWeekly, IDC_CHK_WD_0))
		// return 1;
		BitOn(&result, 1);
	if(IsDlgButtonChecked(m_Dns.hWeekly, IDC_CHK_WD_1))
		// return 2;
		BitOn(&result, 2);
	if(IsDlgButtonChecked(m_Dns.hWeekly, IDC_CHK_WD_2))
		// return 3;
		BitOn(&result, 3);
	if(IsDlgButtonChecked(m_Dns.hWeekly, IDC_CHK_WD_3))
		// return 4;
		BitOn(&result, 4);
	if(IsDlgButtonChecked(m_Dns.hWeekly, IDC_CHK_WD_4))
		// return 5;
		BitOn(&result, 5);
	if(IsDlgButtonChecked(m_Dns.hWeekly, IDC_CHK_WD_5))
		// return 6;
		BitOn(&result, 6);
	if(IsDlgButtonChecked(m_Dns.hWeekly, IDC_CHK_WD_6))
		// return 7;
		BitOn(&result, 7);
	return (short)result;
}

static void SetWeekdayText(int id, int day, const wchar_t * lpFile){

	wchar_t 		szBuffer[24], szDefault[24], szId[12];

	_itow(day, szId, 10);
	GetDlgItemTextW(m_Dns.hWeekly, id, szDefault, 24);
	GetPrivateProfileStringW(S_WEEKDAYS, szId, szDefault, szBuffer, 24, lpFile);
	SetDlgItemTextW(m_Dns.hWeekly, id, szBuffer);
}

static void SetPesterPart(LPSYSTEMTIME lps){
	SetDlgItemInt(m_Dns.hRepeat, IDC_EDT_DAYS, lps->wDay, FALSE);
	SetDlgItemInt(m_Dns.hRepeat, IDC_EDT_HOURS, lps->wHour, FALSE);
	SetDlgItemInt(m_Dns.hRepeat, IDC_EDT_MINS, lps->wMinute, FALSE);
	SetDlgItemInt(m_Dns.hRepeat, IDC_EDT_SECS, lps->wSecond, FALSE);
}

static void GetPesterPart(LPSYSTEMTIME lps){
	lps->wDay = GetDlgItemInt(m_Dns.hRepeat, IDC_EDT_DAYS, NULL, FALSE);
	lps->wHour = GetDlgItemInt(m_Dns.hRepeat, IDC_EDT_HOURS, NULL, FALSE);
	lps->wMinute = GetDlgItemInt(m_Dns.hRepeat, IDC_EDT_MINS, NULL, FALSE);
	lps->wSecond = GetDlgItemInt(m_Dns.hRepeat, IDC_EDT_SECS, NULL, FALSE);
}

static void SetSizes(HWND hwnd, int cx, int cy){
	
	RECT		rc, rcd;

	//size and move controls on adjust dialog
	SetRect(&rc, 0, 0, cx, cy);
	MoveWindow(GetDlgItem(hwnd, IDC_TAB_ADJUST), 0, 0, cx, cy, TRUE);
	SendDlgItemMessageW(hwnd, IDC_TAB_ADJUST, TCM_ADJUSTRECT, FALSE, (LPARAM)&rc);
	MoveWindow(GetDlgItem(hwnd, IDC_ST_TAB), rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
	GetWindowRect(GetDlgItem(hwnd, IDC_CMD_STD_VIEW), &rcd);
	MapWindowPoints(HWND_DESKTOP, hwnd, (LPPOINT)&rcd, 2);
	MoveWindow(m_Dlgs.hSchedule, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top - (rcd.bottom - rcd.top + 2 * (rc.bottom - rcd.bottom)), TRUE);
	MoveWindow(m_Dlgs.hAppearance, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top - (rcd.bottom - rcd.top + 2 * (rc.bottom - rcd.bottom)), TRUE);
}

static void SetNoteCaption(HWND hwnd){
	P_NOTE_DATA			pD;
	wchar_t				szText[256];

	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	wcscpy(szText, NOTE_CAPTION_FIRST_PART);
	wcscat(szText, pD->szName);
	wcscat(szText, L"]");
	SetWindowTextW(hwnd, szText);
}

/*-@@+@@------------------------------------------------------------------
 Procedure: PaintGrip
 Created  : Sun Sep  9 13:51:36 2007
 Modified : Sun Sep  9 13:51:36 2007

 Synopsys : Paints "sizeable" grip
 Input    : hwnd - note's handle
            hdcPS - device context
            pA - note's appearance handles
 Output   : Nothing
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/
static void PaintGrip(HWND hwnd, HDC hdcPS, COLORREF crWindow){
	RECT				rc;
	HPEN				hPen, hOldPen;
	POINT				pt;
	HBRUSH				hBrush;

	GetClientRect(hwnd, &rc);
	hBrush = CreateSolidBrush(crWindow);
	FillRect(hdcPS, &rc, hBrush);
	hPen = CreatePen(PS_SOLID, 2, GetSysColor(COLOR_BTNSHADOW));
	hOldPen = SelectObject(hdcPS, hPen);
	MoveToEx(hdcPS, rc.right - 5, rc.bottom + 1, &pt);
	LineTo(hdcPS, rc.right + 1, rc.bottom - 5);
	MoveToEx(hdcPS, rc.right - 10, rc.bottom + 1, &pt);
	LineTo(hdcPS, rc.right + 1, rc.bottom - 10);
	MoveToEx(hdcPS, rc.right - 15, rc.bottom + 1, &pt);
	LineTo(hdcPS, rc.right + 1, rc.bottom - 15);
	hPen = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_WINDOW));
	SelectObject(hdcPS, hPen);
	MoveToEx(hdcPS, rc.right - 6, rc.bottom + 1, &pt);
	LineTo(hdcPS, rc.right + 1, rc.bottom - 6);
	MoveToEx(hdcPS, rc.right - 11, rc.bottom + 1, &pt);
	LineTo(hdcPS, rc.right + 1, rc.bottom - 11);
	MoveToEx(hdcPS, rc.right - 16, rc.bottom + 1, &pt);
	LineTo(hdcPS, rc.right + 1, rc.bottom - 16);
	SelectObject(hdcPS, hOldPen);
	DeleteObject(hPen);
	DeleteObject(hBrush);
}

/*-@@+@@------------------------------------------------------------------
 Procedure: PaintSimpleWindow
 Created  : Sun Sep  9 13:44:08 2007
 Modified : Sun Sep  9 13:44:08 2007

 Synopsys : Paints skinless note
 Input    : hwnd - note's handle
            hdcPS - device context
            pH - note's run-time handles
 Output   : Nothing
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/
static void PaintSimpleWindow(HWND hwnd, HDC hdcPS, P_NOTE_RTHANDLES pH){
	P_NOTE_FLAGS		pF;
	P_SCHEDULE_TYPE		pS;
	P_NOTE_APPEARANCE	pA;
	P_NOTE_DATA			pD;
	RECT				rc, rc1;
	HPEN				hPen, hOldPen;
	HBRUSH				hBrush, hOldBrush, hTextBrush;
	HFONT				hFont, hOldFont;
	COLORREF			crWindow, crCaption;

	//get needed parameters
	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	pS = (P_SCHEDULE_TYPE)GetWindowLongW(hwnd, OFFSET_SCHEDULE);
	pA = (P_NOTE_APPEARANCE)GetWindowLongW(hwnd, OFFSET_APPEARANCE);
	if(DockType(pD->dockData) != DOCK_NONE && g_DockSettings.fCustColor)
		crWindow = g_DockSettings.crWindow;
	else
		crWindow = pA->crWindow;
	if(DockType(pD->dockData) != DOCK_NONE && g_DockSettings.fCustCaption){
		crCaption = g_DockSettings.crCaption;
		hFont = g_DRTHandles.hFCaption;
	}
	else{
		crCaption = pA->crCaption;
		hFont = pH->hFCaption;
	}
	//get note's dimensions
	GetClientRect(hwnd, &rc);
	//create solid brush with needed color
	hBrush = CreateSolidBrush(crWindow);
	//fill note's surface with brush
	FillRect(hdcPS, &rc, hBrush);
	//draw note's caption
	FillSkinlessNoteCaption(hdcPS, &pH->rcCaption, crWindow, &pH->rcGrip);
	//save original brush
	hOldBrush = SelectObject(hdcPS, hBrush);
	//draw caption text
	hTextBrush = CreateSolidBrush(crCaption);
	SetBkMode(hdcPS, TRANSPARENT);
	SetTextColor(hdcPS, crCaption);
	hOldFont = SelectObject(hdcPS, hFont);
	DrawTextW(hdcPS, pD->szName, -1, &pH->rcTextCaption, DT_LEFT | DT_SINGLELINE | DT_VCENTER | DT_END_ELLIPSIS);
	//create solid pen with caption font color
	hPen = CreatePen(PS_SOLID, 1, crCaption);
	hOldPen = SelectObject(hdcPS, hPen);
	if(!pF->saved){
		//draw "changed" sign
		CopyRect(&rc1, &pH->rcNotSaved);
		rc1.left += 4;
		rc1.top += 1;
		FrameRect(hdcPS, &rc1, hTextBrush);
		MoveToEx(hdcPS, pH->rcNotSaved.left, pH->rcNotSaved.top, NULL);
		LineTo(hdcPS, pH->rcNotSaved.left + 7, pH->rcNotSaved.top + 7);
	}
	if(pS->scType != 0){
		//draw "scheduled" sign
		CopyRect(&pH->rcSchedule, &pH->rcNotSaved);
		if(!pF->saved){
			OffsetRect(&pH->rcSchedule, pH->rcNotSaved.right - pH->rcNotSaved.left + 2, 0);
		}
		Ellipse(hdcPS, pH->rcSchedule.left, pH->rcSchedule.top, pH->rcSchedule.right, pH->rcSchedule.bottom);
		MoveToEx(hdcPS, pH->rcSchedule.left + 5, pH->rcSchedule.top + 2, NULL);
		LineTo(hdcPS, pH->rcSchedule.left + 5, pH->rcSchedule.top + 6);
		LineTo(hdcPS, pH->rcSchedule.left + 7, pH->rcSchedule.top + 6);
	}
	//frame note
	FrameSkinlessNote(hdcPS, &rc, crWindow);
	//restore DC
	SelectObject(hdcPS, hOldBrush);
	SelectObject(hdcPS, hOldFont);
	SelectObject(hdcPS, hOldPen);
	DeleteObject(hPen);
	DeleteObject(hBrush);
	DeleteObject(hTextBrush);
}

/*-@@+@@------------------------------------------------------------------
 Procedure: PaintSkinnableWindow
 Created  : Sun Sep  9 13:39:04 2007
 Modified : Sun Sep  9 13:39:04 2007

 Synopsys : Paints skinnable note
 Input    : hwnd - note handle
            hdcPS - device context
            pH - note's run-time handles
 Output   : Nothing
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/
static void PaintSkinnableWindow(HWND hwnd, HDC hdcPS, P_NOTE_RTHANDLES pH){
	P_NOTE_FLAGS		pF;
	P_SCHEDULE_TYPE		pS;
	HDC					hdc, hdcSave;
	HBITMAP				hOriginal, hBmSave, hBmSaveOld;
	RECT				rc;
	// int					xLeft;

	//get note's dimensions
	GetClientRect(hwnd, &rc);
	//get needed parameters
	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	pS = (P_SCHEDULE_TYPE)GetWindowLongW(hwnd, OFFSET_SCHEDULE);
	//save original bitmap
	hOriginal = pH->hbSkin;
	//prepare compatible DC
	hdc = CreateCompatibleDC(hdcPS);
	//set back mode to transparent
	SetBkMode(hdc, TRANSPARENT);
	//select original bitmap in temporary DC
	SelectObject(hdc, hOriginal);
	//create DC to store original bitmap
	hdcSave = CreateCompatibleDC(hdcPS);
	hBmSave = CreateCompatibleBitmap(hdcPS, rc.right - rc.left, rc.bottom - rc.top);
	hBmSaveOld = SelectObject(hdcSave, hBmSave);
	//Save the original bitmap, because it will be overwritten
	BitBlt(hdcSave, 0, 0, rc.right - rc.left, rc.bottom - rc.top, hdc, 0, 0, SRCCOPY);
	//draw "changed" and "scheduled" icons
	if(!pF->saved)
		DrawTransparentBitmap(pH->hbSys, hdc, pH->rcNotSaved.left, pH->rcNotSaved.top, (pH->rcNotSaved.right - pH->rcNotSaved.left + 1), (pH->rcNotSaved.bottom - pH->rcNotSaved.top + 1), (pH->rcSchedule.right - pH->rcSchedule.left + 1), 0, pH->crMask);
	if(pS->scType != 0){
		// xLeft = !pF->saved ? pH->rcSchedule.left : pH->rcNotSaved.left;
		DrawTransparentBitmap(pH->hbSys, hdc, pH->rcSchedule.left, pH->rcSchedule.top, (pH->rcSchedule.right - pH->rcSchedule.left + 1), (pH->rcSchedule.bottom - pH->rcSchedule.top + 1), 0, 0, pH->crMask);
	}
	//copy prepared bitmap onto destination
	BitBlt(hdcPS, 0, 0, rc.right - rc.left, rc.bottom - rc.top, hdc, 0, 0, SRCCOPY);
	//Place the original bitmap back into the bitmap sent here
	BitBlt(hdc, 0, 0, rc.right - rc.left, rc.bottom - rc.top, hdcSave, 0, 0, SRCCOPY);
	//clean up
	DeleteObject(SelectObject(hdcSave, hBmSaveOld));
	DeleteDC(hdcSave);
	DeleteDC(hdc);
}

/*-@@+@@------------------------------------------------------------------
 Procedure: MeasureSkinlessNoteRectangles
 Created  : Sun Sep  9 13:36:39 2007
 Modified : Sun Sep  9 13:36:39 2007

 Synopsys : Measures rectangles for skinless note
 Input    : pH - note's run-time handles
            cx - note's width
            cy - note's height
 Output   : Nothing
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/
static void MeasureSkinlessNoteRectangles(P_NOTE_RTHANDLES pH, int cx, int cy){
	int					width, height;
	BITMAP				bm;

	//get width of grip rectangle
	width = GetSystemMetrics(SM_CXVSCROLL);
	//get caption height
	height = GetSystemMetrics(SM_CYSMCAPTION);
	//set all needed rectangles
	SetRect(&pH->rcGrip, cx - width, cy - width, cx, cy);
	if(pH->hbDelHide){
		SetRect(&pH->rcDH, cx - SMALL_RECT * 2 - 2, 2, cx - 2, SMALL_RECT + 2);
	}
	SetRect(&pH->rcCaption, 0, 0, cx, height);
	SetRect(&pH->rcEdit, 1, pH->rcCaption.bottom + 1, cx - 2, pH->rcGrip.top - 1);
	SetRect(&pH->rcTextCaption, 2, 0, pH->rcDH.left, height);
	SetRect(&pH->rcNotSaved, 2, cy - SMALL_RECT - 2, SMALL_RECT + 2, cy - 2);
	CopyRect(&pH->rcTooltip, &pH->rcTextCaption);
	if(pH->hbCommand){
		int diff;
		GetObject(pH->hbCommand, sizeof(bm), &bm);
		diff = ((pH->rcGrip.bottom - pH->rcGrip.top) - SMALL_RECT) / 2;
		SetRect(&pH->rcCommands, pH->rcNotSaved.right + SMALL_RECT + 4, pH->rcGrip.top + diff, pH->rcNotSaved.right + SMALL_RECT + 4 + bm.bmWidth, pH->rcGrip.top + diff + SMALL_RECT);
	}
}

static DWORD InStreamCallback(DWORD_PTR dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb){
	ReadFile((HANDLE)dwCookie, pbBuff, cb, (PULONG)pcb, NULL);
	return 0;
}

static DWORD OutStreamCallback(DWORD_PTR dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb){
	WriteFile((HANDLE)dwCookie, pbBuff, cb, (PULONG)pcb, NULL);
	return 0;
}

static LRESULT CALLBACK Tbr_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam){

	switch(msg){
	HANDLE_MSG (hwnd, WM_DRAWITEM, Tbr_OnDrawItem);
	HANDLE_MSG (hwnd, WM_MEASUREITEM, Tbr_OnMeasureItem);
	HANDLE_MSG (hwnd, WM_COMMAND, Tbr_OnCommand);
	HANDLE_MSG (hwnd, WM_INITMENUPOPUP, Tbr_OnInitMenuPopup);

	case CHM_NEW_FONT_FACE:
		SetREFontFace(GetParent(hwnd), &g_SelectedFont);
		SetFocus(GetParent(hwnd));
		return FALSE;
	default:
		return DefWindowProcW(hwnd, msg, wParam, lParam);
	}
}

static void Tbr_OnInitMenuPopup(HWND hwnd, HMENU hMenu, UINT item, BOOL fSystemMenu)
{
	PrepareMenuGradientColors(GetSysColor(COLOR_BTNFACE));
}

static void Tbr_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	RemoveBell(GetParent(hwnd));
	if(id > IDI_SIZE_START && id <= IDI_SIZE_START + MAX_FONT_SIZE){
		SetREFontSize(GetParent(hwnd), id - IDI_SIZE_START);
		RemoveSizeMenu();
		SetFocus(GetParent(hwnd));
		//if(id - IDI_SIZE_START != GetCFSize()){
			//SetREFontSize(GetParent(hwnd), id - IDI_SIZE_START);
			//RemoveSizeMenu();
			//SetFocus(GetParent(hwnd));
		//}
	}
	else if(id >= IDI_FORMAT_COLOR_START && id <= IDI_FORMAT_COLOR_END){
		m_CurrColorsChoice = CC_TEXT;
		ColorChanged(GetParent(hwnd), id);
	}
	else if(id >= IDI_FORMAT_HIGHLIGHT_START && id <= IDI_FORMAT_HIGHLIGHT_END){
		m_CurrColorsChoice = CC_HIGHLIGHT;
		ColorChanged(GetParent(hwnd), id);
	}
	else{
		switch(id){
		case CMD_BOLD + 5000:
			SetREFontDecoration(GetParent(hwnd), FD_BOLD);
			SetFocus(GetParent(hwnd));
			break;
		case CMD_ITALIC + 5000:
			SetREFontDecoration(GetParent(hwnd), FD_ITALIC);
			SetFocus(GetParent(hwnd));
			break;
		case CMD_UNDERLINE + 5000:
			SetREFontDecoration(GetParent(hwnd), FD_UNDERLINE);
			SetFocus(GetParent(hwnd));
			break;
		case CMD_STRIKETHROUGH + 5000:
			SetREFontDecoration(GetParent(hwnd), FD_STRIKETHROUGH);
			SetFocus(GetParent(hwnd));
			break;
		case CMD_A_LEFT + 5000:
			SetREAlignment(GetParent(hwnd), PFA_LEFT);
			SetFocus(GetParent(hwnd));
			break;
		case CMD_A_CENTER + 5000:
			SetREAlignment(GetParent(hwnd), PFA_CENTER);
			SetFocus(GetParent(hwnd));
			break;
		case CMD_A_RIGHT + 5000:
			SetREAlignment(GetParent(hwnd), PFA_RIGHT);
			SetFocus(GetParent(hwnd));
			break;
		case CMD_BULLETS + 5000:
			SetREBullets(GetParent(hwnd));
			SetFocus(GetParent(hwnd));
			break;
		}
	}
}

static void Tbr_OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT * lpDrawItem)
{
	DrawMItem(lpDrawItem, g_hBmpMenuNormal, g_hBmpMenuGray, CLR_MASK);
}

static void Tbr_OnMeasureItem(HWND hwnd, MEASUREITEMSTRUCT * lpMeasureItem)
{
	MeasureMItem(g_hMenuFont, lpMeasureItem);
}

static LRESULT CALLBACK Command_WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam){

	switch(msg){
	HANDLE_MSG (hwnd, WM_PAINT, Command_OnPaint);
	HANDLE_MSG (hwnd, WM_LBUTTONUP, Command_OnLButtonUp);

	case WM_RBUTTONUP:
	case WM_MBUTTONUP:{
		RecreateCommandTooltip(hwnd);
		return FALSE;
	}
	default:
		return DefWindowProcW(hwnd, msg, wParam, lParam);
	}
}

static void Command_OnLButtonUp(HWND hwnd, int x, int y, UINT keyFlags)
{
	POINT		pt;
	int			id;
	BOOL		delete = FALSE;

	GetCursorPos(&pt);
	id = GetDlgCtrlID(hwnd);
	switch(id){
	case CMD_FONT + 5000:{
		CBCREATION cbr;
		cbr.hParent = g_hMain;
		cbr.hButton = hwnd;
		cbr.hEdit = (HWND)GetPropW(cbr.hParent, PH_EDIT);
		SetCurrentFontFace((HWND)GetPropW(GetParent(GetParent(hwnd)), PH_EDIT));
		CreateComboDialog(&cbr, (HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE));
		break;
	}
	case CMD_COLOR + 5000:
	case CMD_HIGHLIGHT + 5000:{
		if(id == CMD_COLOR + 5000){
			m_CurrColorsChoice = CC_TEXT;
			TrackPopupMenu(m_hMenuTColor, TPM_LEFTALIGN | TPM_LEFTBUTTON, pt.x, pt.y, 0, GetParent(hwnd), NULL);
		}
		else{
			m_CurrColorsChoice = CC_HIGHLIGHT;
			TrackPopupMenu(m_hMenuTHighlight, TPM_LEFTALIGN | TPM_LEFTBUTTON, pt.x, pt.y, 0, GetParent(hwnd), NULL);
		}
		break;
	}
	case CMD_SIZE + 5000:{
		MENUITEMINFOW mi;
		int size = GetCurrentFontSize((HWND)GetPropW(GetParent(GetParent(hwnd)), PH_EDIT));
		SetCFSize(size);
		CreateSizeMenu(&g_hSizePopUp, TRUE);
		ZeroMemory(&mi, sizeof(mi));
		mi.cbSize = sizeof(mi);
		mi.fMask = MIIM_STATE;
		mi.fState = MFS_HILITE;
		SetMenuItemInfoW(g_hSizePopUp, IDI_SIZE_START + size, FALSE, &mi);
		TrackPopupMenu(g_hSizePopUp, TPM_LEFTALIGN | TPM_LEFTBUTTON, pt.x, pt.y, 0, GetParent(hwnd), NULL);
		break;
	}
	case CMD_BOLD + 5000:
	case CMD_ITALIC + 5000:
	case CMD_UNDERLINE + 5000:
	case CMD_STRIKETHROUGH + 5000:
	case CMD_A_LEFT + 5000:
	case CMD_A_CENTER + 5000:
	case CMD_A_RIGHT + 5000:
	case CMD_BULLETS + 5000:
		//forward processing to toolbar window
		SendMessageW(GetParent(hwnd), WM_COMMAND, MAKEWPARAM(id, 0), (LPARAM)hwnd);
		break;
	case IDM_DEL:
		delete = TRUE;
		SendMessageW(GetParent(GetParent(hwnd)), WM_COMMAND, MAKEWPARAM(IDM_DEL, 0), 0);
		break;
	case IDM_HIDE:
		SendMessageW(GetParent(GetParent(hwnd)), WM_COMMAND, MAKEWPARAM(IDM_HIDE, 0), 0);
		break;
	}
	if(!delete)
		RecreateCommandTooltip(hwnd);
}

static void Command_OnPaint(HWND hwnd)
{
	PAINTSTRUCT			ps;
	P_NOTE_RTHANDLES	pH;
	P_NOTE_DATA			pD;
	RECT				rc;
	int					id;

	BeginPaint(hwnd, &ps);
	id = GetDlgCtrlID(hwnd);
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(GetParent(GetParent(hwnd)), OFFSET_HANDLES);
	pD = (P_NOTE_DATA)GetWindowLongW(GetParent(GetParent(hwnd)), OFFSET_DATA);
	if(pH){
		//check for docked note
		if(DockType(pD->dockData) != DOCK_NONE){
			if(g_DockSettings.fCustSkin){
				if(g_RTHandles.hbSkin){
					pH = &g_DRTHandles;
				}
			}
		}
		GetClientRect(hwnd, &rc);
		switch(id){
		case IDM_DEL:
		case IDM_HIDE:
			if(pH->hbDelHide){
				DrawTransparentBitmap(pH->hbDelHide, ps.hdc, 0, 0, (rc.right - rc.left), (rc.bottom - rc.top), (id - IDM_HIDE) * (rc.right - rc.left), 0, pH->crMask);
			}
			break;
		default:
			if(pH->hbCommand){
				if(!pH->vertTbr)
					DrawTransparentBitmap(pH->hbCommand, ps.hdc, 0, 0, (rc.right - rc.left), (rc.bottom - rc.top), (id - 5000) * (rc.right - rc.left), 0, pH->crMask);
				else
					DrawTransparentBitmap(pH->hbCommand, ps.hdc, 0, 0, (rc.right - rc.left), (rc.bottom - rc.top), 0, (id - 5000) * (rc.bottom - rc.top), pH->crMask);
			}
		}
	}
	EndPaint(hwnd, &ps);
}

/*-@@+@@------------------------------------------------------------------
 Procedure: SetCommandsTooltips
 Created  : Tue Jan  8 12:06:50 2008
 Modified : Tue Jan  8 12:06:50 2008

 Synopsys : Creates toolbar tooltips
 Input    : hwnd - note window handle
 Output   : 
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/
static void SetCommandsTooltips(HWND hwnd){

	wchar_t 		szBuffer[256], szFile[MAX_PATH], szKey[12];
	P_NCMD_BAR		pC, pDH;
	TOOLINFOW		ti;
	HINSTANCE		hInstance;

	hInstance = (HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE);
	
	ZeroMemory(&ti, sizeof(ti));
	ti.cbSize = sizeof(ti);
	ti.uFlags = TTF_IDISHWND | TTF_SUBCLASS;
	wcscpy(szFile, g_NotePaths.LangDir);
	wcscat(szFile, g_NotePaths.LangFile);
	pC = (P_NCMD_BAR)GetWindowLongW(hwnd, OFFSET_COMMANDS);
	for(int i = CMD_FONT; i < CMD_TBR_BUTTONS; i++){
		_itow(5000 + i, szKey, 10);
		GetPrivateProfileStringW(S_COMMANDS, szKey, NULL, szBuffer, 256, szFile);
		ti.uId = (UINT)pC->hCmd;
		ti.hwnd = pC->hCmd;
		if(pC->hTtp != NULL){
			SendMessageW(pC->hTtp, TTM_DELTOOLW, 0, (LPARAM)&ti);
			DestroyWindow(pC->hTtp);
		}
		ti.lpszText = szBuffer;
		pC->hTtp = CreateWindowExW(WS_EX_TOPMOST, __T("Tooltips_class32"), NULL, TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
		SendMessageW(pC->hTtp, TTM_ADDTOOLW, 0, (LPARAM)&ti);
		pC++;
	}
	pDH = (P_NCMD_BAR)GetWindowLongW(hwnd, OFFSET_DH);
	for(int i = 0; i < CMD_SYS_BUTTONS; i++){
		_itow(IDM_HIDE + i, szKey, 10);
		GetPrivateProfileStringW(S_MENU, szKey, NULL, szBuffer, 256, szFile);
		ti.uId = (UINT)pDH->hCmd;
		if(pDH->hTtp != NULL){
			SendMessageW(pDH->hTtp, TTM_DELTOOLW, 0, (LPARAM)&ti);
			DestroyWindow(pDH->hTtp);
		}
		ti.lpszText = szBuffer;
		pDH->hTtp = CreateWindowExW(WS_EX_TOPMOST, __T("Tooltips_class32"), NULL, TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
		SendMessageW(pDH->hTtp, TTM_ADDTOOLW, 0, (LPARAM)&ti);
		pDH++;
	}
}

/*-@@+@@------------------------------------------------------------------
 Procedure: RecreateCommandTooltip
 Created  : Tue Jan  8 12:05:48 2008
 Modified : Tue Jan  8 12:05:48 2008

 Synopsys : Recreates toolbar tooltips
 Input    : hwnd - note window handle
 Output   : 
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/
static void RecreateCommandTooltip(HWND hwnd){

	wchar_t 		szBuffer[256], szFile[MAX_PATH], szKey[12], szSection[64];
	P_NCMD_BAR		pCmd;
	TOOLINFOW		ti;
	HINSTANCE		hInstance;
	int				id;

	hInstance = (HINSTANCE)GetWindowLongW(hwnd, GWL_HINSTANCE);
	ZeroMemory(&ti, sizeof(ti));
	ti.cbSize = sizeof(ti);
	ti.uFlags = TTF_IDISHWND | TTF_SUBCLASS;

	wcscpy(szFile, g_NotePaths.LangDir);
	wcscat(szFile, g_NotePaths.LangFile);
	id = GetDlgCtrlID(hwnd);
	switch(id){
	case IDM_HIDE:
	case IDM_DEL:
		pCmd = (P_NCMD_BAR)GetWindowLongW(GetParent(GetParent(hwnd)), OFFSET_DH);
		pCmd += (id - IDM_HIDE);
		wcscpy(szSection, S_MENU);
		break;
	default:
		pCmd = (P_NCMD_BAR)GetWindowLongW(GetParent(GetParent(hwnd)), OFFSET_COMMANDS);
		pCmd += (id - 5000);
		wcscpy(szSection, S_COMMANDS);
	}
	_itow(id, szKey, 10);
	GetPrivateProfileStringW(szSection, szKey, NULL, szBuffer, 256, szFile);
	ti.uId = (UINT)hwnd;
	ti.hwnd = hwnd;
	if(pCmd->hTtp != NULL){
		SendMessageW(pCmd->hTtp, TTM_DELTOOLW, 0, (LPARAM)&ti);
		DestroyWindow(pCmd->hTtp);
	}
	ti.lpszText = szBuffer;
	pCmd->hTtp = CreateWindowExW(WS_EX_TOPMOST, __T("Tooltips_class32"), NULL, TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
	SendMessageW(pCmd->hTtp, TTM_ADDTOOLW, 0, (LPARAM)&ti);
}

static void DeleteRTFile(wchar_t * id){
	wchar_t		szPath[MAX_PATH];

	if(NotesDirExists(g_NotePaths.DataDir)){
		wcscpy(szPath, g_NotePaths.DataDir);
		wcscat(szPath, id);
		wcscat(szPath, NOTE_EXTENTION);
		DeleteFileW(szPath);
		DeleteAutoSavedFile(id);
	}
}

static void DeleteAutoSavedFile(wchar_t * id){
	wchar_t		szPath[MAX_PATH];

	if(NotesDirExists(g_NotePaths.DataDir)){
		wcscpy(szPath, g_NotePaths.DataDir);
		wcscat(szPath, L"~");
		wcscat(szPath, id);
		wcscat(szPath, L".no~");
		DeleteFileW(szPath);
	}
}

/*-@@+@@------------------------------------------------------------------
 Procedure: GetTemporarySaveName
 Created  : Tue Jan  8 12:04:33 2008
 Modified : Tue Jan  8 12:04:33 2008

 Synopsys : Creates temporary name when note is saving for the first time 
            (the first nMax characters from note text)
 Input    : hNote - note window handle
            lpName - buffer for note name
            nMax - max count of characters to get from note text
 Output   : Temporary name length
 Errors   : 
 ------------------------------------------------------------------@@-@@-*/
static int GetTemporarySaveName(HWND hNote, wchar_t * lpName, int nMax){

	HWND		hEdit;
	wchar_t		szBuffer[nMax];
	int			count = 0;

	hEdit = (HWND)GetPropW(hNote, PH_EDIT);
	count = GetWindowTextLengthW(hEdit);
	if(count > nMax)
		count = GetWindowTextW(hEdit, szBuffer, nMax);
	else
		count = GetWindowTextW(hEdit, szBuffer, count + 1);
	if(count > 0){
		wcscpy(lpName, szBuffer);
		while(*lpName++){
			if(*lpName == '\r' || *lpName == '\n'){
				*lpName = '\0';
				break;
			}
		}
	}
	return count;
}

static void SetCurrentFontFace(HWND hEdit){

	CHARFORMAT2W		chf;

	ZeroMemory(&chf, sizeof(chf));
	chf.cbSize = sizeof(chf);
	chf.dwMask = CFM_FACE;
	RichEdit_GetCharFormat(hEdit, SCF_SELECTION, &chf);
	wcscpy(g_CurrentFontFace, chf.szFaceName);

}

static int GetCurrentFontSize(HWND hEdit){

	CHARFORMAT2W		chf;
	int					size;
	HDC					hdc;

	ZeroMemory(&chf, sizeof(chf));
	chf.cbSize = sizeof(chf);
	chf.dwMask = CFM_SIZE;
	RichEdit_GetCharFormat(hEdit, SCF_SELECTION, &chf);
	hdc = GetDC(hEdit);
	size = -(chf.yHeight * GetDeviceCaps(hdc, LOGPIXELSY)) / (20 * 72);
	size = -MulDiv(size, 72, GetDeviceCaps(hdc, LOGPIXELSY));
	ReleaseDC(hEdit, hdc);
	return size;
}

static void InitAppearancePart(HWND hwnd){
	//fill skins
	if(m_TNHandles.hbSkin){
		FillSkins(hwnd, m_TNAppearance.szSkin, TRUE, FALSE);
	}
	else{
		FillSkins(hwnd, m_TNAppearance.szSkin, FALSE, TRUE);
		RedrawWindow(GetDlgItem(hwnd, IDC_ST_BMP), NULL, NULL, RDW_INVALIDATE);
	}
}

static void InitSchedulePart(HWND hwnd){

	P_SCHEDULE_TYPE		pS;

	//schedule part
	pS = (P_SCHEDULE_TYPE)GetWindowLongW((HWND)GetWindowLongW(GetParent(hwnd), GWL_USERDATA), OFFSET_SCHEDULE);
	SendDlgItemMessageW(hwnd, IDC_CBO_NOTE_SCHEDULE, CB_SETCURSEL, pS->scType, 0);
	SendMessageW(m_Dlgs.hSchedule, WM_COMMAND, MAKEWPARAM(IDC_CBO_NOTE_SCHEDULE, CBN_SELENDOK), (LPARAM)GetDlgItem(m_Dlgs.hSchedule, IDC_CBO_NOTE_SCHEDULE));
	if(pS->scType == SCH_ED){
		SendDlgItemMessageW(m_Dns.hEveryDay, IDC_DTP_EV_DAY, DTM_SETSYSTEMTIME, GDT_VALID, (LPARAM)&pS->scDate); 
	}
	else if(pS->scType == SCH_ONCE){
		SendDlgItemMessageW(m_Dns.hOnce, IDC_DTP_ONCE, DTM_SETSYSTEMTIME, GDT_VALID, (LPARAM)&pS->scDate); 
	}
	else if(pS->scType == SCH_REP){
		SetPesterPart(&pS->scDate);
	}
	else if(pS->scType == SCH_PER_WD){
		for(int i = 0; i <= 6; i++){
			if(IsBitOn(pS->scDate.wDayOfWeek, i + 1)){
				CheckDlgButton(m_Dns.hWeekly, IDC_CHK_WD_0 + i, BST_CHECKED);
			}
			else{
				CheckDlgButton(m_Dns.hWeekly, IDC_CHK_WD_0 + i, BST_UNCHECKED);
			}
		}
		SendDlgItemMessageW(m_Dns.hWeekly, IDC_DTP_TIME_EVERY_WD, DTM_SETSYSTEMTIME, GDT_VALID, (LPARAM)&pS->scDate); 
	}
	else if(pS->scType == SCH_AFTER){
		SendDlgItemMessageW(m_Dns.hAfter, IDC_DTP_AFTER, DTM_SETSYSTEMTIME, GDT_VALID, (LPARAM)&pS->scStart); 
		SetDlgItemInt(m_Dns.hAfter, IDC_EDT_YEARS, pS->scDate.wYear, FALSE);
		SetDlgItemInt(m_Dns.hAfter, IDC_EDT_MONTHS, pS->scDate.wMonth, FALSE);
		SetDlgItemInt(m_Dns.hAfter, IDC_EDT_WEEKS, pS->scDate.wDayOfWeek, FALSE);
		SetDlgItemInt(m_Dns.hAfter, IDC_EDT_DAYS, pS->scDate.wDay, FALSE);
		SetDlgItemInt(m_Dns.hAfter, IDC_EDT_HOURS, pS->scDate.wHour, FALSE);
		SetDlgItemInt(m_Dns.hAfter, IDC_EDT_MINS, pS->scDate.wMinute, FALSE);
		SetDlgItemInt(m_Dns.hAfter, IDC_EDT_SECS, pS->scDate.wSecond, FALSE);
	}
	//'repeat' dialog
	SendDlgItemMessageW(m_Dns.hRepeat, IDC_UPD_DAYS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 31, (short) 0));
	SendDlgItemMessageW(m_Dns.hRepeat, IDC_UPD_HOURS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 24, (short) 0));
	SendDlgItemMessageW(m_Dns.hRepeat, IDC_UPD_MINS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 60, (short) 0));
	SendDlgItemMessageW(m_Dns.hRepeat, IDC_UPD_SECS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 60, (short) 0));
	//'after' dialog
	SendDlgItemMessageW(m_Dns.hAfter, IDC_UPD_YEARS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 99, (short) 0));
	SendDlgItemMessageW(m_Dns.hAfter, IDC_UPD_MONTHS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 12, (short) 0));
	SendDlgItemMessageW(m_Dns.hAfter, IDC_UPD_WEEKS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 48, (short) 0));
	SendDlgItemMessageW(m_Dns.hAfter, IDC_UPD_DAYS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 31, (short) 0));
	SendDlgItemMessageW(m_Dns.hAfter, IDC_UPD_HOURS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 24, (short) 0));
	SendDlgItemMessageW(m_Dns.hAfter, IDC_UPD_MINS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 60, (short) 0));
	SendDlgItemMessageW(m_Dns.hAfter, IDC_UPD_SECS, UDM_SETRANGE, 0, (LPARAM) MAKELONG((short) 60, (short) 0));
}

static BOOL CALLBACK NApp_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	HANDLE_MSG (hwnd, WM_DRAWITEM, NApp_OnDrawItem);
	HANDLE_MSG (hwnd, WM_COMMAND, NApp_OnCommand);
	HANDLE_MSG (hwnd, WM_INITDIALOG, NApp_OnInitDialog);

	// HANDLE_MSG (hwnd, WM_CTLCOLORSTATIC, NApp_OnCtlColorStatic);
	case CHM_DRAW_SN_PVW:
		DrawSimpleNotePreview((HWND)wParam, (DRAWITEMSTRUCT *)lParam, &m_rWindow, &m_rCaption, &m_TNAppearance, &m_TNHandles);
		return TRUE;
	default: return FALSE;
	}
}

static BOOL NApp_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	wchar_t		szFile[MAX_PATH];
	RECT		rc1, rc2;

	GetWindowRect(GetDlgItem(hwnd, IDC_ST_BMP), &rc2);
	MapWindowPoints(HWND_DESKTOP, hwnd, (LPPOINT)&rc2, 2);
	if(!g_RTHandles.hbSkin){
		GetClientRect(hwnd, &rc1);
		ShowWindow(GetDlgItem(hwnd, IDC_LST_SKIN), SW_HIDE);
		wcscpy(szFile, g_NotePaths.LangDir);
		wcscat(szFile, g_NotePaths.LangFile);
		SetDlgCtlText(hwnd, IDC_CMD_SKNLESS_BCOLOR, szFile, __T("Background color"));
		SetDlgCtlText(hwnd, IDC_CMD_SKNLESS_CFONT, szFile, __T("Caption font"));
		ShowWindow(GetDlgItem(hwnd, IDC_CMD_SKNLESS_BCOLOR), SW_SHOW);
		ShowWindow(GetDlgItem(hwnd, IDC_CMD_SKNLESS_CFONT), SW_SHOW);
		MoveWindow(GetDlgItem(hwnd, IDC_ST_BMP), ((rc1.right - rc1.left) - (rc2.right - rc2.left)) / 2, rc2.top, rc2.right - rc2.left, rc2.bottom - rc2.top, TRUE);
	}
	else{
		GetWindowRect(GetDlgItem(hwnd, IDC_PLACEHOLDER), &rc1);
		MapWindowPoints(HWND_DESKTOP, hwnd, (LPPOINT)&rc1, 2);
		MoveWindow(GetDlgItem(hwnd, IDC_ST_BMP), rc1.left, rc2.top, rc2.right - rc2.left, rc2.bottom - rc2.top, TRUE);
	}
	return TRUE;
}

static void NApp_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	HWND				hNote = (HWND)GetWindowLongW(GetParent(hwnd), GWL_USERDATA);
	P_NOTE_FLAGS 		pF = (P_NOTE_FLAGS)GetWindowLongW(hNote, OFFSET_FLAGS);
	P_NOTE_RTHANDLES	pH;
	// POINT				pt;

	switch(id){
		case IDC_CMD_SKNLESS_BCOLOR:
			if(codeNotify == BN_CLICKED){
				if(ChangeSkinlessColor(hwnd, &m_TNAppearance.crWindow, IDC_ST_BMP)){
					pF->maskChanged |= F_B_COLOR;
				}
			}
			break;
		case IDC_CMD_SKNLESS_CFONT:{
				int res = ChangeSkinlessCaptionFont(hwnd, &m_TNAppearance.lfCaption, &m_TNAppearance.crCaption, &m_TNHandles, IDC_ST_BMP);
				if((res & F_C_FONT) == F_C_FONT)
					pF->maskChanged |= F_C_FONT;
				if((res & F_C_COLOR) == F_C_COLOR)
					pF->maskChanged |= F_C_COLOR;
			}
			break;
		// case IDC_ST_BMP:
			// if(codeNotify == STN_DBLCLK){
				// if(!m_TNHandles.hbSkin){
					// GetCursorPos(&pt);
					// MapWindowPoints(HWND_DESKTOP, hwndCtl, &pt, 1);
					// if(PtInRect(&m_rWindow, pt)){
						// if(ChangeSkinlessColor(hwnd, &m_TNAppearance)){
							// pF->maskChanged |= F_B_COLOR;
						// }
					// }
					// else if(PtInRect(&m_rCaption, pt)){
						// int res = ChangeSkinlessCaptionFont(hwnd, &m_TNAppearance, &m_TNHandles);
						// if((res & F_C_FONT) == F_C_FONT)
							// pF->maskChanged |= F_C_FONT;
						// if((res & F_C_COLOR) == F_C_COLOR)
							// pF->maskChanged |= F_C_COLOR;
					// }
				// }
			// }
			// break;
		case IDC_LST_SKIN:	//show skin preview
			if(codeNotify == LBN_SELCHANGE){
				pH = (P_NOTE_RTHANDLES)GetWindowLongW(hNote, OFFSET_HANDLES);
				ChangeSkinPreview(hwnd, IDC_LST_SKIN, IDC_ST_BMP, &m_TNHandles, m_TNAppearance.szSkin, (pH->hbSkin == m_TNHandles.hbSkin) ? FALSE : TRUE);
				pF->maskChanged |= F_SKIN;
			}
			break;
	}
}

static void NApp_OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT * lpDrawItem)
{
	if(lpDrawItem->CtlType == ODT_STATIC && lpDrawItem->CtlID == IDC_ST_BMP)
		//draw skin preview
		DrawSkinPreview(GetDlgItem(hwnd, IDC_ST_BMP), &m_TNHandles, &m_TNAppearance, m_TNHandles.crMask, lpDrawItem);
}

static BOOL CALLBACK NSched_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	HANDLE_MSG (hwnd, WM_COMMAND, NSched_OnCommand);
	HANDLE_MSG (hwnd, WM_DRAWITEM, NSched_OnDrawItem);

	default: return FALSE;
	}
}

static void NSched_OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT * lpDrawItem)
{
	if(lpDrawItem->CtlType == ODT_COMBOBOX)
		DrawComboItem(lpDrawItem);
}

static void NSched_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch(id){
		case IDC_CBO_NOTE_SCHEDULE:
			if(codeNotify == CBN_SELENDOK){
				HWND		*pDlg = (HWND *)&m_Dns;
				int			count = SendDlgItemMessageW(hwnd, IDC_CBO_NOTE_SCHEDULE, CB_GETCOUNT, 0, 0);
				int			selected = SendDlgItemMessageW(hwnd, IDC_CBO_NOTE_SCHEDULE, CB_GETCURSEL, 0, 0);

				for(int i = 1; i < count; i++){
					if(i == selected)
						ShowWindow(*pDlg, SW_SHOW);
					else
						ShowWindow(*pDlg, SW_HIDE);
					pDlg++;
				}
			}
			break;
	}
}

static void RemoveBell(HWND hwnd){
	HWND				hBell;

	if(g_NoteSettings.visualAlert){
		//destoy bell window, if any
		hBell = (HWND)GetPropW(hwnd, PH_BELL);
		if(hBell && IsWindow(hBell)){
			//destroy bell window only if it exists
			DestroyWindow(hBell);
		}
	}
}

static void SaveNewCFont(HWND hwnd, LPLOGFONTW lplf){
	NOTE_APPEARANCE	app;
	P_NOTE_FLAGS		pF;

	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	GetPrivateProfileStructW(pF->id, S_APPEARANCE, &app, sizeof(NOTE_APPEARANCE), g_NotePaths.DataFile);
	memcpy(&app.lfCaption, lplf, sizeof(LOGFONTW));
	app.fFontSet = TRUE;
	app.nPrivate |= F_C_FONT;
	WritePrivateProfileStructW(pF->id, S_APPEARANCE, &app, sizeof(NOTE_APPEARANCE), g_NotePaths.DataFile);
}

static void SaveNewCColor(HWND hwnd, COLORREF crCaption){
	NOTE_APPEARANCE	app;
	P_NOTE_FLAGS		pF;

	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	GetPrivateProfileStructW(pF->id, S_APPEARANCE, &app, sizeof(NOTE_APPEARANCE), g_NotePaths.DataFile);
	app.crCaption = crCaption;
	app.fFontSet = TRUE;
	app.nPrivate |= F_C_COLOR;
	WritePrivateProfileStructW(pF->id, S_APPEARANCE, &app, sizeof(NOTE_APPEARANCE), g_NotePaths.DataFile);
}

static void SaveNewBColor(HWND hwnd, COLORREF crWindow){
	NOTE_APPEARANCE	app;
	P_NOTE_FLAGS		pF;

	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	GetPrivateProfileStructW(pF->id, S_APPEARANCE, &app, sizeof(NOTE_APPEARANCE), g_NotePaths.DataFile);
	app.crWindow = crWindow;
	app.nPrivate |= F_B_COLOR;
	WritePrivateProfileStructW(pF->id, S_APPEARANCE, &app, sizeof(NOTE_APPEARANCE), g_NotePaths.DataFile);
}

static void SaveVisibility(HWND hwnd, BOOL visible){
	P_NOTE_DATA			pD;
	P_NOTE_FLAGS		pF;
	NOTE_DATA			data;

	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	if(pD){
		pD->visible = visible;
		//save only the visibility data
		if(pF->fromDB){
			//version 4.0
			if(GetPrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile)){
				data.visible = visible;
				WritePrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile);
			}
			else{
				//version 3.5
				if(GetPrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA) - sizeof(RECT) - sizeof(BOOL), g_NotePaths.DataFile)){
					data.visible = visible;
					WritePrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile);
				}
			}
		}		
	}
}

static void WriteNotePlacement(HWND hwnd, BOOL fSave){
	P_NOTE_DATA			pD;
	P_NOTE_FLAGS		pF;
	P_NOTE_RTHANDLES	pH;
	RECT				rcNote;
	NOTE_DATA			data;
	NOTE_REL_POSITION	nrp;
	SIZE				sz;

	GetWindowRect(hwnd, &rcNote);
	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	pH = (P_NOTE_RTHANDLES)GetWindowLongW(hwnd, OFFSET_HANDLES);
	if(pD){
		if(DockType(pD->dockData) == DOCK_NONE){
			//save current absolute position
			if(pH->rolled)
				rcNote.bottom = rcNote.top + (pH->rcSize.bottom - pH->rcSize.top);
			CopyRect(&pD->rcp, &rcNote);
			if(fSave){
				GetPrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile);
				CopyRect(&data.rcp, &rcNote);
				//save rolled status
				if(pH->rolled != data.rolled){
					data.rolled = pH->rolled;
				}
				WritePrivateProfileStructW(pF->id, S_DATA, &data, sizeof(NOTE_DATA), g_NotePaths.DataFile);
				//save current relational position
				sz = GetScreenMetrics();
				nrp.left = (double)rcNote.left / (double)sz.cx;
				nrp.top = (double)rcNote.top / (double)sz.cy;
				nrp.width = rcNote.right - rcNote.left;
				nrp.height = rcNote.bottom - rcNote.top;
				WritePrivateProfileStructW(pF->id, IK_RELPOSITION, &nrp, sizeof(nrp), g_NotePaths.DataFile);
			}
		}
	}
}

static SIZE GetScreenMetrics(void){
	SIZE		sz;
	RECT		rcDesktop;

	if(GetSystemMetrics(SM_CMONITORS) == 1){
		GetClientRect(GetDesktopWindow(), &rcDesktop);
		sz.cx = rcDesktop.right - rcDesktop.left;
		sz.cy = rcDesktop.bottom - rcDesktop.top;
	}
	else{
		sz.cx = GetSystemMetrics(SM_CXVIRTUALSCREEN);
		sz.cy = GetSystemMetrics(SM_CYVIRTUALSCREEN);
	}
	return sz;
}

static void AddToFavorites(HWND hwnd){
	P_NOTE_DATA		pD;
	P_NOTE_FLAGS	pF;
	wchar_t			szBuffer[256];

	pD = (P_NOTE_DATA)GetWindowLongW(hwnd, OFFSET_DATA);
	pF = (P_NOTE_FLAGS)GetWindowLongW(hwnd, OFFSET_FLAGS);
	GetPrivateProfileStringW(S_FAVORITES, pF->id, NULL, szBuffer, 256, g_NotePaths.INIFile);
	if(wcslen(szBuffer) == 0)
		WritePrivateProfileStringW(S_FAVORITES, pF->id, pD->szName, g_NotePaths.INIFile);
	else
		WritePrivateProfileStringW(S_FAVORITES, pF->id, NULL, g_NotePaths.INIFile);
}

PMItem NoteMenu(void){
	return m_NMenus;
}

PMItem EditMenu(void){
	return m_EMenus;
}

int NoteMenuCount(void){
	return NELEMS(m_NMenus);
}

int EditMenuCount(void){
	return NELEMS(m_EMenus);
}

static BOOL CALLBACK Dns_DlgProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	HANDLE_MSG (hwnd, WM_COMMAND, Dns_OnCommand);
	HANDLE_MSG (hwnd, WM_INITDIALOG, Dns_OnInitDialog);

	default: return FALSE;
	}
}

static BOOL Dns_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	RECT		rc1, rc2, rc3;

	GetClientRect(GetParent(hwnd), &rc1);
	GetClientRect(hwnd, &rc2);
	GetWindowRect(GetDlgItem(GetParent(hwnd), IDC_CBO_NOTE_SCHEDULE), &rc3);
	MapWindowPoints(HWND_DESKTOP, GetParent(hwnd), (LPPOINT)&rc3, 2);

	MoveWindow(hwnd, ((rc1.right - rc1.left) - (rc2.right - rc2.left)) / 2, rc3.top + (rc3.bottom - rc3.top) * 2, rc2.right - rc2.left, rc2.bottom - rc2.top, TRUE);
	return FALSE;
}

static void Dns_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch(id){
		case IDC_EDT_DAYS:
		case IDC_EDT_HOURS:
		case IDC_EDT_MINS:
		case IDC_EDT_SECS:
			if(hwnd == m_Dns.hRepeat){
				if(codeNotify == EN_UPDATE){
					BOOL	translated;
					//prevent empty control
					if(GetDlgItemInt(hwnd, id, &translated, FALSE) == 0 && translated == FALSE)
						SetDlgItemInt(hwnd, id, 0, FALSE);
				}
			}
			break;
	}
}

static COLORREF MakeRandomColor(void){
	BYTE	r, g, b;

	rseed((unsigned)time(NULL));
	r = rnd(255);
	g = rnd(255);
	b = rnd(255);
	return RGB(r, g, b);
}

static int rnd(int base){
	__asm{
		mov eax, rnd_seed
		test eax, 80000000h
    	jz  _next
    	add eax, 7fffffffh
		_next:
		xor edx, edx
	    mov ecx, 127773
	    div ecx
	    mov ecx, eax
	    mov eax, 16807
	    mul edx
	    mov edx, ecx
	    mov ecx, eax
	    mov eax, 2836
	    mul edx
	    sub ecx, eax
	    xor edx, edx
	    mov eax, ecx
	    mov rnd_seed, ecx
	    div base

	    mov eax, edx
	}
}

static void rseed(int seed){
	rnd_seed = 12345678;

	rnd_seed = seed;
}
